@@ -1,11 +1,14 @@
 """Contains the CLI."""
 
+from itertools import chain
 import sys
 import json
 import logging
 import time
+from logging import LogRecord
+from typing import Callable, Tuple, NoReturn, Optional, List, cast
 
-import oyaml as yaml
+import yaml
 
 import click
 
@@ -15,6 +18,8 @@ from io import StringIO
 
 # To enable colour cross platform
 import colorama
+from tqdm import tqdm
+from sqlfluff.cli.autocomplete import shell_completion_enabled, dialect_shell_complete
 
 from sqlfluff.cli.formatters import (
     format_rules,
@@ -33,23 +38,51 @@ from sqlfluff.core import (
     Linter,
     FluffConfig,
     SQLLintError,
+    SQLTemplaterError,
+    SQLFluffUserError,
     dialect_selector,
     dialect_readout,
     TimingSummary,
 )
+from sqlfluff.core.config import progress_bar_configuration
+
+from sqlfluff.core.enums import FormatType, Color
+from sqlfluff.core.linter import ParsedString
+from sqlfluff.core.plugin.host import get_plugin_manager
+from sqlfluff.core.linter import LintingResult
 
 
 class RedWarningsFilter(logging.Filter):
     """This filter makes all warnings or above red."""
 
-    def filter(self, record):
+    def filter(self, record: logging.LogRecord) -> bool:
         """Filter any warnings (or above) to turn them red."""
         if record.levelno >= logging.WARNING:
-            record.msg = colorize(record.msg, "red") + " "
+            record.msg = f"{colorize(record.msg, Color.red)} "
         return True
 
 
-def set_logging_level(verbosity, logger=None, stderr_output=False):
+class StreamHandlerTqdm(logging.StreamHandler):
+    """Modified StreamHandler which takes care of writing within `tqdm` context.
+
+    It uses `tqdm` write which takes care of conflicting prints with progressbar.
+    Without it, there were left artifacts in DEBUG mode (not sure about another ones,
+    but probably would happen somewhere).
+    """
+
+    def emit(self, record: LogRecord) -> None:
+        """Behaves like original one except uses `tqdm` to write."""
+        try:
+            msg = self.format(record)
+            tqdm.write(msg, file=self.stream)
+            self.flush()
+        except Exception:  # pragma: no cover
+            self.handleError(record)
+
+
+def set_logging_level(
+    verbosity: int, logger: Optional[logging.Logger] = None, stderr_output: bool = False
+) -> None:
     """Set up logging for the CLI.
 
     We either set up global logging based on the verbosity
@@ -69,8 +102,9 @@ def set_logging_level(verbosity, logger=None, stderr_output=False):
     # Enable colorama
     colorama.init()
 
-    # Set up the log handler to log to stdout
-    handler = logging.StreamHandler(stream=sys.stderr if stderr_output else sys.stdout)
+    # Set up the log handler which is able to print messages without overlapping
+    # with progressbars.
+    handler = StreamHandlerTqdm(stream=sys.stderr if stderr_output else sys.stdout)
     # NB: the unicode character at the beginning is to squash any badly
     # tamed ANSI colour statements, and return us to normality.
     handler.setFormatter(logging.Formatter("\u001b[0m%(levelname)-10s %(message)s"))
@@ -101,7 +135,7 @@ def set_logging_level(verbosity, logger=None, stderr_output=False):
         parser_logger.setLevel(logging.DEBUG)
 
 
-def common_options(f):
+def common_options(f: Callable) -> Callable:
     """Add common options to commands via a decorator.
 
     These are applied to all of the cli commands.
@@ -111,36 +145,61 @@ def common_options(f):
         "-v",
         "--verbose",
         count=True,
+        default=None,
         help=(
-            "Verbosity, how detailed should the output be. This is *stackable*, so `-vv`"
-            " is more verbose than `-v`. For the most verbose option try `-vvvv` or `-vvvvv`."
+            "Verbosity, how detailed should the output be. This is *stackable*, so "
+            "`-vv` is more verbose than `-v`. For the most verbose option try `-vvvv` "
+            "or `-vvvvv`."
         ),
     )(f)
     f = click.option(
         "-n",
         "--nocolor",
         is_flag=True,
-        help="No color - if this is set then the output will be without ANSI color codes.",
+        default=None,
+        help="No color - output will be without ANSI color codes.",
     )(f)
+
     return f
 
 
-def core_options(f):
+def core_options(f: Callable) -> Callable:
     """Add core operation options to commands via a decorator.
 
     These are applied to the main (but not all) cli commands like
     `parse`, `lint` and `fix`.
     """
+    # Only enable dialect completion if on version of click
+    # that supports it
+    if shell_completion_enabled:
+        f = click.option(
+            "--dialect",
+            default=None,
+            help="The dialect of SQL to lint (default=ansi)",
+            shell_complete=dialect_shell_complete,
+        )(f)
+    else:  # pragma: no cover
+        f = click.option(
+            "--dialect",
+            default=None,
+            help="The dialect of SQL to lint (default=ansi)",
+        )(f)
     f = click.option(
-        "--dialect", default=None, help="The dialect of SQL to lint (default=ansi)"
-    )(f)
-    f = click.option(
-        "--templater", default=None, help="The templater to use (default=jinja)"
+        "--templater",
+        default=None,
+        help="The templater to use (default=jinja)",
+        type=click.Choice(
+            [
+                templater.name
+                for templater in chain.from_iterable(
+                    get_plugin_manager().hook.get_templaters()
+                )
+            ]
+        ),
     )(f)
     f = click.option(
         "--rules",
         default=None,
-        # short_help='Specify a particular rule, or comma separated rules, to check',
         help=(
             "Narrow the search to only specific rules. For example "
             "specifying `--rules L001` will only search for rule `L001` (Unnecessary "
@@ -152,25 +211,56 @@ def core_options(f):
     f = click.option(
         "--exclude-rules",
         default=None,
-        # short_help='Specify a particular rule, or comma separated rules to exclude',
         help=(
             "Exclude specific rules. For example "
             "specifying `--exclude-rules L001` will remove rule `L001` (Unnecessary "
             "trailing whitespace) from the set of considered rules. This could either "
-            "be the whitelist, or the general set if there is no specific whitelist. "
+            "be the allowlist, or the general set if there is no specific allowlist. "
             "Multiple rules can be specified with commas e.g. "
             "`--exclude-rules L001,L002` will exclude violations of rule "
             "`L001` and rule `L002`."
         ),
     )(f)
+    f = click.option(
+        "--config",
+        "extra_config_path",
+        default=None,
+        help=(
+            "Include additional config file. By default the config is generated "
+            "from the standard configuration files described in the documentation. "
+            "This argument allows you to specify an additional configuration file that "
+            "overrides the standard configuration files. N.B. cfg format is required."
+        ),
+        type=click.Path(),
+    )(f)
+    f = click.option(
+        "--ignore-local-config",
+        is_flag=True,
+        help=(
+            "Ignore config files in default search path locations. "
+            "This option allows the user to lint with the default config "
+            "or can be used in conjunction with --config to only "
+            "reference the custom config file."
+        ),
+    )(f)
+    f = click.option(
+        "--encoding",
+        default="autodetect",
+        help=(
+            "Specify encoding to use when reading and writing files. Defaults to "
+            "autodetect."
+        ),
+    )(f)
     f = click.option(
         "--ignore",
         default=None,
         help=(
             "Ignore particular families of errors so that they don't cause a failed "
             "run. For example `--ignore parsing` would mean that any parsing errors "
-            "are ignored and don't influence the success or fail of a run. Multiple "
-            "options are possible if comma separated e.g. `--ignore parsing,templating`."
+            "are ignored and don't influence the success or fail of a run. "
+            "`--ignore` behaves somewhat like `noqa` comments, except it "
+            "applies globally. Multiple options are possible if comma separated: "
+            "e.g. `--ignore parsing,templating`."
         ),
     )(f)
     f = click.option(
@@ -185,36 +275,94 @@ def core_options(f):
         ),
         help="Choose to limit the logging to one of the loggers.",
     )(f)
+    f = click.option(
+        "--disable-noqa",
+        is_flag=True,
+        default=None,
+        help="Set this flag to ignore inline noqa comments.",
+    )(f)
     return f
 
 
-def get_config(**kwargs):
+def get_config(
+    extra_config_path: Optional[str] = None,
+    ignore_local_config: bool = False,
+    **kwargs,
+) -> FluffConfig:
     """Get a config object from kwargs."""
-    if kwargs.get("dialect", None):
+    if "dialect" in kwargs:
         try:
-            # We're just making sure it exists at this stage - it will be fetched properly in the linter
+            # We're just making sure it exists at this stage.
+            # It will be fetched properly in the linter.
             dialect_selector(kwargs["dialect"])
+        except SQLFluffUserError as err:
+            click.echo(
+                colorize(
+                    f"Error loading dialect '{kwargs['dialect']}': {str(err)}",
+                    color=Color.red,
+                )
+            )
+            sys.exit(66)
         except KeyError:
-            click.echo("Error: Unknown dialect {!r}".format(kwargs["dialect"]))
+            click.echo(
+                colorize(
+                    f"Error: Unknown dialect '{kwargs['dialect']}'", color=Color.red
+                )
+            )
             sys.exit(66)
     # Instantiate a config object (filtering out the nulls)
     overrides = {k: kwargs[k] for k in kwargs if kwargs[k] is not None}
-    return FluffConfig.from_root(overrides=overrides)
+    try:
+        return FluffConfig.from_root(
+            extra_config_path=extra_config_path,
+            ignore_local_config=ignore_local_config,
+            overrides=overrides,
+        )
+    except SQLFluffUserError as err:  # pragma: no cover
+        click.echo(
+            colorize(
+                f"Error loading config: {str(err)}",
+                color=Color.red,
+            )
+        )
+        sys.exit(66)
 
 
-def get_linter_and_formatter(cfg, silent=False):
+def _callback_handler(cfg: FluffConfig) -> Callable:
+    """Returns function which will be bound as a callback for printing passed message.
+
+    Called in `get_linter_and_formatter`.
+    """
+
+    def _echo_with_tqdm_lock(message: str) -> None:
+        """Makes sure that message printing (echoing) will be not in conflict with tqdm.
+
+        It may happen that progressbar conflicts with extra printing. Nothing very
+        serious happens then, except that there is printed (not removed) progressbar
+        line. The `external_write_mode` allows to disable tqdm for writing time.
+        """
+        with tqdm.external_write_mode():
+            click.echo(message=message, color=cfg.get("color"))
+
+    return _echo_with_tqdm_lock
+
+
+def get_linter_and_formatter(
+    cfg: FluffConfig, silent: bool = False
+) -> Tuple[Linter, CallbackFormatter]:
     """Get a linter object given a config."""
     try:
-        # We're just making sure it exists at this stage - it will be fetched properly in the linter
+        # We're just making sure it exists at this stage.
+        # It will be fetched properly in the linter.
         dialect_selector(cfg.get("dialect"))
-    except KeyError:
-        click.echo("Error: Unknown dialect {!r}".format(cfg.get("dialect")))
+    except KeyError:  # pragma: no cover
+        click.echo(f"Error: Unknown dialect '{cfg.get('dialect')}'")
         sys.exit(66)
 
     if not silent:
-        # Instantiate the linter and return (with an output function)
+        # Instantiate the linter and return it (with an output function)
         formatter = CallbackFormatter(
-            callback=lambda m: click.echo(m, color=cfg.get("color")),
+            callback=_callback_handler(cfg=cfg),
             verbosity=cfg.get("verbose"),
             output_line_length=cfg.get("output_line_length"),
         )
@@ -226,7 +374,7 @@ def get_linter_and_formatter(cfg, silent=False):
         return Linter(config=cfg), formatter
 
 
-@click.group()
+@click.group(context_settings={"help_option_names": ["-h", "--help"]})
 @click.version_option()
 def cli():
     """Sqlfluff is a modular sql linter for humans."""
@@ -234,7 +382,7 @@ def cli():
 
 @cli.command()
 @common_options
-def version(**kwargs):
+def version(**kwargs) -> None:
     """Show the version of sqlfluff."""
     c = get_config(**kwargs)
     if c.get("verbose") > 0:
@@ -249,7 +397,7 @@ def version(**kwargs):
 
 @cli.command()
 @common_options
-def rules(**kwargs):
+def rules(**kwargs) -> None:
     """Show the current rules in use."""
     c = get_config(**kwargs)
     lnt, _ = get_linter_and_formatter(c)
@@ -258,12 +406,23 @@ def rules(**kwargs):
 
 @cli.command()
 @common_options
-def dialects(**kwargs):
+def dialects(**kwargs) -> None:
     """Show the current dialects available."""
     c = get_config(**kwargs)
     click.echo(format_dialects(dialect_readout), color=c.get("color"))
 
 
+def dump_file_payload(filename: Optional[str], payload: str):
+    """Write the output file content to stdout or file."""
+    # If there's a file specified to write to, write to it.
+    if filename:
+        with open(filename, "w") as out_file:
+            out_file.write(payload)
+    # Otherwise write to stdout
+    else:
+        click.echo(payload)
+
+
 @cli.command()
 @common_options
 @core_options
@@ -272,16 +431,25 @@ def dialects(**kwargs):
     "--format",
     "format",
     default="human",
-    type=click.Choice(
-        ["human", "json", "yaml", "github-annotation"], case_sensitive=False
-    ),
+    type=click.Choice([ft.value for ft in FormatType], case_sensitive=False),
     help="What format to return the lint result in (default=human).",
 )
+@click.option(
+    "--write-output",
+    help=(
+        "Optionally provide a filename to write the results to, mostly used in "
+        "tandem with --format. NB: Setting an output file re-enables normal "
+        "stdout logging."
+    ),
+)
 @click.option(
     "--annotation-level",
     default="notice",
     type=click.Choice(["notice", "warning", "failure"], case_sensitive=False),
-    help="When format is set to github-annotation, default annotation level (default=notice).",
+    help=(
+        "When format is set to github-annotation, "
+        "default annotation level (default=notice)."
+    ),
 )
 @click.option(
     "--nofail",
@@ -294,7 +462,7 @@ def dialects(**kwargs):
 @click.option(
     "--disregard-sqlfluffignores",
     is_flag=True,
-    help=("Perform the operation regardless of .sqlfluffignore configurations"),
+    help="Perform the operation regardless of .sqlfluffignore configurations",
 )
 @click.option(
     "-p",
@@ -303,18 +471,27 @@ def dialects(**kwargs):
     default=1,
     help="The number of parallel processes to run.",
 )
-@click.argument("paths", nargs=-1)
+@click.option(
+    "--disable_progress_bar",
+    is_flag=True,
+    help="Disables progress bars.",
+)
+@click.argument("paths", nargs=-1, type=click.Path(allow_dash=True))
 def lint(
-    paths,
-    processes,
-    format,
-    annotation_level,
-    nofail,
-    disregard_sqlfluffignores,
-    logger=None,
-    bench=False,
+    paths: Tuple[str],
+    processes: int,
+    format: str,
+    write_output: Optional[str],
+    annotation_level: str,
+    nofail: bool,
+    disregard_sqlfluffignores: bool,
+    logger: Optional[logging.Logger] = None,
+    bench: bool = False,
+    disable_progress_bar: Optional[bool] = False,
+    extra_config_path: Optional[str] = None,
+    ignore_local_config: bool = False,
     **kwargs,
-):
+) -> NoReturn:
     """Lint SQL files via passing a list of files or using stdin.
 
     PATH is the path to a sql file or directory to lint. This can be either a
@@ -333,10 +510,13 @@ def lint(
         echo 'select col from tbl' | sqlfluff lint -
 
     """
-    config = get_config(**kwargs)
-    non_human_output = format != "human"
+    config = get_config(extra_config_path, ignore_local_config, **kwargs)
+    non_human_output = (format != FormatType.human.value) or (write_output is not None)
+    file_output = None
     lnt, formatter = get_linter_and_formatter(config, silent=non_human_output)
+
     verbose = config.get("verbose")
+    progress_bar_configuration.disable_progress_bar = disable_progress_bar
 
     formatter.dispatch_config(lnt)
 
@@ -359,10 +539,9 @@ def lint(
         except OSError:
             click.echo(
                 colorize(
-                    "The path(s) {!r} could not be accessed. Check it/they exist(s).".format(
-                        paths
-                    ),
-                    "red",
+                    f"The path(s) '{paths}' could not be accessed. Check it/they "
+                    "exist(s).",
+                    Color.red,
                 )
             )
             sys.exit(1)
@@ -370,11 +549,11 @@ def lint(
         if verbose >= 1:
             click.echo(format_linting_stats(result, verbose=verbose))
 
-    if format == "json":
-        click.echo(json.dumps(result.as_records()))
-    elif format == "yaml":
-        click.echo(yaml.dump(result.as_records()))
-    elif format == "github-annotation":
+    if format == FormatType.json.value:
+        file_output = json.dumps(result.as_records())
+    elif format == FormatType.yaml.value:
+        file_output = yaml.dump(result.as_records(), sort_keys=False)
+    elif format == FormatType.github_annotation.value:
         github_result = []
         for record in result.as_records():
             filepath = record["filepath"]
@@ -394,7 +573,10 @@ def lint(
                         "annotation_level": annotation_level,
                     }
                 )
-        click.echo(json.dumps(github_result))
+        file_output = json.dumps(github_result)
+
+    if file_output:
+        dump_file_payload(write_output, cast(str, file_output))
 
     if bench:
         click.echo("==== overall timings ====")
@@ -412,6 +594,30 @@ def lint(
         sys.exit(0)
 
 
+def _handle_files_with_tmp_or_prs_errors(lint_result: LintingResult) -> int:
+    """Discard lint fixes for files with templating or parse errors.
+
+    Returns 1 if there are any files with templating or parse errors after
+    filtering, else 0. (Intended as a process exit code.)
+    """
+    total_errors, num_filtered_errors = lint_result.count_tmp_prs_errors()
+    lint_result.discard_fixes_for_lint_errors_in_files_with_tmp_or_prs_errors()
+    if total_errors:
+        click.echo(
+            colorize(f"  [{total_errors} templating/parsing errors found]", Color.red)
+        )
+        if num_filtered_errors < total_errors:
+            color = Color.red if num_filtered_errors else Color.green
+            click.echo(
+                colorize(
+                    f"  [{num_filtered_errors} templating/parsing errors "
+                    f'remaining after "ignore"]',
+                    color,
+                )
+            )
+    return 1 if num_filtered_errors else 0
+
+
 def do_fixes(lnt, result, formatter=None, **kwargs):
     """Actually do the fixes."""
     click.echo("Persisting Changes...")
@@ -420,9 +626,13 @@ def do_fixes(lnt, result, formatter=None, **kwargs):
         click.echo("Done. Please check your files to confirm.")
         return True
     # If some failed then return false
-    click.echo("Done. Some operations failed. Please check your files to confirm.")
-    click.echo("Some errors cannot be fixed or there is another error blocking it.")
-    return False
+    click.echo(
+        "Done. Some operations failed. Please check your files to confirm."
+    )  # pragma: no cover
+    click.echo(
+        "Some errors cannot be fixed or there is another error blocking it."
+    )  # pragma: no cover
+    return False  # pragma: no cover
 
 
 @cli.command()
@@ -447,8 +657,36 @@ def do_fixes(lnt, result, formatter=None, **kwargs):
     default=1,
     help="The number of parallel processes to run.",
 )
-@click.argument("paths", nargs=-1)
-def fix(force, paths, processes, bench=False, fixed_suffix="", logger=None, **kwargs):
+@click.option(
+    "--disable_progress_bar",
+    is_flag=True,
+    help="Disables progress bars.",
+)
+@click.option(
+    "--FIX-EVEN-UNPARSABLE",
+    is_flag=True,
+    help=(
+        "Enables fixing of files that have templating or parse errors. "
+        "Note that the similar-sounding '--ignore' or 'noqa' features merely "
+        "prevent errors from being *displayed*. For safety reasons, the 'fix'"
+        "command will not make any fixes in files that have templating or parse "
+        "errors unless '--FIX-EVEN-UNPARSABLE' is enabled on the command line"
+        "or in the .sqlfluff config file."
+    ),
+)
+@click.argument("paths", nargs=-1, type=click.Path(allow_dash=True))
+def fix(
+    force: bool,
+    paths: Tuple[str],
+    processes: int,
+    bench: bool = False,
+    fixed_suffix: str = "",
+    logger: Optional[logging.Logger] = None,
+    disable_progress_bar: Optional[bool] = False,
+    extra_config_path: Optional[str] = None,
+    ignore_local_config: bool = False,
+    **kwargs,
+) -> NoReturn:
     """Fix SQL files.
 
     PATH is the path to a sql file or directory to lint. This can be either a
@@ -459,9 +697,14 @@ def fix(force, paths, processes, bench=False, fixed_suffix="", logger=None, **kw
     # some quick checks
     fixing_stdin = ("-",) == paths
 
-    config = get_config(**kwargs)
+    config = get_config(extra_config_path, ignore_local_config, **kwargs)
+    fix_even_unparsable = config.get("fix_even_unparsable")
     lnt, formatter = get_linter_and_formatter(config, silent=fixing_stdin)
+
     verbose = config.get("verbose")
+    progress_bar_configuration.disable_progress_bar = disable_progress_bar
+
+    exit_code = 0
 
     formatter.dispatch_config(lnt)
 
@@ -471,39 +714,72 @@ def fix(force, paths, processes, bench=False, fixed_suffix="", logger=None, **kw
     # handle stdin case. should output formatted sql to stdout and nothing else.
     if fixing_stdin:
         stdin = sys.stdin.read()
+
         result = lnt.lint_string_wrapped(stdin, fname="stdin", fix=True)
-        stdout = result.paths[0].files[0].fix_string()[0]
+        templater_error = result.num_violations(types=SQLTemplaterError) > 0
+        unfixable_error = result.num_violations(types=SQLLintError, fixable=False) > 0
+        if not fix_even_unparsable:
+            exit_code = _handle_files_with_tmp_or_prs_errors(result)
+
+        if result.num_violations(types=SQLLintError, fixable=True) > 0:
+            stdout = result.paths[0].files[0].fix_string()[0]
+        else:
+            stdout = stdin
+
+        if templater_error:
+            click.echo(
+                colorize(
+                    "Fix aborted due to unparseable template variables.",
+                    Color.red,
+                ),
+                err=True,
+            )
+            click.echo(
+                colorize(
+                    "Use --fix-even-unparsable' to attempt to fix the SQL anyway.",
+                    Color.red,
+                ),
+                err=True,
+            )
+
+        if unfixable_error:
+            click.echo(colorize("Unfixable violations detected.", Color.red), err=True)
+
         click.echo(stdout, nl=False)
-        sys.exit()
+        sys.exit(1 if templater_error or unfixable_error else exit_code)
 
     # Lint the paths (not with the fix argument at this stage), outputting as we go.
     click.echo("==== finding fixable violations ====")
     try:
         result = lnt.lint_paths(
-            paths, fix=True, ignore_non_existent_files=False, processes=processes
+            paths,
+            fix=True,
+            ignore_non_existent_files=False,
+            processes=processes,
         )
     except OSError:
         click.echo(
             colorize(
-                "The path(s) {!r} could not be accessed. Check it/they exist(s).".format(
-                    paths
-                ),
-                "red",
-            )
+                f"The path(s) '{paths}' could not be accessed. Check it/they exist(s).",
+                Color.red,
+            ),
+            err=True,
         )
         sys.exit(1)
 
+    if not fix_even_unparsable:
+        exit_code = _handle_files_with_tmp_or_prs_errors(result)
+
     # NB: We filter to linting violations here, because they're
     # the only ones which can be potentially fixed.
     if result.num_violations(types=SQLLintError, fixable=True) > 0:
         click.echo("==== fixing violations ====")
         click.echo(
-            "{} fixable linting violations found".format(
-                result.num_violations(types=SQLLintError, fixable=True)
-            )
+            f"{result.num_violations(types=SQLLintError, fixable=True)} fixable "
+            "linting violations found"
         )
         if force:
-            click.echo(colorize("FORCE MODE", "red") + ": Attempting fixes...")
+            click.echo(f"{colorize('FORCE MODE', Color.red)}: Attempting fixes...")
             success = do_fixes(
                 lnt,
                 result,
@@ -512,7 +788,7 @@ def fix(force, paths, processes, bench=False, fixed_suffix="", logger=None, **kw
                 fixed_file_suffix=fixed_suffix,
             )
             if not success:
-                sys.exit(1)
+                sys.exit(1)  # pragma: no cover
         else:
             click.echo(
                 "Are you sure you wish to attempt to fix these? [Y/n] ", nl=False
@@ -529,24 +805,33 @@ def fix(force, paths, processes, bench=False, fixed_suffix="", logger=None, **kw
                     fixed_file_suffix=fixed_suffix,
                 )
                 if not success:
-                    sys.exit(1)
+                    sys.exit(1)  # pragma: no cover
                 else:
                     _completion_message(config)
             elif c == "n":
                 click.echo("Aborting...")
-            else:
+                exit_code = 1
+            else:  # pragma: no cover
                 click.echo("Invalid input, please enter 'Y' or 'N'")
                 click.echo("Aborting...")
+                exit_code = 1
     else:
         click.echo("==== no fixable linting violations found ====")
-        if result.num_violations(types=SQLLintError, fixable=False) > 0:
-            click.echo(
-                "  [{} unfixable linting violations found]".format(
-                    result.num_violations(types=SQLLintError, fixable=False)
-                )
-            )
         _completion_message(config)
 
+    error_types = [
+        (
+            dict(types=SQLLintError, fixable=False),
+            "  [{} unfixable linting violations found]",
+            1,
+        ),
+    ]
+    for num_violations_kwargs, message_format, error_level in error_types:
+        num_violations = result.num_violations(**num_violations_kwargs)
+        if num_violations > 0:
+            click.echo(message_format.format(num_violations))
+            exit_code = max(exit_code, error_level)
+
     if bench:
         click.echo("==== overall timings ====")
         click.echo(cli_table([("Clock time", result.total_time)]))
@@ -555,12 +840,13 @@ def fix(force, paths, processes, bench=False, fixed_suffix="", logger=None, **kw
             click.echo(f"=== {step} ===")
             click.echo(cli_table(timing_summary[step].items()))
 
-    sys.exit(0)
+    sys.exit(exit_code)
 
 
-def _completion_message(config):
+def _completion_message(config: FluffConfig) -> None:
     click.echo(
-        "All Finished{emojis}!".format(emojis="" if config.get("nocolor") else " 📜 🎉")
+        "All Finished"
+        f"{'' if (config.get('nocolor') or not sys.stdout.isatty()) else ' 📜 🎉'}!"
     )
 
 
@@ -575,7 +861,7 @@ def quoted_presenter(dumper, data):
 @cli.command()
 @common_options
 @core_options
-@click.argument("path", nargs=1)
+@click.argument("path", nargs=1, type=click.Path(allow_dash=True))
 @click.option(
     "--recurse", default=0, help="The depth to recursively parse to (0 for unlimited)"
 )
@@ -597,10 +883,25 @@ def quoted_presenter(dumper, data):
 @click.option(
     "-f",
     "--format",
-    default="human",
-    type=click.Choice(["human", "json", "yaml"], case_sensitive=False),
+    default=FormatType.human.value,
+    type=click.Choice(
+        [
+            FormatType.human.value,
+            FormatType.json.value,
+            FormatType.yaml.value,
+        ],
+        case_sensitive=False,
+    ),
     help="What format to return the parse result in.",
 )
+@click.option(
+    "--write-output",
+    help=(
+        "Optionally provide a filename to write the results to, mostly used in "
+        "tandem with --format. NB: Setting an output file re-enables normal "
+        "stdout logging."
+    ),
+)
 @click.option(
     "--profiler", is_flag=True, help="Set this flag to engage the python profiler."
 )
@@ -613,16 +914,19 @@ def quoted_presenter(dumper, data):
     ),
 )
 def parse(
-    path,
-    code_only,
-    include_meta,
-    format,
-    profiler,
-    bench,
-    nofail,
-    logger=None,
+    path: str,
+    code_only: bool,
+    include_meta: bool,
+    format: str,
+    write_output: Optional[str],
+    profiler: bool,
+    bench: bool,
+    nofail: bool,
+    logger: Optional[logging.Logger] = None,
+    extra_config_path: Optional[str] = None,
+    ignore_local_config: bool = False,
     **kwargs,
-):
+) -> NoReturn:
     """Parse SQL files and just spit out the result.
 
     PATH is the path to a sql file or directory to lint. This can be either a
@@ -630,25 +934,28 @@ def parse(
     character to indicate reading from *stdin* or a dot/blank ('.'/' ') which will
     be interpreted like passing the current working directory as a path argument.
     """
-    c = get_config(**kwargs)
+    c = get_config(extra_config_path, ignore_local_config, **kwargs)
     # We don't want anything else to be logged if we want json or yaml output
-    non_human_output = format in ("json", "yaml")
+    # unless we're writing to a file.
+    non_human_output = (format != FormatType.human.value) or (write_output is not None)
     lnt, formatter = get_linter_and_formatter(c, silent=non_human_output)
     verbose = c.get("verbose")
     recurse = c.get("recurse")
 
+    progress_bar_configuration.disable_progress_bar = True
+
     formatter.dispatch_config(lnt)
 
     # Set up logging.
     set_logging_level(verbosity=verbose, logger=logger, stderr_output=non_human_output)
 
     # TODO: do this better
-    nv = 0
+
     if profiler:
         # Set up the profiler if required
         try:
             import cProfile
-        except ImportError:
+        except ImportError:  # pragma: no cover
             click.echo("The cProfiler is not available on your platform.")
             sys.exit(1)
         pr = cProfile.Profile()
@@ -656,51 +963,31 @@ def parse(
 
     try:
         t0 = time.monotonic()
+
         # handle stdin if specified via lone '-'
         if "-" == path:
-            # put the parser result in a list to iterate later
-            result = [
+            parsed_strings = [
                 lnt.parse_string(
-                    sys.stdin.read(), "stdin", recurse=recurse, config=lnt.config
+                    sys.stdin.read(),
+                    "stdin",
+                    recurse=recurse,
+                    config=lnt.config,
                 ),
             ]
         else:
             # A single path must be specified for this command
-            result = lnt.parse_path(path, recurse=recurse)
+            parsed_strings = list(lnt.parse_path(path, recurse=recurse))
+
         total_time = time.monotonic() - t0
+        violations_count = 0
 
         # iterative print for human readout
-        if format == "human":
-            timing = TimingSummary()
-            for parsed_string in result:
-                timing.add(parsed_string.time_dict)
-                if parsed_string.tree:
-                    click.echo(parsed_string.tree.stringify(code_only=code_only))
-                else:
-                    # TODO: Make this prettier
-                    click.echo("...Failed to Parse...")
-                nv += len(parsed_string.violations)
-                if parsed_string.violations:
-                    click.echo("==== parsing violations ====")
-                for v in parsed_string.violations:
-                    click.echo(format_violation(v))
-                if (
-                    parsed_string.violations
-                    and parsed_string.config.get("dialect") == "ansi"
-                ):
-                    click.echo(format_dialect_warning())
-                if verbose >= 2:
-                    click.echo("==== timings ====")
-                    click.echo(cli_table(parsed_string.time_dict.items()))
-            if verbose >= 2 or bench:
-                click.echo("==== overall timings ====")
-                click.echo(cli_table([("Clock time", total_time)]))
-                timing_summary = timing.summary()
-                for step in timing_summary:
-                    click.echo(f"=== {step} ===")
-                    click.echo(cli_table(timing_summary[step].items()))
+        if format == FormatType.human.value:
+            violations_count = _print_out_violations_and_timing(
+                bench, code_only, total_time, verbose, parsed_strings
+            )
         else:
-            result = [
+            parsed_strings_dict = [
                 dict(
                     filepath=linted_result.fname,
                     segments=linted_result.tree.as_record(
@@ -709,22 +996,27 @@ def parse(
                     if linted_result.tree
                     else None,
                 )
-                for linted_result in result
+                for linted_result in parsed_strings
             ]
 
-            if format == "yaml":
-                # For yaml dumping always dump double quoted strings if they contain tabs or newlines.
+            if format == FormatType.yaml.value:
+                # For yaml dumping always dump double quoted strings if they contain
+                # tabs or newlines.
                 yaml.add_representer(str, quoted_presenter)
+                file_output = yaml.dump(parsed_strings_dict, sort_keys=False)
+            elif format == FormatType.json.value:
+                file_output = json.dumps(parsed_strings_dict)
 
-                click.echo(yaml.dump(result))
-            elif format == "json":
-                click.echo(json.dumps(result))
-    except OSError:
+            # Dump the output to stdout or to file as appropriate.
+            dump_file_payload(write_output, file_output)
+
+    except OSError:  # pragma: no cover
         click.echo(
             colorize(
-                f"The path {path!r} could not be accessed. Check it exists.",
-                "red",
-            )
+                f"The path '{path}' could not be accessed. Check it exists.",
+                Color.red,
+            ),
+            err=True,
         )
         sys.exit(1)
 
@@ -737,14 +1029,57 @@ def parse(
         # Only print the first 50 lines of it
         click.echo("\n".join(profiler_buffer.getvalue().split("\n")[:50]))
 
-    if nv > 0 and not nofail:
-        sys.exit(66)
+    if violations_count > 0 and not nofail:
+        sys.exit(66)  # pragma: no cover
     else:
         sys.exit(0)
 
 
+def _print_out_violations_and_timing(
+    bench: bool,
+    code_only: bool,
+    total_time: float,
+    verbose: int,
+    parsed_strings: List[ParsedString],
+) -> int:
+    """Used by human formatting during the parse."""
+    violations_count = 0
+    timing = TimingSummary()
+
+    for parsed_string in parsed_strings:
+        timing.add(parsed_string.time_dict)
+
+        if parsed_string.tree:
+            click.echo(parsed_string.tree.stringify(code_only=code_only))
+        else:
+            # TODO: Make this prettier
+            click.echo("...Failed to Parse...")  # pragma: no cover
+
+        violations_count += len(parsed_string.violations)
+        if parsed_string.violations:
+            click.echo("==== parsing violations ====")  # pragma: no cover
+        for v in parsed_string.violations:
+            click.echo(format_violation(v))  # pragma: no cover
+        if parsed_string.violations and parsed_string.config.get("dialect") == "ansi":
+            click.echo(format_dialect_warning())  # pragma: no cover
+
+        if verbose >= 2:
+            click.echo("==== timings ====")
+            click.echo(cli_table(parsed_string.time_dict.items()))
+
+    if verbose >= 2 or bench:
+        click.echo("==== overall timings ====")
+        click.echo(cli_table([("Clock time", total_time)]))
+        timing_summary = timing.summary()
+        for step in timing_summary:
+            click.echo(f"=== {step} ===")
+            click.echo(cli_table(timing_summary[step].items()))
+
+    return violations_count
+
+
 # This "__main__" handler allows invoking SQLFluff using "python -m", which
 # simplifies the use of cProfile, e.g.:
 # python -m cProfile -s cumtime -m sqlfluff.cli.commands lint slow_file.sql
 if __name__ == "__main__":
-    cli.main(sys.argv[1:])
+    cli.main(sys.argv[1:])  # pragma: no cover
