@@ -2,6 +2,7 @@
 
 
 from io import StringIO
+from typing import Callable, List, Union
 
 from sqlfluff.cli.helpers import (
     colorize,
@@ -11,22 +12,26 @@ from sqlfluff.cli.helpers import (
     get_python_implementation,
     pad_line,
 )
-
-from sqlfluff.core import SQLBaseError
+from sqlfluff.core import SQLBaseError, FluffConfig, Linter
+from sqlfluff.core.enums import Color
+from sqlfluff.core.linter import LintedFile
 
 
-def format_filename(filename, success=False, success_text="PASS"):
+def format_filename(
+    filename: str, success: Union[str, bool] = False, success_text: str = "PASS"
+) -> str:
     """Format filenames."""
     if isinstance(success, str):
         status_string = success
     else:
         status_string = colorize(
-            success_text if success else "FAIL", "green" if success else "red"
+            success_text if success else "FAIL",
+            Color.green if success else Color.red,
         )
-    return "== [" + colorize(f"{filename}", "lightgrey") + "] " + status_string
+    return f"== [{colorize(filename, Color.lightgrey)}] {status_string}"
 
 
-def split_string_on_spaces(s, line_length=100):
+def split_string_on_spaces(s: str, line_length: int = 100) -> List[str]:
     """Split a string into lines based on whitespace."""
     line_buff = []
     str_buff = ""
@@ -48,23 +53,23 @@ def split_string_on_spaces(s, line_length=100):
     return line_buff
 
 
-def format_violation(violation, max_line_length=90):
+def format_violation(violation: SQLBaseError, max_line_length: int = 90) -> str:
     """Format a violation."""
     if isinstance(violation, SQLBaseError):
         desc = violation.desc()
         if violation.line_no is not None:
             line_elem = f"{violation.line_no:4d}"
         else:
-            line_elem = "   -"
+            line_elem = "   -"  # pragma: no cover
         if violation.line_pos is not None:
             pos_elem = f"{violation.line_pos:4d}"
         else:
-            pos_elem = "   -"
-    else:
+            pos_elem = "   -"  # pragma: no cover
+    else:  # pragma: no cover
         raise ValueError(f"Unexpected violation format: {violation}")
 
     if violation.ignore:
-        desc = "IGNORE: " + desc
+        desc = "IGNORE: " + desc  # pragma: no cover
 
     split_desc = split_string_on_spaces(desc, line_length=max_line_length - 25)
 
@@ -72,17 +77,18 @@ def format_violation(violation, max_line_length=90):
     for idx, line in enumerate(split_desc):
         if idx == 0:
             out_buff += colorize(
-                "L:{} | P:{} | {} | ".format(
-                    line_elem, pos_elem, violation.rule_code().rjust(4)
-                ),
+                f"L:{line_elem} | P:{pos_elem} | {violation.rule_code().rjust(4)} | ",
                 # Grey out the violation if we're ignoring it.
-                "lightgrey" if violation.ignore else "blue",
+                Color.lightgrey if violation.ignore else Color.blue,
             )
         else:
             out_buff += (
                 "\n"
                 + (" " * 23)
-                + colorize("| ", "lightgrey" if violation.ignore else "blue")
+                + colorize(
+                    "| ",
+                    Color.lightgrey if violation.ignore else Color.blue,
+                )
             )
         out_buff += line
     return out_buff
@@ -136,14 +142,14 @@ def format_config_vals(config_vals):
         val = "" if v is None else str(v)
         text_buffer.write(
             ("    " * i)
-            + colorize(pad_line(str(k) + ":", 20, "left"), color="lightgrey")
+            + colorize(pad_line(str(k) + ":", 20, "left"), color=Color.lightgrey)
             + pad_line(val, 20, "left")
             + "\n"
         )
     return text_buffer.getvalue()
 
 
-def format_rules(linter, verbose=0):
+def format_rules(linter: Linter, verbose: int = 0) -> str:
     """Format the a set of rules given a `Linter`."""
     text_buffer = StringIO()
     text_buffer.write("==== sqlfluff - rules ====\n")
@@ -152,7 +158,7 @@ def format_rules(linter, verbose=0):
             linter.rule_tuples(),
             col_width=80,
             cols=1,
-            label_color="blue",
+            label_color=Color.blue,
             val_align="left",
         )
     )
@@ -166,26 +172,30 @@ def format_dialects(dialect_readout, verbose=0):
     readouts = [
         (
             dialect.label,
-            "{dialect.name} dialect [inherits from '{dialect.inherits_from}']".format(
-                dialect=dialect
-            ),
+            f"{dialect.name} dialect [inherits from '{dialect.inherits_from}']",
         )
         for dialect in dialect_readout()
     ]
     text_buffer.write(
-        cli_table(readouts, col_width=60, cols=1, label_color="blue", val_align="right")
+        cli_table(
+            readouts,
+            col_width=60,
+            cols=1,
+            label_color=Color.blue,
+            val_align="right",
+        )
     )
     return text_buffer.getvalue()
 
 
-def format_dialect_warning():
+def format_dialect_warning():  # pragma: no cover
     """Output a warning for parsing errors found on the ansi dialect."""
     return colorize(
         (
             "WARNING: Parsing errors found and dialect is set to "
             "'ansi'. Have you configured your dialect?"
         ),
-        "lightgrey",
+        Color.lightgrey,
     )
 
 
@@ -211,22 +221,28 @@ class CallbackFormatter:
 
     """
 
-    def __init__(self, callback, verbosity=0, filter_empty=True, output_line_length=80):
+    def __init__(
+        self,
+        callback: Callable,
+        verbosity: int = 0,
+        filter_empty: bool = True,
+        output_line_length: int = 80,
+    ):
         self._callback = callback
         self._verbosity = verbosity
         self._filter_empty = filter_empty
         self.output_line_length = output_line_length
 
-    def _dispatch(self, s):
+    def _dispatch(self, s: str) -> None:
         """Dispatch a string to the callback.
 
         This method is designed as a point for subclassing.
         """
         # The strip here is to filter out any empty messages
         if (not self._filter_empty) or s.strip(" \n\t"):
-            return self._callback(s)
+            self._callback(s)
 
-    def _format_config(self, linter):
+    def _format_config(self, linter: Linter) -> str:
         """Format the config of a `Linter`."""
         text_buffer = StringIO()
         # Only show version information if verbosity is high enough
@@ -243,10 +259,10 @@ class CallbackFormatter:
                 cli_table(config_content, col_width=30, max_label_width=15)
             )
             text_buffer.write("\n")
-            if linter.config.get("rule_whitelist"):
+            if linter.config.get("rule_allowlist"):
                 text_buffer.write(
                     cli_table(
-                        [("rules", ", ".join(linter.config.get("rule_whitelist")))],
+                        [("rules", ", ".join(linter.config.get("rule_allowlist")))],
                         col_width=41,
                     )
                 )
@@ -255,9 +271,9 @@ class CallbackFormatter:
                 text_buffer.write(format_config_vals(linter.config.iter_vals()))
         return text_buffer.getvalue()
 
-    def dispatch_config(self, linter):
+    def dispatch_config(self, linter: Linter) -> None:
         """Dispatch configuration output appropriately."""
-        return self._dispatch(self._format_config(linter))
+        self._dispatch(self._format_config(linter))
 
     def dispatch_persist_filename(self, filename, result):
         """Dispatch filenames during a persist operation."""
@@ -266,16 +282,18 @@ class CallbackFormatter:
             self._dispatch(format_filename(filename=filename, success=result))
 
     @staticmethod
-    def _format_path(path):
+    def _format_path(path: str) -> str:
         """Format paths."""
-        return "=== [ path: {} ] ===\n".format(colorize(path, "lightgrey"))
+        return f"=== [ path: {colorize(path, Color.lightgrey)} ] ===\n"
 
-    def dispatch_path(self, path):
+    def dispatch_path(self, path: str) -> None:
         """Dispatch paths for display."""
         if self._verbosity > 0:
             self._dispatch(self._format_path(path))
 
-    def dispatch_template_header(self, fname, linter_config, file_config):
+    def dispatch_template_header(
+        self, fname: str, linter_config: FluffConfig, file_config: FluffConfig
+    ) -> None:
         """Dispatch the header displayed before templating."""
         if self._verbosity > 1:
             self._dispatch(format_filename(filename=fname, success="TEMPLATING"))
@@ -283,31 +301,35 @@ class CallbackFormatter:
             if file_config:
                 # Only output config diffs if there is a config to diff to.
                 config_diff = file_config.diff_to(linter_config)
-                if config_diff:
+                if config_diff:  # pragma: no cover
                     self._dispatch("   Config Diff:")
                     self._dispatch(
                         format_config_vals(linter_config.iter_vals(cfg=config_diff))
                     )
 
-    def dispatch_parse_header(self, fname):
+    def dispatch_parse_header(self, fname: str) -> None:
         """Dispatch the header displayed before parsing."""
         if self._verbosity > 1:
             self._dispatch(format_filename(filename=fname, success="PARSING"))
 
-    def dispatch_lint_header(self, fname):
+    def dispatch_lint_header(self, fname: str) -> None:
         """Dispatch the header displayed before linting."""
         if self._verbosity > 1:
             self._dispatch(format_filename(filename=fname, success="LINTING"))
 
     def dispatch_compilation_header(self, templater, message):
         """Dispatch the header displayed before linting."""
-        self._dispatch("=== [" + colorize(templater, "lightgrey") + "] " + message)
+        self._dispatch(
+            f"=== [{colorize(templater, Color.lightgrey)}] {message}"
+        )  # pragma: no cover
 
-    def dispatch_dialect_warning(self):
+    def dispatch_dialect_warning(self) -> None:
         """Dispatch a warning for dialects."""
-        self._dispatch(format_dialect_warning())
+        self._dispatch(format_dialect_warning())  # pragma: no cover
 
-    def _format_file_violations(self, fname, violations):
+    def _format_file_violations(
+        self, fname: str, violations: List[SQLBaseError]
+    ) -> str:
         """Format a set of violations in a `LintingResult`."""
         text_buffer = StringIO()
         # Success is having no violations (which aren't ignored)
@@ -333,7 +355,9 @@ class CallbackFormatter:
             str_buffer = str_buffer[:-1]
         return str_buffer
 
-    def dispatch_file_violations(self, fname, linted_file, only_fixable):
+    def dispatch_file_violations(
+        self, fname: str, linted_file: LintedFile, only_fixable: bool
+    ) -> None:
         """Dispatch any violations found in a file."""
         s = self._format_file_violations(
             fname, linted_file.get_violations(fixable=True if only_fixable else None)
