@@ -7,6 +7,8 @@ post linting.
 
 import os
 import logging
+import shutil
+import tempfile
 from typing import (
     Any,
     Iterable,
@@ -47,7 +49,7 @@ class LintedFile(NamedTuple):
     templated_file: TemplatedFile
     encoding: str
 
-    def check_tuples(self) -> List[CheckTuple]:
+    def check_tuples(self, raise_on_non_linting_violations=True) -> List[CheckTuple]:
         """Make a list of check_tuples.
 
         This assumes that all the violations found are
@@ -59,7 +61,7 @@ class LintedFile(NamedTuple):
         for v in self.get_violations():
             if hasattr(v, "check_tuple"):
                 vs.append(v.check_tuple())
-            else:
+            elif raise_on_non_linting_violations:
                 raise v
         return vs
 
@@ -83,7 +85,7 @@ class LintedFile(NamedTuple):
             if isinstance(types, type) and issubclass(types, SQLBaseError):
                 types = (types,)
             else:
-                types = tuple(types)
+                types = tuple(types)  # pragma: no cover TODO?
             violations = [v for v in violations if isinstance(v, types)]
         # Filter rules
         if rules:
@@ -101,7 +103,7 @@ class LintedFile(NamedTuple):
             violations = [v for v in violations if not v.ignore]
             # Ignore any rules in the ignore mask
             if self.ignore_mask:
-                violations = self._ignore_masked_violations(violations)
+                violations = self.ignore_masked_violations(violations, self.ignore_mask)
         return violations
 
     @staticmethod
@@ -170,8 +172,9 @@ class LintedFile(NamedTuple):
                 result.append(v)
         return result
 
-    def _ignore_masked_violations(
-        self, violations: List[SQLBaseError]
+    @classmethod
+    def ignore_masked_violations(
+        cls, violations: List[SQLBaseError], ignore_mask: List[NoQaDirective]
     ) -> List[SQLBaseError]:
         """Remove any violations specified by ignore_mask.
 
@@ -179,12 +182,12 @@ class LintedFile(NamedTuple):
         1. Filter out violations affected by single-line "noqa" directives.
         2. Filter out violations affected by disable/enable "noqa" directives.
         """
-        ignore_specific = [ignore for ignore in self.ignore_mask if not ignore.action]
-        ignore_range = [ignore for ignore in self.ignore_mask if ignore.action]
-        violations = self._ignore_masked_violations_single_line(
+        ignore_specific = [ignore for ignore in ignore_mask if not ignore.action]
+        ignore_range = [ignore for ignore in ignore_mask if ignore.action]
+        violations = cls._ignore_masked_violations_single_line(
             violations, ignore_specific
         )
-        violations = self._ignore_masked_violations_line_range(violations, ignore_range)
+        violations = cls._ignore_masked_violations_line_range(violations, ignore_range)
         return violations
 
     def num_violations(self, **kwargs) -> int:
@@ -205,15 +208,19 @@ class LintedFile(NamedTuple):
     ):
         """Log hints for debugging during patch generation."""
         # This next bit is ALL FOR LOGGING AND DEBUGGING
-        if patch.templated_slice.start >= 10:
+        max_log_length = 10
+        if patch.templated_slice.start >= max_log_length:
             pre_hint = templated_file.templated_str[
-                patch.templated_slice.start - 10 : patch.templated_slice.start
+                patch.templated_slice.start
+                - max_log_length : patch.templated_slice.start
             ]
         else:
             pre_hint = templated_file.templated_str[: patch.templated_slice.start]
-        if patch.templated_slice.stop + 10 < len(templated_file.templated_str):
+        if patch.templated_slice.stop + max_log_length < len(
+            templated_file.templated_str
+        ):
             post_hint = templated_file.templated_str[
-                patch.templated_slice.stop : patch.templated_slice.stop + 10
+                patch.templated_slice.stop : patch.templated_slice.stop + max_log_length
             ]
         else:
             post_hint = templated_file.templated_str[patch.templated_slice.stop :]
@@ -284,9 +291,10 @@ class LintedFile(NamedTuple):
                 source_slice = self.templated_file.templated_slice_to_source_slice(
                     patch.templated_slice,
                 )
-            except ValueError:
+            except ValueError:  # pragma: no cover
                 linter_logger.info(
-                    "      - Skipping. Source space Value Error. i.e. attempted insertion within templated section."
+                    "      - Skipping. Source space Value Error. i.e. attempted "
+                    "insertion within templated section."
                 )
                 # If we try and slice within a templated section, then we may fail
                 # in which case, we should skip this patch.
@@ -323,10 +331,11 @@ class LintedFile(NamedTuple):
                 source_str=self.templated_file.source_str[source_slice],
             )
 
-            # Deal with the easy case of only literals
-            if set(local_type_list) == {"literal"}:
+            # Deal with the easy cases of 1) New code at end 2) only literals
+            if not local_type_list or set(local_type_list) == {"literal"}:
                 linter_logger.info(
-                    "      * Keeping patch on literal-only section: %s", enriched_patch
+                    "      * Keeping patch on new or literal-only section: %s",
+                    enriched_patch,
                 )
                 filtered_source_patches.append(enriched_patch)
                 dedupe_buffer.append(enriched_patch.dedupe_tuple())
@@ -342,49 +351,53 @@ class LintedFile(NamedTuple):
                 filtered_source_patches.append(enriched_patch)
                 dedupe_buffer.append(enriched_patch.dedupe_tuple())
             # If it's ONLY templated then we should skip it.
-            elif "literal" not in local_type_list:
+            elif "literal" not in local_type_list:  # pragma: no cover
                 linter_logger.info(
                     "      - Skipping patch over templated section: %s", enriched_patch
                 )
             # If we span more than two slices then we should just skip it. Too Hard.
-            elif len(local_raw_slices) > 2:
+            elif len(local_raw_slices) > 2:  # pragma: no cover
                 linter_logger.info(
                     "      - Skipping patch over more than two raw slices: %s",
                     enriched_patch,
                 )
-            # If it's an insertion (i.e. the string in the pre-fix template is '') then we
-            # won't be able to place it, so skip.
-            elif not enriched_patch.templated_str:
+            # If it's an insertion (i.e. the string in the pre-fix template is '') then
+            # we won't be able to place it, so skip.
+            elif not enriched_patch.templated_str:  # pragma: no cover TODO?
                 linter_logger.info(
                     "      - Skipping insertion patch in templated section: %s",
                     enriched_patch,
                 )
-            # If the string from the templated version isn't in the source, then we can't fix it.
-            elif enriched_patch.templated_str not in enriched_patch.source_str:
+            # If the string from the templated version isn't in the source, then we
+            # can't fix it.
+            elif (
+                enriched_patch.templated_str not in enriched_patch.source_str
+            ):  # pragma: no cover TODO?
                 linter_logger.info(
                     "      - Skipping edit patch on templated content: %s",
                     enriched_patch,
                 )
-            else:
+            else:  # pragma: no cover
                 # Identify all the places the string appears in the source content.
                 positions = list(
                     findall(enriched_patch.templated_str, enriched_patch.source_str)
                 )
                 if len(positions) != 1:
                     linter_logger.debug(
-                        "        - Skipping edit patch on non-unique templated content: %s",
+                        "        - Skipping edit patch on non-unique templated "
+                        "content: %s",
                         enriched_patch,
                     )
                     continue
-                # We have a single occurrences of the thing we want to patch. This
+                # We have a single occurrence of the thing we want to patch. This
                 # means we can use its position to place our patch.
-                new_source_slice = slice(
+                new_source_slice = slice(  # pragma: no cover
                     enriched_patch.source_slice.start + positions[0],
                     enriched_patch.source_slice.start
                     + positions[0]
                     + len(enriched_patch.templated_str),
                 )
-                enriched_patch = EnrichedFixPatch(
+                enriched_patch = EnrichedFixPatch(  # pragma: no cover
                     source_slice=new_source_slice,
                     templated_slice=enriched_patch.templated_slice,
                     patch_category=enriched_patch.patch_category,
@@ -392,15 +405,16 @@ class LintedFile(NamedTuple):
                     templated_str=enriched_patch.templated_str,
                     source_str=enriched_patch.source_str,
                 )
-                linter_logger.debug(
-                    "      * Keeping Tricky Case. Positions: %s, New Slice: %s, Patch: %s",
+                linter_logger.debug(  # pragma: no cover
+                    "      * Keeping Tricky Case. Positions: %s, New Slice: %s, "
+                    "Patch: %s",
                     positions,
                     new_source_slice,
                     enriched_patch,
                 )
-                filtered_source_patches.append(enriched_patch)
-                dedupe_buffer.append(enriched_patch.dedupe_tuple())
-                continue
+                filtered_source_patches.append(enriched_patch)  # pragma: no cover
+                dedupe_buffer.append(enriched_patch.dedupe_tuple())  # pragma: no cover
+                continue  # pragma: no cover
 
         # Sort the patches before building up the file.
         filtered_source_patches = sorted(
@@ -486,7 +500,24 @@ class LintedFile(NamedTuple):
             if suffix:
                 root, ext = os.path.splitext(fname)
                 fname = root + suffix + ext
-            # Actually write the file.
-            with open(fname, "w", encoding=self.encoding) as f:
-                f.write(write_buff)
+            self._safe_create_replace_file(fname, write_buff, self.encoding)
         return success
+
+    @staticmethod
+    def _safe_create_replace_file(fname, write_buff, encoding):
+        # Write to a temporary file first, so in case of encoding or other
+        # issues, we don't delete or corrupt the user's existing file.
+        dirname, basename = os.path.split(fname)
+        with tempfile.NamedTemporaryFile(
+            mode="w",
+            encoding=encoding,
+            prefix=basename,
+            dir=dirname,
+            suffix=os.path.splitext(fname)[1],
+            delete=False,
+        ) as tmp:
+            tmp.file.write(write_buff)
+            tmp.flush()
+            os.fsync(tmp.fileno())
+        # Once the temp file is safely written, replace the existing file.
+        shutil.move(tmp.name, fname)
