@@ -1,21 +1,23 @@
 """Defines the linter class."""
 
+import fnmatch
 import os
 import time
 import logging
 from typing import (
     Any,
-    Generator,
     List,
     Sequence,
     Optional,
     Tuple,
-    Union,
     cast,
     Iterable,
+    Iterator,
 )
 
 import pathspec
+import regex
+from tqdm import tqdm
 
 from sqlfluff.core.errors import (
     SQLBaseError,
@@ -24,14 +26,13 @@ from sqlfluff.core.errors import (
     SQLParseError,
     SQLTemplaterSkipFile,
 )
-from sqlfluff.core.parser import Lexer, Parser
+from sqlfluff.core.parser import Lexer, Parser, RegexLexer
 from sqlfluff.core.file_helpers import get_encoding
 from sqlfluff.core.templaters import TemplatedFile
 from sqlfluff.core.rules import get_ruleset
-from sqlfluff.core.config import FluffConfig, ConfigLoader
+from sqlfluff.core.config import FluffConfig, ConfigLoader, progress_bar_configuration
 
 # Classes needed only for type checking
-from sqlfluff.core.linter.runner import get_runner
 from sqlfluff.core.parser.segments.base import BaseSegment
 from sqlfluff.core.parser.segments.meta import MetaSegment
 from sqlfluff.core.parser.segments.raw import RawSegment
@@ -65,12 +66,16 @@ class Linter:
         config: Optional[FluffConfig] = None,
         formatter: Any = None,
         dialect: Optional[str] = None,
-        rules: Optional[Union[str, List[str]]] = None,
-        user_rules: Optional[Union[str, List[str]]] = None,
+        rules: Optional[List[str]] = None,
+        user_rules: Optional[List[BaseRule]] = None,
+        exclude_rules: Optional[List[str]] = None,
     ) -> None:
         # Store the config object
         self.config = FluffConfig.from_kwargs(
-            config=config, dialect=dialect, rules=rules
+            config=config,
+            dialect=dialect,
+            rules=rules,
+            exclude_rules=exclude_rules,
         )
         # Get the dialect and templater
         self.dialect = self.config.get("dialect_obj")
@@ -98,7 +103,9 @@ class Linter:
     # These are the building blocks of the linting process.
 
     @staticmethod
-    def _load_raw_file_and_config(fname, root_config):
+    def _load_raw_file_and_config(
+        fname: str, root_config: FluffConfig
+    ) -> Tuple[str, FluffConfig, str]:
         """Load a raw file and the associated config."""
         file_config = root_config.make_child_from_path(fname)
         encoding = get_encoding(fname=fname, config=file_config)
@@ -109,6 +116,11 @@ class Linter:
         # Return the raw file and config
         return raw_file, file_config, encoding
 
+    @staticmethod
+    def _normalise_newlines(string: str) -> str:
+        """Normalise newlines to unix-style line endings."""
+        return regex.sub(r"\r\n|\r", "\n", string)
+
     @staticmethod
     def _lex_templated_file(
         templated_file: TemplatedFile, config: FluffConfig
@@ -135,7 +147,7 @@ class Linter:
             violations.append(err)
             return None, violations, config
 
-        if not tokens:
+        if not tokens:  # pragma: no cover TODO?
             return None, violations, config
 
         # Check that we've got sensible indentation from the lexer.
@@ -154,13 +166,12 @@ class Linter:
             )
             if indent_balance != 0:
                 linter_logger.debug(
-                    "Indent balance test failed for %r. Template indents will not be linted for this file.",
+                    "Indent balance test failed for %r. Template indents will not be "
+                    "linted for this file.",
                     templated_file.fname,
                 )
                 # Don't enable the templating blocks.
                 templating_blocks_indent = False
-                # Disable the linting of L003 on templated tokens.
-                config.set_value(["rules", "L003", "lint_templated_tokens"], False)
 
         # The file will have been lexed without config, so check all indents
         # are enabled.
@@ -173,18 +184,26 @@ class Linter:
                     if not templating_blocks_indent:
                         continue
             new_tokens.append(token)
+
         # Return new buffer
         return new_tokens, violations, config
 
     @staticmethod
     def _parse_tokens(
-        tokens: Sequence[BaseSegment], config: FluffConfig, recurse: bool = True
+        tokens: Sequence[BaseSegment],
+        config: FluffConfig,
+        recurse: bool = True,
+        fname: Optional[str] = None,
     ) -> Tuple[Optional[BaseSegment], List[SQLParseError]]:
         parser = Parser(config=config)
         violations = []
         # Parse the file and log any problems
         try:
-            parsed: Optional[BaseSegment] = parser.parse(tokens, recurse=recurse)
+            parsed: Optional[BaseSegment] = parser.parse(
+                tokens,
+                recurse=recurse,
+                fname=fname,
+            )
         except SQLParseError as err:
             linter_logger.info("PARSING FAILED! : %s", err)
             violations.append(err)
@@ -193,13 +212,15 @@ class Linter:
         if parsed:
             linter_logger.info("\n###\n#\n# {}\n#\n###".format("Parsed Tree:"))
             linter_logger.info("\n" + parsed.stringify())
-            # We may succeed parsing, but still have unparsable segments. Extract them here.
+            # We may succeed parsing, but still have unparsable segments. Extract them
+            # here.
             for unparsable in parsed.iter_unparsables():
                 # No exception has been raised explicitly, but we still create one here
                 # so that we can use the common interface
                 violations.append(
                     SQLParseError(
-                        "Line {0[0]}, Position {0[1]}: Found unparsable section: {1!r}".format(
+                        "Line {0[0]}, Position {0[1]}: Found unparsable section: "
+                        "{1!r}".format(
                             unparsable.pos_marker.working_loc,
                             unparsable.raw
                             if len(unparsable.raw) < 40
@@ -213,9 +234,19 @@ class Linter:
         return parsed, violations
 
     @staticmethod
-    def parse_noqa(comment: str, line_no: int):
+    def parse_noqa(
+        comment: str,
+        line_no: int,
+        rule_codes: List[str],
+    ):
         """Extract ignore mask entries from a comment string."""
         # Also trim any whitespace afterward
+
+        # Comment lines can also have noqa e.g.
+        # --dafhsdkfwdiruweksdkjdaffldfsdlfjksd -- noqa: L016
+        # Therefore extract last possible inline ignore.
+        comment = [c.strip() for c in comment.split("--")][-1]
+
         if comment.startswith("noqa"):
             # This is an ignore identifier
             comment_remainder = comment[4:]
@@ -230,7 +261,7 @@ class Linter:
                     action: Optional[str]
                     if "=" in comment_remainder:
                         action, rule_part = comment_remainder.split("=", 1)
-                        if action not in {"disable", "enable"}:
+                        if action not in {"disable", "enable"}:  # pragma: no cover
                             return SQLParseError(
                                 "Malformed 'noqa' section. "
                                 "Expected 'noqa: enable=<rule>[,...] | all' "
@@ -249,7 +280,27 @@ class Linter:
                             )
                     rules: Optional[Tuple[str, ...]]
                     if rule_part != "all":
-                        rules = tuple(r.strip() for r in rule_part.split(","))
+                        # Rules can be globs therefore we compare to the rule_set to
+                        # expand the globs.
+                        unexpanded_rules = tuple(
+                            r.strip() for r in rule_part.split(",")
+                        )
+                        expanded_rules = []
+                        for r in unexpanded_rules:
+                            expanded_rule = [
+                                x
+                                for x in fnmatch.filter(rule_codes, r)
+                                if x not in expanded_rules
+                            ]
+                            if expanded_rule:
+                                expanded_rules.extend(expanded_rule)
+                            elif r not in expanded_rules:
+                                # We were unable to expand the glob.
+                                # Therefore assume the user is referencing
+                                # a special error type (e.g. PRS, LXR, or TMP)
+                                # and add this to the list of rules to ignore.
+                                expanded_rules.append(r)
+                        rules = tuple(expanded_rules)
                     else:
                         rules = None
                     return NoQaDirective(line_no, rules, action)
@@ -258,22 +309,25 @@ class Linter:
 
     @staticmethod
     def remove_templated_errors(
-        linting_errors: List[SQLLintError],
-    ) -> List[SQLLintError]:
-        """Filter a list of lint errors, removing those which only occur in templated slices."""
+        linting_errors: List[SQLBaseError],
+    ) -> List[SQLBaseError]:
+        """Filter a list of lint errors, removing those from the templated slices."""
         # Filter out any linting errors in templated sections if relevant.
-        linting_errors = list(
-            filter(
-                lambda e: (
+        result: List[SQLBaseError] = []
+        for e in linting_errors:
+            if isinstance(e, SQLLintError):
+                if (
                     # Is it in a literal section?
                     e.segment.pos_marker.is_literal()
                     # Is it a rule that is designed to work on templated sections?
                     or e.rule.targets_templated
-                ),
-                linting_errors,
-            )
-        )
-        return linting_errors
+                ):
+                    result.append(e)
+            else:
+                # If it's another type, just keep it. (E.g. SQLParseError from
+                # malformed "noqa" comment).
+                result.append(e)
+        return result
 
     @staticmethod
     def _warn_unfixable(code: str):
@@ -285,7 +339,11 @@ class Linter:
     # These compose the base static methods into useful recipes.
 
     @classmethod
-    def parse_rendered(cls, rendered: RenderedFile, recurse: bool = True):
+    def parse_rendered(
+        cls,
+        rendered: RenderedFile,
+        recurse: bool = True,
+    ) -> ParsedString:
         """Parse a rendered file."""
         t0 = time.monotonic()
         violations = cast(List[SQLBaseError], rendered.templater_violations)
@@ -302,7 +360,12 @@ class Linter:
         linter_logger.info("PARSING (%s)", rendered.fname)
 
         if tokens:
-            parsed, pvs = cls._parse_tokens(tokens, rendered.config, recurse=recurse)
+            parsed, pvs = cls._parse_tokens(
+                tokens,
+                rendered.config,
+                recurse=recurse,
+                fname=rendered.fname,
+            )
             violations += pvs
         else:
             parsed = None
@@ -319,29 +382,36 @@ class Linter:
             rendered.templated_file,
             rendered.config,
             rendered.fname,
+            rendered.source_str,
         )
 
     @classmethod
-    def extract_ignore_from_comment(cls, comment: RawSegment):
+    def extract_ignore_from_comment(
+        cls,
+        comment: RawSegment,
+        rule_codes: List[str],
+    ):
         """Extract ignore mask entries from a comment segment."""
         # Also trim any whitespace afterward
         comment_content = comment.raw_trimmed().strip()
         comment_line, _ = comment.pos_marker.source_position()
-        result = cls.parse_noqa(comment_content, comment_line)
+        result = cls.parse_noqa(comment_content, comment_line, rule_codes)
         if isinstance(result, SQLParseError):
             result.segment = comment
         return result
 
     @classmethod
-    def extract_ignore_mask(cls, tree: Optional[BaseSegment]):
+    def extract_ignore_mask_tree(
+        cls,
+        tree: BaseSegment,
+        rule_codes: List[str],
+    ) -> Tuple[List[NoQaDirective], List[SQLBaseError]]:
         """Look for inline ignore comments and return NoQaDirectives."""
         ignore_buff: List[NoQaDirective] = []
-        violations: List[SQLParseError] = []
-        if not tree:
-            return ignore_buff, violations
+        violations: List[SQLBaseError] = []
         for comment in tree.recursive_crawl("comment"):
             if comment.name == "inline_comment":
-                ignore_entry = cls.extract_ignore_from_comment(comment)
+                ignore_entry = cls.extract_ignore_from_comment(comment, rule_codes)
                 if isinstance(ignore_entry, SQLParseError):
                     violations.append(ignore_entry)
                 elif ignore_entry:
@@ -350,6 +420,34 @@ class Linter:
             linter_logger.info("Parsed noqa directives from file: %r", ignore_buff)
         return ignore_buff, violations
 
+    @classmethod
+    def extract_ignore_mask_source(
+        cls,
+        source: str,
+        inline_comment_regex: RegexLexer,
+        rule_codes: List[str],
+    ) -> Tuple[List[NoQaDirective], List[SQLBaseError]]:
+        """Look for inline ignore comments and return NoQaDirectives.
+
+        Very similar to extract_ignore_mask_tree(), but can be run on raw source
+        (i.e. does not require the code to have parsed successfully).
+        """
+        ignore_buff: List[NoQaDirective] = []
+        violations: List[SQLBaseError] = []
+        for idx, line in enumerate(source.split("\n")):
+            match = inline_comment_regex.search(line) if line else None
+            if match:
+                ignore_entry = cls.parse_noqa(
+                    line[match[0] : match[1]], idx + 1, rule_codes
+                )
+                if isinstance(ignore_entry, SQLParseError):
+                    violations.append(ignore_entry)  # pragma: no cover
+                elif ignore_entry:
+                    ignore_buff.append(ignore_entry)
+        if ignore_buff:
+            linter_logger.info("Parsed noqa directives from file: %r", ignore_buff)
+        return ignore_buff, violations
+
     @classmethod
     def lint_fix_parsed(
         cls,
@@ -360,7 +458,7 @@ class Linter:
         fname: Optional[str] = None,
         templated_file: Optional[TemplatedFile] = None,
         formatter: Any = None,
-    ) -> Tuple[BaseSegment, List[SQLLintError]]:
+    ) -> Tuple[BaseSegment, List[SQLBaseError], List[NoQaDirective]]:
         """Lint and optionally fix a tree object."""
         # Keep track of the linting errors
         all_linting_errors = []
@@ -369,16 +467,36 @@ class Linter:
         # Keep a set of previous versions to catch infinite loops.
         previous_versions = {tree.raw}
 
-        # If we are fixing then we want to loop up to the runaway_limit, otherwise just once for linting.
+        # If we are fixing then we want to loop up to the runaway_limit, otherwise just
+        # once for linting.
         loop_limit = config.get("runaway_limit") if fix else 1
 
         # Dispatch the output for the lint header
         if formatter:
             formatter.dispatch_lint_header(fname)
 
+        # Look for comment segments which might indicate lines to ignore.
+        if not config.get("disable_noqa"):
+            rule_codes = [r.code for r in rule_set]
+            ignore_buff, ivs = cls.extract_ignore_mask_tree(tree, rule_codes)
+            all_linting_errors += ivs
+        else:
+            ignore_buff = []
+
+        save_tree = tree
         for loop in range(loop_limit):
             changed = False
-            for crawler in rule_set:
+
+            progress_bar_crawler = tqdm(
+                rule_set,
+                desc="lint by rules",
+                leave=False,
+                disable=progress_bar_configuration.disable_progress_bar,
+            )
+
+            for crawler in progress_bar_crawler:
+                progress_bar_crawler.set_description(f"rule {crawler.code}")
+
                 # fixes should be a dict {} with keys edit, delete, create
                 # delete is just a list of segments to delete
                 # edit and create are list of tuples. The first element is the
@@ -386,6 +504,7 @@ class Linter:
                 # The second is the element to insert or create.
                 linting_errors, _, fixes, _ = crawler.crawl(
                     tree,
+                    ignore_mask=ignore_buff,
                     dialect=config.get("dialect_obj"),
                     fname=fname,
                     templated_file=templated_file,
@@ -395,7 +514,7 @@ class Linter:
                 if fix and fixes:
                     linter_logger.info(f"Applying Fixes [{crawler.code}]: {fixes}")
                     # Do some sanity checks on the fixes before applying.
-                    if fixes == last_fixes:
+                    if fixes == last_fixes:  # pragma: no cover
                         cls._warn_unfixable(crawler.code)
                     else:
                         last_fixes = fixes
@@ -420,16 +539,40 @@ class Linter:
                 # We did not change the file. Either the file is clean (no fixes), or
                 # any fixes which are present will take us back to a previous state.
                 linter_logger.info(
-                    f"Fix loop complete. Stability achieved after {loop}/{loop_limit} loops."
+                    f"Fix loop complete. Stability achieved after {loop}/{loop_limit} "
+                    "loops."
                 )
                 break
-        if fix and loop + 1 == loop_limit:
-            linter_logger.warning(f"Loop limit on fixes reached [{loop_limit}].")
+        else:
+            if fix:
+                # The linter loop hit the limit before reaching a stable point
+                # (i.e. free of lint errors). If this happens, it's usually
+                # because one or more rules produced fixes which did not address
+                # the original issue **or** created new issues.
+                linter_logger.warning(f"Loop limit on fixes reached [{loop_limit}].")
+
+                # Discard any fixes for the linting errors, since they caused a
+                # loop. IMPORTANT: By doing this, we are telling SQLFluff that
+                # these linting errors are "unfixable". This is important,
+                # because when "sqlfluff fix" encounters unfixable lint errors,
+                # it exits with a "failure" exit code, which is exactly what we
+                # want in this situation. (Reason: Although this is more of an
+                # internal SQLFluff issue, users deserve to know about it,
+                # because it means their file(s) weren't fixed.
+                for violation in initial_linting_errors:
+                    if isinstance(violation, SQLLintError):
+                        violation.fixes = []
+
+                # Return the original parse tree, before any fixes were applied.
+                # Reason: When the linter hits the loop limit, the file is often
+                # messy, e.g. some of the fixes were applied repeatedly, possibly
+                # other weird things. We don't want the user to see this junk!
+                return save_tree, initial_linting_errors, ignore_buff
 
         if config.get("ignore_templated_areas", default=True):
             initial_linting_errors = cls.remove_templated_errors(initial_linting_errors)
 
-        return tree, initial_linting_errors
+        return tree, initial_linting_errors, ignore_buff
 
     @classmethod
     def lint_parsed(
@@ -443,15 +586,11 @@ class Linter:
         """Lint a ParsedString and return a LintedFile."""
         violations = parsed.violations
         time_dict = parsed.time_dict
-        # Look for comment segments which might indicate lines to ignore.
-        ignore_buff, ivs = cls.extract_ignore_mask(parsed.tree)
-        violations += ivs
-
         tree: Optional[BaseSegment]
         if parsed.tree:
             t0 = time.monotonic()
             linter_logger.info("LINTING (%s)", parsed.fname)
-            tree, initial_linting_errors = cls.lint_fix_parsed(
+            tree, initial_linting_errors, ignore_buff = cls.lint_fix_parsed(
                 parsed.tree,
                 config=parsed.config,
                 rule_set=rule_set,
@@ -468,8 +607,23 @@ class Linter:
             violations += initial_linting_errors
         else:
             # If no parsed tree, set to None
-
             tree = None
+            ignore_buff = []
+            if not parsed.config.get("disable_noqa"):
+                # Templating and/or parsing have failed. Look for "noqa"
+                # comments (the normal path for identifying these comments
+                # requires access to the parse tree, and because of the failure,
+                # we don't have a parse tree).
+                ignore_buff, ignore_violations = cls.extract_ignore_mask_source(
+                    parsed.source_str,
+                    [
+                        lm
+                        for lm in parsed.config.get("dialect_obj").lexer_matchers
+                        if lm.name == "inline_comment"
+                    ][0],
+                    [r.code for r in rule_set],
+                )
+                violations += ignore_violations
 
         # We process the ignore config here if appropriate
         for violation in violations:
@@ -495,7 +649,7 @@ class Linter:
         if parsed.config.get("dialect") == "ansi" and linted_file.get_violations(
             fixable=True if fix else None, types=SQLParseError
         ):
-            if formatter:
+            if formatter:  # pragma: no cover TODO?
                 formatter.dispatch_dialect_warning()
 
         return linted_file
@@ -531,20 +685,27 @@ class Linter:
         # Start the templating timer
         t0 = time.monotonic()
 
+        # Newlines are normalised to unix-style line endings (\n).
+        # The motivation is that Jinja normalises newlines during templating and
+        # we want consistent mapping between the raw and templated slices.
+        in_str = self._normalise_newlines(in_str)
+
         if not config.get("templater_obj") == self.templater:
             linter_logger.warning(
                 (
-                    f"Attempt to set templater to {config.get('templater_obj').name} failed. Using {self.templater.name} "
-                    "templater. Templater cannot be set in a .sqlfluff file in a subdirectory of the current working "
-                    "directory. It can be set in a .sqlfluff in the current working directory. See Nesting section of the "
-                    "docs for more details."
+                    f"Attempt to set templater to {config.get('templater_obj').name} "
+                    f"failed. Using {self.templater.name} templater. Templater cannot "
+                    "be set in a .sqlfluff file in a subdirectory of the current "
+                    "working directory. It can be set in a .sqlfluff in the current "
+                    "working directory. See Nesting section of the docs for more "
+                    "details."
                 )
             )
         try:
             templated_file, templater_violations = self.templater.process(
                 in_str=in_str, fname=fname, config=config, formatter=self.formatter
             )
-        except SQLTemplaterSkipFile as s:
+        except SQLTemplaterSkipFile as s:  # pragma: no cover
             linter_logger.warning(str(s))
             templated_file = None
             templater_violations = []
@@ -556,7 +717,13 @@ class Linter:
         time_dict = {"templating": time.monotonic() - t0}
 
         return RenderedFile(
-            templated_file, templater_violations, config, time_dict, fname, encoding
+            templated_file,
+            templater_violations,
+            config,
+            time_dict,
+            fname,
+            encoding,
+            in_str,
         )
 
     def render_file(self, fname: str, root_config: FluffConfig) -> RenderedFile:
@@ -601,11 +768,11 @@ class Linter:
         config: Optional[FluffConfig] = None,
         fname: Optional[str] = None,
         templated_file: Optional[TemplatedFile] = None,
-    ) -> Tuple[BaseSegment, List[SQLLintError]]:
+    ) -> Tuple[BaseSegment, List[SQLBaseError]]:
         """Return the fixed tree and violations from lintfix when we're fixing."""
         config = config or self.config
         rule_set = self.get_ruleset(config=config)
-        fixed_tree, violations = self.lint_fix_parsed(
+        fixed_tree, violations, _ = self.lint_fix_parsed(
             tree,
             config,
             rule_set,
@@ -622,11 +789,11 @@ class Linter:
         config: Optional[FluffConfig] = None,
         fname: Optional[str] = None,
         templated_file: Optional[TemplatedFile] = None,
-    ) -> List[SQLLintError]:
+    ) -> List[SQLBaseError]:
         """Return just the violations from lintfix when we're only linting."""
         config = config or self.config
         rule_set = self.get_ruleset(config=config)
-        _, violations = self.lint_fix_parsed(
+        _, violations, _ = self.lint_fix_parsed(
             tree,
             config,
             rule_set,
@@ -654,12 +821,20 @@ class Linter:
         # Sort out config, defaulting to the built in config if no override
         config = config or self.config
         # Parse the string.
-        parsed = self.parse_string(in_str=in_str, fname=fname, config=config)
+        parsed = self.parse_string(
+            in_str=in_str,
+            fname=fname,
+            config=config,
+        )
         # Get rules as appropriate
         rule_set = self.get_ruleset(config=config)
         # Lint the file and return the LintedFile
         return self.lint_parsed(
-            parsed, rule_set, fix=fix, formatter=self.formatter, encoding=encoding
+            parsed,
+            rule_set,
+            fix=fix,
+            formatter=self.formatter,
+            encoding=encoding,
         )
 
     def paths_from_path(
@@ -692,6 +867,7 @@ class Linter:
         # matched, but we warn the users when that happens
         is_exact_file = os.path.isfile(path)
 
+        path_walk: WalkableType
         if is_exact_file:
             # When the exact file to lint is passed, we
             # fill path_walk with an input that follows
@@ -699,28 +875,30 @@ class Linter:
             #   (root, directories, files)
             dirpath = os.path.dirname(path)
             files = [os.path.basename(path)]
-            ignore_file_paths = ConfigLoader.find_ignore_config_files(
-                path=path, working_path=working_path, ignore_file_name=ignore_file_name
-            )
-            # Add paths that could contain "ignore files"
-            # to the path_walk list
-            path_walk_ignore_file = [
-                (
-                    os.path.dirname(ignore_file_path),
-                    None,
-                    # Only one possible file, since we only
-                    # have one "ignore file name"
-                    [os.path.basename(ignore_file_path)],
-                )
-                for ignore_file_path in ignore_file_paths
-            ]
-            path_walk: WalkableType = [(dirpath, None, files)] + path_walk_ignore_file
+            path_walk = [(dirpath, None, files)]
         else:
-            path_walk = os.walk(path)
+            path_walk = list(os.walk(path))
+
+        ignore_file_paths = ConfigLoader.find_ignore_config_files(
+            path=path, working_path=working_path, ignore_file_name=ignore_file_name
+        )
+        # Add paths that could contain "ignore files"
+        # to the path_walk list
+        path_walk_ignore_file = [
+            (
+                os.path.dirname(ignore_file_path),
+                None,
+                # Only one possible file, since we only
+                # have one "ignore file name"
+                [os.path.basename(ignore_file_path)],
+            )
+            for ignore_file_path in ignore_file_paths
+        ]
+        path_walk += path_walk_ignore_file
 
         # If it's a directory then expand the path!
         buffer = []
-        ignore_set = set()
+        ignores = {}
         for dirpath, _, filenames in path_walk:
             for fname in filenames:
                 fpath = os.path.join(dirpath, fname)
@@ -728,10 +906,7 @@ class Linter:
                 if ignore_files and fname == ignore_file_name:
                     with open(fpath) as fh:
                         spec = pathspec.PathSpec.from_lines("gitwildmatch", fh)
-                    matches = spec.match_tree(dirpath)
-                    for m in matches:
-                        ignore_path = os.path.join(dirpath, m)
-                        ignore_set.add(os.path.abspath(ignore_path))
+                        ignores[dirpath] = spec
                     # We don't need to process the ignore file any futher
                     continue
 
@@ -751,26 +926,40 @@ class Linter:
         filtered_buffer = []
 
         for fpath in buffer:
-            if os.path.abspath(fpath) not in ignore_set:
+            abs_fpath = os.path.abspath(fpath)
+            for ignore_base, ignore_spec in ignores.items():
+                abs_ignore_base = os.path.abspath(ignore_base)
+                if abs_fpath.startswith(
+                    abs_ignore_base + os.sep
+                ) and ignore_spec.match_file(
+                    os.path.relpath(abs_fpath, abs_ignore_base)
+                ):
+                    # This file is ignored, skip it.
+                    if is_exact_file:
+                        linter_logger.warning(
+                            "Exact file path %s was given but "
+                            "it was ignored by a %s pattern in %s, "
+                            "re-run with `--disregard-sqlfluffignores` to "
+                            "skip %s"
+                            % (
+                                path,
+                                ignore_file_name,
+                                ignore_base,
+                                ignore_file_name,
+                            )
+                        )
+                    break
+            else:
                 filtered_buffer.append(os.path.normpath(fpath))
-            elif is_exact_file:
-                linter_logger.warning(
-                    "Exact file path %s was given but "
-                    "it was ignored by a %s pattern, "
-                    "re-run with `--disregard-sqlfluffignores` to "
-                    "skip %s"
-                    % (
-                        path,
-                        ignore_file_name,
-                        ignore_file_name,
-                    )
-                )
 
         # Return
         return sorted(filtered_buffer)
 
     def lint_string_wrapped(
-        self, string: str, fname: str = "<string input>", fix: bool = False
+        self,
+        string: str,
+        fname: str = "<string input>",
+        fix: bool = False,
     ) -> LintingResult:
         """Lint strings directly."""
         result = LintingResult()
@@ -799,18 +988,43 @@ class Linter:
                 ignore_files=ignore_files,
             )
         )
+
+        # to avoid circular import
+        from sqlfluff.core.linter.runner import get_runner
+
         runner = get_runner(
             self,
             self.config,
             processes=processes,
             allow_process_parallelism=self.allow_process_parallelism,
         )
-        for linted_file in runner.run(fnames, fix):
+
+        # Show files progress bar only when there is more than one.
+        files_count = len(fnames)
+        progress_bar_files = tqdm(
+            total=files_count,
+            desc=f"file {os.path.basename(fnames[0] if fnames else '')}",
+            leave=False,
+            disable=files_count <= 1 or progress_bar_configuration.disable_progress_bar,
+        )
+
+        for i, linted_file in enumerate(runner.run(fnames, fix), start=1):
             linted_path.add(linted_file)
             # If any fatal errors, then stop iteration.
-            if any(v.fatal for v in linted_file.violations):
+            if any(v.fatal for v in linted_file.violations):  # pragma: no cover
                 linter_logger.error("Fatal linting error. Halting further linting.")
                 break
+
+            # Progress bar for files is rendered only when there is more than one file.
+            # Additionally as it's updated after each loop, we need to get file name
+            # from the next loop. This is why `enumerate` starts with `1` and there
+            # is `i < len` to not exceed files list length.
+            progress_bar_files.update(n=1)
+            if i < len(fnames):
+                progress_bar_files.set_description(
+                    f"file {os.path.basename(fnames[i])}"
+                )
+
         return linted_path
 
     def lint_paths(
@@ -822,14 +1036,25 @@ class Linter:
         processes: int = 1,
     ) -> LintingResult:
         """Lint an iterable of paths."""
+        paths_count = len(paths)
+
         # If no paths specified - assume local
-        if len(paths) == 0:
+        if not paths_count:  # pragma: no cover
             paths = (os.getcwd(),)
         # Set up the result to hold what we get back
         result = LintingResult()
+
+        progress_bar_paths = tqdm(
+            total=paths_count,
+            desc="path",
+            leave=False,
+            disable=paths_count <= 1 or progress_bar_configuration.disable_progress_bar,
+        )
         for path in paths:
-            # Iterate through files recursively in the specified directory (if it's a directory)
-            # or read the file directly if it's not
+            progress_bar_paths.set_description(f"path {path}")
+
+            # Iterate through files recursively in the specified directory (if it's a
+            # directory) or read the file directly if it's not
             result.add(
                 self.lint_path(
                     path,
@@ -839,12 +1064,17 @@ class Linter:
                     processes=processes,
                 )
             )
+
+            progress_bar_paths.update(1)
+
         result.stop_timer()
         return result
 
     def parse_path(
-        self, path: str, recurse: bool = True
-    ) -> Generator[ParsedString, None, None]:
+        self,
+        path: str,
+        recurse: bool = True,
+    ) -> Iterator[ParsedString]:
         """Parse a path of sql files.
 
         NB: This a generator which will yield the result of each file
@@ -858,5 +1088,9 @@ class Linter:
                 fname, self.config
             )
             yield self.parse_string(
-                raw_file, fname=fname, recurse=recurse, config=config, encoding=encoding
+                raw_file,
+                fname=fname,
+                recurse=recurse,
+                config=config,
+                encoding=encoding,
             )
