@@ -22,6 +22,7 @@ class AnyNumberOf(BaseGrammar):
     def __init__(self, *args, **kwargs):
         self.max_times = kwargs.pop("max_times", None)
         self.min_times = kwargs.pop("min_times", 0)
+        self.max_times_per_element = kwargs.pop("max_times_per_element", None)
         # Any patterns to _prevent_ a match.
         self.exclude = kwargs.pop("exclude", None)
         super().__init__(*args, **kwargs)
@@ -48,12 +49,18 @@ class AnyNumberOf(BaseGrammar):
         """
         return self.optional or self.min_times == 0
 
+    @staticmethod
+    def _first_non_whitespace(segments) -> Optional[str]:
+        """Return the upper first non-whitespace segment in the iterable."""
+        for segment in segments:
+            if segment.raw_segments_upper:
+                return segment.raw_segments_upper
+        return None
+
     def _prune_options(
         self, segments: Tuple[BaseSegment, ...], parse_context: ParseContext
     ) -> Tuple[List[MatchableType], List[str]]:
         """Use the simple matchers to prune which options to match on."""
-        str_buff = [segment.raw_upper for segment in self._iter_raw_segs(segments)]
-
         available_options = []
         simple_opts = []
         prune_buff = []
@@ -62,11 +69,7 @@ class AnyNumberOf(BaseGrammar):
         matched_simple = 0
 
         # Find the first code element to match against.
-        first_elem = None
-        for elem in str_buff:
-            if elem.strip():
-                first_elem = elem
-                break
+        first_elem = self._first_non_whitespace(segments)
 
         for opt in self._elements:
             simple = opt.simple(parse_context=parse_context)
@@ -80,22 +83,22 @@ class AnyNumberOf(BaseGrammar):
             # it for pruning.
             for simple_opt in simple:
                 # Check it's not a whitespace option
-                if not simple_opt.strip():
+                if not simple_opt.strip():  # pragma: no cover
                     raise NotImplementedError(
                         "_prune_options not supported for whitespace matching."
                     )
                 # We want to know if the first meaningful element of the str_buff
                 # matches the option.
-                if simple_opt in str_buff:
-                    # match the FIRST non-whitespace element of the list.
-                    if first_elem != simple_opt:
-                        # No match, carry on.
-                        continue
-                    # If we get here, it's matched the FIRST element of the string buffer.
-                    available_options.append(opt)
-                    simple_opts.append(simple_opt)
-                    matched_simple += 1
-                    break
+
+                # match the FIRST non-whitespace element of the list.
+                if first_elem != simple_opt:
+                    # No match, carry on.
+                    continue
+                # If we get here, it's matched the FIRST element of the string buffer.
+                available_options.append(opt)
+                simple_opts.append(simple_opt)
+                matched_simple += 1
+                break
             else:
                 # Ditch this option, the simple match has failed
                 prune_buff.append(opt)
@@ -119,7 +122,7 @@ class AnyNumberOf(BaseGrammar):
 
     def _match_once(
         self, segments: Tuple[BaseSegment, ...], parse_context: ParseContext
-    ) -> MatchResult:
+    ) -> Tuple[MatchResult, Optional["MatchableType"]]:
         """Match the forward segments against the available elements once.
 
         This serves as the main body of OneOf, but also a building block
@@ -139,14 +142,14 @@ class AnyNumberOf(BaseGrammar):
             return MatchResult.from_unmatched(segments)
 
         with parse_context.deeper_match() as ctx:
-            match, _ = self._longest_trimmed_match(
+            match, matched_option = self._longest_trimmed_match(
                 segments,
                 available_options,
                 parse_context=ctx,
                 trim_noncode=False,
             )
 
-        return match
+        return match, matched_option
 
     @match_wrapper()
     @allow_ephemeral
@@ -169,6 +172,13 @@ class AnyNumberOf(BaseGrammar):
         matched_segments: MatchResult = MatchResult.from_empty()
         unmatched_segments: Tuple[BaseSegment, ...] = segments
         n_matches = 0
+
+        # Keep track of the number of times each option has been matched.
+        available_options, _ = self._prune_options(
+            segments, parse_context=parse_context
+        )
+        available_option_counter = {str(o): 0 for o in available_options}
+
         while True:
             if self.max_times and n_matches >= self.max_times:
                 # We've matched as many times as we can
@@ -183,7 +193,7 @@ class AnyNumberOf(BaseGrammar):
                     return MatchResult(
                         matched_segments.matched_segments, unmatched_segments
                     )
-                else:
+                else:  # pragma: no cover TODO?
                     # We didn't meet the hurdle
                     return MatchResult.from_unmatched(unmatched_segments)
 
@@ -195,7 +205,23 @@ class AnyNumberOf(BaseGrammar):
             else:
                 pre_seg = ()  # empty tuple
 
-            match = self._match_once(unmatched_segments, parse_context=parse_context)
+            match, matched_option = self._match_once(
+                unmatched_segments, parse_context=parse_context
+            )
+
+            # Increment counter for matched option.
+            if matched_option and (str(matched_option) in available_option_counter):
+                available_option_counter[str(matched_option)] += 1
+                # Check if we have matched an option too many times.
+                if (
+                    self.max_times_per_element
+                    and available_option_counter[str(matched_option)]
+                    > self.max_times_per_element
+                ):
+                    return MatchResult(
+                        matched_segments.matched_segments, unmatched_segments
+                    )
+
             if match:
                 matched_segments += pre_seg + match.matched_segments
                 unmatched_segments = match.unmatched_segments
@@ -238,3 +264,10 @@ class OptionallyBracketed(OneOf):
             args[0] if len(args) == 1 else Sequence(*args),
             **kwargs,
         )
+
+
+class AnySetOf(AnyNumberOf):
+    """Match any number of the elements but each element can only be matched once."""
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, max_times_per_element=1, **kwargs)
