@@ -101,9 +101,8 @@ class BaseGrammar(Matchable):
             ):
                 return init_func(elem)
         raise TypeError(
-            "Grammar element [{!r}] was found of unexpected type [{}] was found.".format(
-                elem, type(elem)
-            )
+            "Grammar element [{!r}] was found of unexpected type [{}] was "
+            "found.".format(elem, type(elem))  # pragma: no cover
         )
 
     def __init__(
@@ -135,8 +134,8 @@ class BaseGrammar(Matchable):
                 chunk of code that might be parsed separately.
         """
         # We provide a common interface for any grammar that allows positional elements.
-        # If *any* for the elements are a string and not a grammar, then this is a shortcut
-        # to the Ref.keyword grammar by default.
+        # If *any* for the elements are a string and not a grammar, then this is a
+        # shortcut to the Ref.keyword grammar by default.
         if self.allow_keyword_string_refs:
             self._elements = []
             for elem in args:
@@ -172,18 +171,13 @@ class BaseGrammar(Matchable):
         """
         raise NotImplementedError(
             f"{self.__class__.__name__} has no match function implemented"
-        )
+        )  # pragma: no cover
 
     @cached_method_for_parse_context
     def simple(self, parse_context: ParseContext) -> Optional[List[str]]:
         """Does this matcher support a lowercase hash matching route?"""
         return None
 
-    @staticmethod
-    def _iter_raw_segs(segments):
-        for segment in segments:
-            yield from segment.iter_raw_seg()
-
     @classmethod
     def _longest_trimmed_match(
         cls,
@@ -235,7 +229,8 @@ class BaseGrammar(Matchable):
                     best_match_length = res_match.matched_length
             # We could stash segments here, but given we might have some successful
             # matches here, we shouldn't, because they'll be mutated in the wrong way.
-            # Eventually there might be a performance gain from doing that sensibly here.
+            # Eventually there might be a performance gain from doing that sensibly
+            # here.
 
         # If we get here, then there wasn't a complete match. If we
         # has a best_match, return that.
@@ -280,11 +275,11 @@ class BaseGrammar(Matchable):
 
         # Do some type munging
         matchers = list(matchers)
-        if isinstance(segments, BaseSegment):
+        if isinstance(segments, BaseSegment):  # pragma: no cover TODO?
             segments = [segments]
 
         # Have we been passed an empty list?
-        if len(segments) == 0:
+        if len(segments) == 0:  # pragma: no cover TODO?
             return ((), MatchResult.from_empty(), None)
 
         # Here we enable a performance optimisation. Most of the time in this cycle
@@ -350,8 +345,8 @@ class BaseGrammar(Matchable):
                 # Here we do the actual transform to the new segment.
                 match = queued_matcher.match(segments[queued_buff_pos:], parse_context)
                 if not match:
-                    # We've had something match in simple matching, but then later excluded.
-                    # Log but then move on to the next item on the list.
+                    # We've had something match in simple matching, but then later
+                    # excluded. Log but then move on to the next item on the list.
                     parse_match_logging(
                         cls.__name__,
                         "_look_ahead_match",
@@ -468,7 +463,7 @@ class BaseGrammar(Matchable):
         """
         # Type munging
         matchers = list(matchers)
-        if isinstance(segments, BaseSegment):
+        if isinstance(segments, BaseSegment):  # pragma: no cover TODO?
             segments = [segments]
 
         # Have we been passed an empty list?
@@ -518,17 +513,20 @@ class BaseGrammar(Matchable):
 
                     if match:
                         # NB: We can only consider this as a nested bracket if the start
-                        # and end tokens are not the same. If a matcher is both a start and
-                        # end token we cannot deepen the bracket stack. In general, quoted
-                        # strings are a typical example where the start and end tokens are
-                        # the same. Currently, though, quoted strings are handled elsewhere
-                        # in the parser, and there are no cases where *this* code has to
-                        # handle identical start and end brackets. For now, consider this
-                        # a small, speculative investment in a possible future requirement.
+                        # and end tokens are not the same. If a matcher is both a start
+                        # and end token we cannot deepen the bracket stack. In general,
+                        # quoted strings are a typical example where the start and end
+                        # tokens are the same. Currently, though, quoted strings are
+                        # handled elsewhere in the parser, and there are no cases where
+                        # *this* code has to handle identical start and end brackets.
+                        # For now, consider this a small, speculative investment in a
+                        # possible future requirement.
                         if matcher in start_brackets and matcher not in end_brackets:
-                            # Add any segments leading up to this to the previous bracket.
+                            # Add any segments leading up to this to the previous
+                            # bracket.
                             bracket_stack[-1].segments += pre
-                            # Add a bracket to the stack and add the matches from the segment.
+                            # Add a bracket to the stack and add the matches from the
+                            # segment.
                             bracket_stack.append(
                                 BracketInfo(
                                     bracket=match.matched_segments[0],
@@ -572,8 +570,8 @@ class BaseGrammar(Matchable):
                                     new_segments = bracket_stack[-1].segments
                                 # Remove the bracket set from the stack
                                 bracket_stack.pop()
-                                # If we're still in a bracket, add the new segments to that bracket
-                                # Otherwise add them to the buffer
+                                # If we're still in a bracket, add the new segments to
+                                # that bracket, otherwise add them to the buffer
                                 if bracket_stack:
                                     bracket_stack[-1].segments += new_segments
                                 else:
@@ -590,9 +588,9 @@ class BaseGrammar(Matchable):
                                     segment=match.matched_segments[0],
                                 )
 
-                        else:
+                        else:  # pragma: no cover
                             raise RuntimeError("I don't know how we get here?!")
-                    else:
+                    else:  # pragma: no cover
                         # No match, we're in a bracket stack. Error.
                         raise SQLParseError(
                             "Couldn't find closing bracket for opening bracket.",
@@ -645,32 +643,34 @@ class BaseGrammar(Matchable):
                                 got=matcher,
                             )
                             # From here we'll drop out to the happy unmatched exit.
-                        else:
+                        else:  # pragma: no cover
                             # This shouldn't happen!?
                             raise NotImplementedError(
-                                "This shouldn't happen. Panic in _bracket_sensitive_look_ahead_match."
+                                "This shouldn't happen. Panic in "
+                                "_bracket_sensitive_look_ahead_match."
                             )
                     # Not in a bracket stack, but no match.
                     # From here we'll drop out to the happy unmatched exit.
             else:
                 # No we're at the end:
                 # Now check have we closed all our brackets?
-                if bracket_stack:
+                if bracket_stack:  # pragma: no cover
                     # No we haven't.
                     raise SQLParseError(
-                        f"Couldn't find closing bracket for opened brackets: `{bracket_stack}`.",
+                        "Couldn't find closing bracket for opened brackets: "
+                        f"`{bracket_stack}`.",
                         segment=bracket_stack[-1].bracket,
                     )
 
             # This is the happy unmatched path. This occurs when:
             # - We reached the end with no open brackets.
             # - No match while outside a bracket stack.
-            # - We found an unexpected end bracket before matching something interesting.
-            # We return with the mutated segments so we can
-            # reuse any bracket matching.
+            # - We found an unexpected end bracket before matching something
+            # interesting. We return with the mutated segments so we can reuse any
+            # bracket matching.
             return ((), MatchResult.from_unmatched(pre_seg_buff + seg_buff), None)
 
-    def __str__(self):
+    def __str__(self):  # pragma: no cover TODO?
         return repr(self)
 
     def __repr__(self):
@@ -734,14 +734,14 @@ class BaseGrammar(Matchable):
             for elem in self._elements
         ]
         if insert:
-            if at is not None and before is not None:
+            if at is not None and before is not None:  # pragma: no cover
                 raise ValueError(
                     "Cannot specify `at` and `before` in BaseGrammar.copy()."
                 )
             if before is not None:
                 try:
                     idx = new_elems.index(before)
-                except ValueError:
+                except ValueError:  # pragma: no cover
                     raise ValueError(
                         "Could not insert {} in copy of {}. {} not Found.".format(
                             insert, self, before
@@ -756,7 +756,7 @@ class BaseGrammar(Matchable):
             for elem in remove:
                 try:
                     new_elems.remove(elem)
-                except ValueError:
+                except ValueError:  # pragma: no cover
                     raise ValueError(
                         "Could not remove {} from copy of {}. Not Found.".format(
                             elem, self
@@ -791,11 +791,10 @@ class Ref(BaseGrammar):
         if len(self._elements) == 1:
             # We're good on length. Get the name of the reference
             return self._elements[0]
-        else:
+        else:  # pragma: no cover
             raise ValueError(
-                "Ref grammar can only deal with precisely one element for now. Instead found {!r}".format(
-                    self._elements
-                )
+                "Ref grammar can only deal with precisely one element for now. Instead "
+                "found {!r}".format(self._elements)
             )
 
     def _get_elem(self, dialect):
@@ -803,7 +802,7 @@ class Ref(BaseGrammar):
         if dialect:
             # Use the dialect to retrieve the grammar it refers to.
             return dialect.ref(self._get_ref())
-        else:
+        else:  # pragma: no cover
             raise ReferenceError("No Dialect has been provided to Ref grammar!")
 
     def __repr__(self):
@@ -821,11 +820,11 @@ class Ref(BaseGrammar):
         on the underlying class.
 
         The match element of Ref, also implements the caching
-        using the parse_context `blacklist` methods.
+        using the parse_context `denylist` methods.
         """
         elem = self._get_elem(dialect=parse_context.dialect)
 
-        if not elem:
+        if not elem:  # pragma: no cover
             raise ValueError(f"Null Element returned! _elements: {self._elements!r}")
 
         # First check against the efficiency Cache.
@@ -834,7 +833,7 @@ class Ref(BaseGrammar):
         # objects.
         seg_tuple = (id(seg) for seg in segments)
         self_name = self._get_ref()
-        if parse_context.blacklist.check(self_name, seg_tuple):
+        if parse_context.denylist.check(self_name, seg_tuple):  # pragma: no cover TODO?
             # This has been tried before.
             parse_match_logging(
                 self.__class__.__name__,
@@ -851,7 +850,7 @@ class Ref(BaseGrammar):
         with parse_context.matching_segment(self._get_ref()) as ctx:
             resp = elem.match(segments=segments, parse_context=ctx)
         if not resp:
-            parse_context.blacklist.mark(self_name, seg_tuple)
+            parse_context.denylist.mark(self_name, seg_tuple)
         return resp
 
     @classmethod
