@@ -2,10 +2,11 @@
 
 from typing import Tuple, List, Any, Iterator, TYPE_CHECKING
 
+from sqlfluff.core.errors import SQLParseError
 from sqlfluff.core.string_helpers import curtail_string
 
 if TYPE_CHECKING:
-    from sqlfluff.core.parser.segments import BaseSegment
+    from sqlfluff.core.parser.segments import BaseSegment  # pragma: no cover
 
 
 def join_segments_raw(segments: Tuple["BaseSegment", ...]) -> str:
@@ -26,11 +27,11 @@ def check_still_complete(
     """Check that the segments in are the same as the segments out."""
     initial_str = join_segments_raw(segments_in)
     current_str = join_segments_raw(matched_segments + unmatched_segments)
-    if initial_str != current_str:
-        raise RuntimeError(
-            "Dropped elements in sequence matching! {!r} != {!r}".format(
-                initial_str, current_str
-            )
+
+    if initial_str != current_str:  # pragma: no cover
+        raise SQLParseError(
+            f"Could not parse: {current_str}",
+            segment=unmatched_segments[0],
         )
     return True
 
@@ -79,7 +80,6 @@ def iter_indices(seq: List, val: Any) -> Iterator[int]:
         >>> print([i for i in iter_indices([1, 0, 2, 3, 2], 2)])
         [2, 4]
     """
-    idx = 0
-    while val in seq[idx:]:
-        idx = seq.index(val, idx) + 1
-        yield idx - 1
+    for idx, el in enumerate(seq):
+        if el == val:
+            yield idx
