@@ -9,10 +9,13 @@ Here we define:
 """
 
 from io import StringIO
-from cached_property import cached_property
 from typing import Any, Callable, Optional, List, Tuple, NamedTuple, Iterator
 import logging
 
+from tqdm import tqdm
+
+from sqlfluff.core.cached_property import cached_property
+from sqlfluff.core.config import progress_bar_configuration
 from sqlfluff.core.string_helpers import (
     frame_msg,
     curtail_string,
@@ -53,13 +56,13 @@ class BaseSegment:
     subclasses rather than directly here.
 
     For clarity, the `BaseSegment` is mostly centered around a segment which contains
-    other subsegments. For segments which don't have *children*, refer to the `RawSegment`
-    class (which still inherits from this one).
+    other subsegments. For segments which don't have *children*, refer to the
+    `RawSegment` class (which still inherits from this one).
 
     Segments are used both as instances to hold chunks of text, but also as classes
-    themselves where they function a lot like grammars, and return instances of themselves
-    when they match. The many classmethods in this class are usually to serve their
-    purpose as a matcher.
+    themselves where they function a lot like grammars, and return instances of
+    themselves when they match. The many classmethods in this class are usually to serve
+    their purpose as a matcher.
     """
 
     # `type` should be the *category* of this kind of segment
@@ -67,7 +70,7 @@ class BaseSegment:
     parse_grammar: Optional[Matchable] = None
     # We define the type here but no value. Subclasses must provide a value.
     match_grammar: Matchable
-    comment_seperate = False
+    comment_separate = False
     optional = False  # NB: See the sequence grammar for details
     _name: Optional[str] = None
     is_meta = False
@@ -79,26 +82,31 @@ class BaseSegment:
     # What other kwargs need to be copied when applying fixes.
     additional_kwargs: List[str] = []
 
-    def __init__(self, segments, pos_marker=None, name: Optional[str] = None):
+    def __init__(
+        self,
+        segments,
+        pos_marker=None,
+        name: Optional[str] = None,
+    ):
         # A cache variable for expandable
-        self._is_expandable = None
+        self._is_expandable: Optional[bool] = None
         # Surrogate name option.
         self._surrogate_name = name
-        if len(segments) == 0:
+
+        if len(segments) == 0:  # pragma: no cover
             raise RuntimeError(
-                "Setting {} with a zero length segment set. This shouldn't happen.".format(
-                    self.__class__
-                )
+                "Setting {} with a zero length segment set. This shouldn't "
+                "happen.".format(self.__class__)
             )
 
-        if hasattr(segments, "matched_segments"):
+        if hasattr(segments, "matched_segments"):  # pragma: no cover TODO?
             # Safely extract segments from a match
             self.segments = segments.matched_segments
         elif isinstance(segments, tuple):
             self.segments = segments
         elif isinstance(segments, list):
             self.segments = tuple(segments)
-        else:
+        else:  # pragma: no cover
             raise TypeError(f"Unexpected type passed to BaseSegment: {type(segments)}")
 
         if not pos_marker:
@@ -107,12 +115,25 @@ class BaseSegment:
                 pos_marker = PositionMarker.from_child_markers(
                     *(seg.pos_marker for seg in segments)
                 )
-            else:
+            else:  # pragma: no cover
                 raise TypeError(
                     f"Unexpected type passed to BaseSegment: {type(segments)}"
                 )
         self.pos_marker: PositionMarker = pos_marker
 
+        self._recalculate_caches()
+
+    def __setattr__(self, key, value):
+
+        try:
+            if key == "segments":
+                self._recalculate_caches()
+
+        except (AttributeError, KeyError):  # pragma: no cover
+            pass
+
+        super().__setattr__(key, value)
+
     def __eq__(self, other):
         # NB: this should also work for RawSegment
         return (
@@ -131,6 +152,15 @@ class BaseSegment:
             )
         )
 
+    def __hash__(self):
+        return hash(
+            (
+                self.__class__.__name__,
+                self.raw,
+                self.pos_marker.source_position() if self.pos_marker else None,
+            )
+        )
+
     def __repr__(self):
         return f"<{self.__class__.__name__}: ({self.pos_marker})>"
 
@@ -142,14 +172,14 @@ class BaseSegment:
         return [seg for seg in self.segments if seg.is_type("comment")]
 
     @property
-    def _non_comments(self):
+    def _non_comments(self):  # pragma: no cover TODO?
         """Returns only the non-comment elements of this segment."""
         return [seg for seg in self.segments if not seg.is_type("comment")]
 
     # ################ PUBLIC PROPERTIES
 
     @property
-    def name(self):
+    def name(self) -> str:
         """The name of this segment.
 
         The reason for three routes for names is that some subclasses
@@ -165,7 +195,7 @@ class BaseSegment:
         return self._surrogate_name or self._name or self.__class__.__name__
 
     @property
-    def is_expandable(self):
+    def is_expandable(self) -> bool:
         """Return true if it is meaningful to call `expand` on this segment.
 
         We need to do this recursively because even if *this* segment doesn't
@@ -186,39 +216,74 @@ class BaseSegment:
             return False
 
     @cached_property
-    def is_code(self):
+    def is_code(self) -> bool:
         """Return True if this segment contains any code."""
         return any(seg.is_code for seg in self.segments)
 
     @cached_property
-    def is_comment(self):
+    def is_comment(self) -> bool:  # pragma: no cover TODO?
         """Return True if this is entirely made of comments."""
         return all(seg.is_comment for seg in self.segments)
 
     @cached_property
-    def is_whitespace(self):
+    def is_whitespace(self) -> bool:
         """Return True if this segment is entirely whitespace."""
         return all(seg.is_whitespace for seg in self.segments)
 
     @cached_property
-    def raw(self):
+    def raw(self) -> str:
         """Make a string from the segments of this segment."""
-        return self._reconstruct()
+        return "".join(seg.raw for seg in self.segments)
 
     @cached_property
-    def raw_upper(self):
+    def raw_upper(self) -> str:
         """Make an uppercase string from the segments of this segment."""
-        return self._reconstruct().upper()
+        return self.raw.upper()
 
     @cached_property
-    def matched_length(self):
+    def matched_length(self) -> int:
         """Return the length of the segment in characters."""
         return sum(seg.matched_length for seg in self.segments)
 
+    @cached_property
+    def raw_segments(self) -> List["BaseSegment"]:
+        """Returns a list of raw segments in this segment."""
+        return self.get_raw_segments()
+
+    @cached_property
+    def raw_segments_upper(self) -> Optional[str]:
+        """Returns the first non-whitespace subsegment of this segment."""
+        for seg in self.raw_segments:
+            if seg.raw_upper.strip():
+                return seg.raw_upper
+        return None
+        # return [seg.raw_upper for seg in self.raw_segments]
+
+    @cached_property
+    def is_templated(self) -> bool:
+        """Returns True if the segment includes any templated code.
+
+        This is a simple, very efficient check that doesn't require looking up
+        the RawFileSlices for the segment.
+
+        NOTE: A segment returning a True result may still have some literal
+        code as well (i.e. a mixture of literal and templated).
+        """
+        # We check two things:
+        # * Source slice not empty: If it's empty, this means it doesn't appear
+        #   in the source, e.g. because it is new code generated by a lint fix.
+        #   Return False for these.
+        # * Source string doesn't match raw segment contents. This can only
+        #   happen if templating is involved.
+        return (
+            self.pos_marker.source_slice.start != self.pos_marker.source_slice.stop
+            and self.raw != self.pos_marker.source_str()
+        )
+
     # ################ STATIC METHODS
 
     @staticmethod
-    def segs_to_tuple(segs, **kwargs):
+    def segs_to_tuple(segs, **kwargs):  # pragma: no cover TODO?
         """Return a tuple structure from an iterable of segments."""
         return tuple(seg.to_tuple(**kwargs) for seg in segs)
 
@@ -230,11 +295,26 @@ class BaseSegment:
         """
         return ""
 
-    @staticmethod
-    def expand(segments, parse_context):
+    @classmethod
+    def expand(cls, segments, parse_context):
         """Expand the list of child segments using their `parse` methods."""
         segs = ()
-        for stmt in segments:
+
+        # Renders progress bar only for `BaseFileSegments`.
+        disable_progress_bar = (
+            not issubclass(cls, BaseFileSegment)
+            or progress_bar_configuration.disable_progress_bar
+        )
+
+        progressbar_segments = tqdm(
+            segments,
+            desc="parsing",
+            miniters=30,
+            leave=False,
+            disable=disable_progress_bar,
+        )
+
+        for stmt in progressbar_segments:
             try:
                 if not stmt.is_expandable:
                     parse_context.logger.info(
@@ -244,17 +324,17 @@ class BaseSegment:
                     )
                     segs += (stmt,)
                     continue
-            except Exception as err:
+            except Exception as err:  # pragma: no cover
                 parse_context.logger.error(
-                    "%s has no attribute `is_expandable`. This segment appears poorly constructed.",
+                    "%s has no attribute `is_expandable`. This segment appears poorly "
+                    "constructed.",
                     stmt,
                 )
                 raise err
-            if not hasattr(stmt, "parse"):
+            if not hasattr(stmt, "parse"):  # pragma: no cover
                 raise ValueError(
-                    "{} has no method `parse`. This segment appears poorly constructed.".format(
-                        stmt
-                    )
+                    "{} has no method `parse`. This segment appears poorly "
+                    "constructed.".format(stmt)
                 )
             parse_depth_msg = "Parse Depth {}. Expanding: {}: {!r}".format(
                 parse_context.parse_depth,
@@ -268,6 +348,7 @@ class BaseSegment:
             else:
                 # We might get back an iterable of segments
                 segs += tuple(res)
+
         # Basic Validation
         check_still_complete(segments, segs, ())
         return segs
@@ -301,7 +382,7 @@ class BaseSegment:
                     line_no = fwd_seg.pos_marker.working_line_no
                     line_pos = fwd_seg.pos_marker.working_line_pos
                     break
-            else:
+            else:  # pragma: no cover
                 linter_logger.warning("SEG: %r, POS: %r", segments, parent_pos)
                 raise ValueError("Unable to find working position.")
 
@@ -324,7 +405,7 @@ class BaseSegment:
                                 idx
                             ].pos_marker = fwd_seg.pos_marker.start_point_marker()
                             break
-                    else:
+                    else:  # pragma: no cover
                         raise ValueError("Unable to position new segment")
 
             # Update the working position.
@@ -356,7 +437,7 @@ class BaseSegment:
         """
         if cls.match_grammar:
             return cls.match_grammar.simple(parse_context=parse_context)
-        else:
+        else:  # pragma: no cover TODO?
             # Other segments will either override this method, or aren't
             # simple.
             return None
@@ -389,7 +470,9 @@ class BaseSegment:
     @classmethod
     def structural_simplify(cls, elem):
         """Simplify the structure recursively so it serializes nicely in json/yaml."""
-        if isinstance(elem, tuple):
+        if len(elem) == 0:
+            return None
+        elif isinstance(elem, tuple):
             # Does this look like an element?
             if len(elem) == 2 and isinstance(elem[0], str):
                 # This looks like a single element, make a dict
@@ -452,8 +535,9 @@ class BaseSegment:
             with parse_context.deeper_match() as ctx:
                 m = cls.match_grammar.match(segments=segments, parse_context=ctx)
 
-            # Calling unify here, allows the MatchResult class to do all the type checking.
-            if not isinstance(m, MatchResult):
+            # Calling unify here, allows the MatchResult class to do all the type
+            # checking.
+            if not isinstance(m, MatchResult):  # pragma: no cover
                 raise TypeError(
                     "[PD:{} MD:{}] {}.match. Result is {}, not a MatchResult!".format(
                         parse_context.parse_depth,
@@ -469,16 +553,26 @@ class BaseSegment:
                 )
             else:
                 return MatchResult.from_unmatched(segments)
-        else:
+        else:  # pragma: no cover
             raise NotImplementedError(
                 f"{cls.__name__} has no match function implemented"
             )
 
     # ################ PRIVATE INSTANCE METHODS
 
-    def _reconstruct(self):
-        """Make a string from the segments of this segment."""
-        return "".join(seg.raw for seg in self.segments)
+    def _recalculate_caches(self):
+
+        for key in [
+            "is_code",
+            "is_comment",
+            "is_whitespace",
+            "raw",
+            "raw_upper",
+            "matched_length",
+            "raw_segments",
+            "raw_segments_upper",
+        ]:
+            self.__dict__.pop(key, None)
 
     def _preface(self, ident, tabsize):
         """Returns the preamble to any logging."""
@@ -505,14 +599,24 @@ class BaseSegment:
         """Is this segment (or its parent) of the given type."""
         return self.class_is_type(*seg_type)
 
+    def get_name(self):
+        """Returns the name of this segment as a string."""
+        return self.name
+
+    def is_name(self, *seg_name):
+        """Is this segment of the given name."""
+        return any(s == self.name for s in seg_name)
+
     def invalidate_caches(self):
         """Invalidate the cached properties.
 
         This should be called whenever the segments within this
         segment is mutated.
         """
-        for key in ["is_code", "is_comment", "raw", "raw_upper", "matched_length"]:
-            self.__dict__.pop(key, None)
+        for seg in self.segments:
+            seg.invalidate_caches()
+
+        self._recalculate_caches()
 
     def get_start_point_marker(self):
         """Get a point marker at the start of this segment."""
@@ -537,8 +641,8 @@ class BaseSegment:
         buff = StringIO()
         preface = self._preface(ident=ident, tabsize=tabsize)
         buff.write(preface + "\n")
-        if not code_only and self.comment_seperate and len(self._comments) > 0:
-            if self._comments:
+        if not code_only and self.comment_separate and len(self._comments) > 0:
+            if self._comments:  # pragma: no cover TODO?
                 buff.write((" " * ((ident + 1) * tabsize)) + "Comments:" + "\n")
                 for seg in self._comments:
                     buff.write(
@@ -548,7 +652,7 @@ class BaseSegment:
                             code_only=code_only,
                         )
                     )
-            if self._non_comments:
+            if self._non_comments:  # pragma: no cover TODO?
                 buff.write((" " * ((ident + 1) * tabsize)) + "Code:" + "\n")
                 for seg in self._non_comments:
                     buff.write(
@@ -560,7 +664,8 @@ class BaseSegment:
                     )
         else:
             for seg in self.segments:
-                # If we're in code_only, only show the code segments, otherwise always true
+                # If we're in code_only, only show the code segments, otherwise always
+                # true
                 if not code_only or seg.is_code:
                     buff.write(
                         seg.stringify(
@@ -613,17 +718,16 @@ class BaseSegment:
         """
         return self.structural_simplify(self.to_tuple(**kwargs))
 
-    def raw_list(self):
+    def raw_list(self):  # pragma: no cover TODO?
         """Return a list of raw elements, mostly for testing or searching."""
         buff = []
         for s in self.segments:
             buff += s.raw_list()
         return buff
 
-    def iter_raw_seg(self):
+    def get_raw_segments(self):
         """Iterate raw segments, mostly for searching."""
-        for s in self.segments:
-            yield from s.iter_raw_seg()
+        return [item for s in self.segments for item in s.raw_segments]
 
     def iter_segments(self, expanding=None, pass_through=False):
         """Iterate raw segments, optionally expanding some chldren."""
@@ -734,26 +838,31 @@ class BaseSegment:
             res = seg.path_to(other)
             if res:
                 return [self] + res
-        return None
+        return None  # pragma: no cover
 
-    def parse(self, parse_context=None, parse_grammar=None):
+    def parse(
+        self,
+        parse_context: ParseContext,
+        parse_grammar: Optional[Matchable] = None,
+    ) -> "BaseSegment":
         """Use the parse grammar to find subsegments within this segment.
 
         A large chunk of the logic around this can be found in the `expand` method.
 
-        Use the parse setting in the context for testing, mostly to check how deep to go.
-        True/False for yes or no, an integer allows a certain number of levels.
+        Use the parse setting in the context for testing, mostly to check how deep to
+        go. True/False for yes or no, an integer allows a certain number of levels.
 
         Optionally, this method allows a custom parse grammar to be
         provided which will override any existing parse grammar
         on the segment.
         """
-        # Clear the blacklist cache so avoid missteps
+        # Clear the denylist cache so avoid missteps
         if parse_context:
-            parse_context.blacklist.clear()
+            parse_context.denylist.clear()
 
-        # the parse_depth and recurse kwargs control how deep we will recurse for testing.
-        if not self.segments:
+        # the parse_depth and recurse kwargs control how deep we will recurse for
+        # testing.
+        if not self.segments:  # pragma: no cover TODO?
             # This means we're a root segment, just return an unmutated self
             return self
 
@@ -776,13 +885,17 @@ class BaseSegment:
             else:
                 pre_nc = ()
                 post_nc = ()
-                if (not segments[0].is_code) and (not segments[0].is_meta):
+                if (not segments[0].is_code) and (
+                    not segments[0].is_meta
+                ):  # pragma: no cover
                     raise ValueError(
                         "Segment {} starts with non code segment: {!r}.\n{!r}".format(
                             self, segments[0].raw, segments
                         )
                     )
-                if (not segments[-1].is_code) and (not segments[-1].is_meta):
+                if (not segments[-1].is_code) and (
+                    not segments[-1].is_meta
+                ):  # pragma: no cover
                     raise ValueError(
                         "Segment {} ends with non code segment: {!r}.\n{!r}".format(
                             self, segments[-1].raw, segments
@@ -793,7 +906,7 @@ class BaseSegment:
             with parse_context.matching_segment(self.__class__.__name__) as ctx:
                 m = parse_grammar.match(segments=segments, parse_context=ctx)
 
-            if not isinstance(m, MatchResult):
+            if not isinstance(m, MatchResult):  # pragma: no cover
                 raise TypeError(
                     "[PD:{}] {}.match. Result is {}, not a MatchResult!".format(
                         parse_context.parse_depth, self.__class__.__name__, type(m)
@@ -809,8 +922,8 @@ class BaseSegment:
                     self.segments = pre_nc + m.matched_segments + post_nc
                 else:
                     # Incomplete match.
-                    # For now this means the parsing has failed. Lets add the unmatched bit at the
-                    # end as something unparsable.
+                    # For now this means the parsing has failed. Lets add the unmatched
+                    # bit at the end as something unparsable.
                     # TODO: Do something more intelligent here.
                     self.segments = (
                         pre_nc
@@ -823,11 +936,13 @@ class BaseSegment:
                         )
                     )
             elif self.allow_empty and not segments:
-                # Very edge case, but some segments are allowed to be empty other than non-code
+                # Very edge case, but some segments are allowed to be empty other than
+                # non-code
                 self.segments = pre_nc + post_nc
             else:
                 # If there's no match at this stage, then it's unparsable. That's
-                # a problem at this stage so wrap it in an unparsable segment and carry on.
+                # a problem at this stage so wrap it in an unparsable segment and carry
+                # on.
                 self.segments = (
                     pre_nc
                     + (
@@ -844,13 +959,19 @@ class BaseSegment:
                 self.__class__.__name__, parse_context.recurse
             )
         )
-        parse_depth_msg = "###\n#\n# Beginning Parse Depth {}: {}\n#\n###\nInitial Structure:\n{}".format(
-            parse_context.parse_depth + 1, self.__class__.__name__, self.stringify()
+        parse_depth_msg = (
+            "###\n#\n# Beginning Parse Depth {}: {}\n#\n###\nInitial Structure:\n"
+            "{}".format(
+                parse_context.parse_depth + 1, self.__class__.__name__, self.stringify()
+            )
         )
         if parse_context.may_recurse():
             parse_context.logger.debug(parse_depth_msg)
             with parse_context.deeper_parse() as ctx:
-                self.segments = self.expand(self.segments, parse_context=ctx)
+                self.segments = self.expand(
+                    self.segments,
+                    parse_context=ctx,
+                )
 
         return self
 
@@ -892,28 +1013,42 @@ class BaseSegment:
                             if f.edit_type == "delete":
                                 # We're just getting rid of this segment.
                                 seg = None
-                            elif f.edit_type in ("edit", "create"):
-                                # We're doing a replacement (it could be a single segment or an iterable)
+                            elif f.edit_type in (
+                                "replace",
+                                "create_before",
+                                "create_after",
+                            ):
+                                if f.edit_type == "create_after":
+                                    # in the case of a creation after, also add this
+                                    # segment before the edit.
+                                    seg_buffer.append(seg)
+
+                                # We're doing a replacement (it could be a single
+                                # segment or an iterable)
                                 if isinstance(f.edit, BaseSegment):
-                                    seg_buffer.append(f.edit)
+                                    seg_buffer.append(f.edit)  # pragma: no cover TODO?
                                 else:
                                     for s in f.edit:
                                         seg_buffer.append(s)
 
-                                if f.edit_type == "create":
-                                    # in the case of a creation, also add this segment on the end
+                                if f.edit_type == "create_before":
+                                    # in the case of a creation before, also add this
+                                    # segment on the end
                                     seg_buffer.append(seg)
-                            else:
+
+                            else:  # pragma: no cover
                                 raise ValueError(
                                     "Unexpected edit_type: {!r} in {!r}".format(
                                         f.edit_type, f
                                     )
                                 )
-                            # We've applied a fix here. Move on, this also consumes the fix
+                            # We've applied a fix here. Move on, this also consumes the
+                            # fix
                             # TODO: Maybe deal with overlapping fixes later.
                             break
                         else:
-                            # We've not used the fix so we should keep it in the list for later.
+                            # We've not used the fix so we should keep it in the list
+                            # for later.
                             unused_fixes.append(f)
                     else:
                         seg_buffer.append(seg)
@@ -960,6 +1095,13 @@ class BaseSegment:
             return
 
         # If we're here, the segment doesn't match the original.
+        linter_logger.debug(
+            "%s at %s: Original: [%r] Fixed: [%r]",
+            type(self).__name__,
+            self.pos_marker.templated_slice,
+            templated_str[self.pos_marker.templated_slice],
+            self.raw,
+        )
 
         # If it's all literal, then we don't need to recurse.
         if self.pos_marker.is_literal():
@@ -969,10 +1111,10 @@ class BaseSegment:
             )
         # Can we go deeper?
         elif not self.segments:
-            # It's not literal, but it's also a raw segment. If were going
+            # It's not literal, but it's also a raw segment. If we're going
             # to yield a change, we would have done it from the parent, so
             # we just abort from here.
-            return
+            return  # pragma: no cover TODO?
         else:
             # This segment isn't a literal, but has changed, we need to go deeper.
 
@@ -982,7 +1124,8 @@ class BaseSegment:
             for seg_idx, segment in enumerate(self.segments):
 
                 # First check for insertions.
-                # We know it's an insertion if it has length but not in the templated file.
+                # We know it's an insertion if it has length but not in the templated
+                # file.
                 if segment.raw and segment.pos_marker.is_point():
                     # Add it to the insertion buffer if it has length:
                     if segment.raw:
@@ -994,13 +1137,15 @@ class BaseSegment:
                         )
                     continue
 
-                # If we get here, then we know it's an original.
-                # Check for deletions at the before this segment (vs the TEMPLATED).
+                # If we get here, then we know it's an original. Check for deletions at
+                # the point before this segment (vs the TEMPLATED).
                 start_diff = segment.pos_marker.templated_slice.start - templated_idx
 
-                # Check to see whether there's a discontinuity before the current segment
+                # Check to see whether there's a discontinuity before the current
+                # segment
                 if start_diff > 0 or insert_buff:
-                    # If we have an insert buffer, then it's an edit, otherwise a deletion.
+                    # If we have an insert buffer, then it's an edit, otherwise a
+                    # deletion.
                     yield FixPatch(
                         slice(
                             segment.pos_marker.templated_slice.start
@@ -1032,6 +1177,10 @@ class BaseSegment:
                     patch_category="end_point",
                 )
 
+    def edit(self, raw):
+        """Stub."""
+        raise NotImplementedError()
+
 
 class BracketedSegment(BaseSegment):
     """A segment containing a bracketed expression."""
@@ -1051,7 +1200,7 @@ class BracketedSegment(BaseSegment):
         **kwargs,
     ):
         """Stash the bracket segments for later."""
-        if not start_bracket or not end_bracket:
+        if not start_bracket or not end_bracket:  # pragma: no cover
             raise ValueError(
                 "Attempted to construct Bracketed segment without specifying brackets."
             )
@@ -1089,7 +1238,7 @@ class UnparsableSegment(BaseSegment):
 
     type = "unparsable"
     # From here down, comments are printed separately.
-    comment_seperate = True
+    comment_separate = True
     _expected = ""
 
     def __init__(self, *args, expected="", **kwargs):
@@ -1109,3 +1258,40 @@ class UnparsableSegment(BaseSegment):
         As this is an unparsable, it should yield itself.
         """
         yield self
+
+
+class BaseFileSegment(BaseSegment):
+    """A segment representing a whole file or script.
+
+    This is also the default "root" segment of the dialect,
+    and so is usually instantiated directly. It therefore
+    has no match_grammar.
+    """
+
+    type = "file"
+    # The file segment is the only one which can start or end with non-code
+    can_start_end_non_code = True
+    # A file can be empty!
+    allow_empty = True
+
+    def __init__(
+        self,
+        segments,
+        pos_marker=None,
+        name: Optional[str] = None,
+        fname: Optional[str] = None,
+    ):
+        self._file_path = fname
+        super().__init__(segments, pos_marker=pos_marker, name=name)
+
+    @property
+    def file_path(self):
+        """File path of a parsed SQL file."""
+        return self._file_path
+
+    def get_table_references(self):
+        """Use parsed tree to extract table references."""
+        references = set()
+        for stmt in self.get_children("statement"):
+            references |= stmt.get_table_references()
+        return references
