@@ -14,73 +14,86 @@ STANDARD_CONFIG_INFO_DICT = {
         "validation": range(100),
         "definition": (
             "The number of spaces to consider equal to one tab. "
-            "Used in the fixing step of this rule"
+            "Used in the fixing step of this rule."
         ),
     },
     "max_line_length": {
         "validation": range(1000),
         "definition": (
-            "The maximum length of a line to allow without " "raising a violation"
+            "The maximum length of a line to allow without raising a violation."
         ),
     },
     "indent_unit": {
         "validation": ["space", "tab"],
-        "definition": "Whether to use tabs or spaces to add new indents",
+        "definition": "Whether to use tabs or spaces to add new indents.",
     },
     "comma_style": {
         "validation": ["leading", "trailing"],
-        "definition": "The comma style to to enforce",
+        "definition": "The comma style to enforce.",
     },
     "allow_scalar": {
         "validation": [True, False],
         "definition": (
             "Whether or not to allow a single element in the "
-            " select clause to be without an alias"
+            " select clause to be without an alias."
         ),
     },
     "single_table_references": {
         "validation": ["consistent", "qualified", "unqualified"],
-        "definition": "The expectation for references in single-table select",
+        "definition": "The expectation for references in single-table select.",
+    },
+    "force_enable": {
+        "validation": [True, False],
+        "definition": (
+            "Run this rule even for dialects where this rule is disabled by default."
+        ),
     },
     "unquoted_identifiers_policy": {
         "validation": ["all", "aliases", "column_aliases"],
-        "definition": "Types of unquoted identifiers to flag violations for",
+        "definition": "Types of unquoted identifiers to flag violations for.",
+    },
+    "quoted_identifiers_policy": {
+        "validation": ["all", "aliases", "column_aliases", "none"],
+        "definition": "Types of quoted identifiers to flag violations for.",
     },
     "capitalisation_policy": {
         "validation": ["consistent", "upper", "lower", "capitalise"],
-        "definition": "The capitalisation policy to enforce",
+        "definition": "The capitalisation policy to enforce.",
     },
     "extended_capitalisation_policy": {
         "validation": ["consistent", "upper", "lower", "pascal", "capitalise"],
         "definition": (
             "The capitalisation policy to enforce, extended with PascalCase. "
-            "This is separate from capitalisation_policy as it should not be "
+            "This is separate from ``capitalisation_policy`` as it should not be "
             "applied to keywords."
         ),
     },
-    "lint_templated_tokens": {
-        "validation": [True, False],
-        "definition": (
-            "Should lines starting with a templating placeholder"
-            " such as `{{blah}}` have their indentation linted"
-        ),
-    },
     "select_clause_trailing_comma": {
         "validation": ["forbid", "require"],
         "definition": (
-            "Should trailing commas within select clauses be required or forbidden"
+            "Should trailing commas within select clauses be required or forbidden?"
         ),
     },
     "ignore_comment_lines": {
         "validation": [True, False],
         "definition": (
             "Should lines that contain only whitespace and comments"
-            " be ignored when linting line lengths"
+            " be ignored when linting line lengths?"
         ),
     },
+    "ignore_comment_clauses": {
+        "validation": [True, False],
+        "definition": (
+            "Should comment clauses (e.g. column comments) be ignored"
+            " when linting line lengths?"
+        ),
+    },
+    "ignore_words": {
+        "definition": ("Comma separated list of words to ignore from rule"),
+    },
     "forbid_subquery_in": {
         "validation": ["join", "from", "both"],
-        "definition": "Which clauses should be linted for subqueries",
+        "definition": "Which clauses should be linted for subqueries?",
     },
     "prefer_count_1": {
         "validation": [True, False],
@@ -92,7 +105,57 @@ STANDARD_CONFIG_INFO_DICT = {
     },
     "operator_new_lines": {
         "validation": ["before", "after"],
-        "definition": ("Should operator be placed before or after newlines."),
+        "definition": ("Should operator be placed before or after newlines?"),
+    },
+    "aliasing": {
+        "validation": ["implicit", "explicit"],
+        "definition": (
+            "Should alias have an explict AS or is implicit aliasing required?"
+        ),
+    },
+    "multiline_newline": {
+        "validation": [True, False],
+        "definition": (
+            "Should semi-colons be placed on a new line after multi-line statements?"
+        ),
+    },
+    "require_final_semicolon": {
+        "validation": [True, False],
+        "definition": (
+            "Should final semi-colons be required? "
+            "(N.B. forcing trailing semi-colons is not recommended for dbt users "
+            "as it can cause issues when wrapping the query within other SQL queries)."
+        ),
+    },
+    "group_by_and_order_by_style": {
+        "validation": ["consistent", "implicit", "explicit"],
+        "definition": (
+            "The expectation for using explicit column name references "
+            "or implicit positional references."
+        ),
+    },
+    "allow_space_in_identifier": {
+        "validation": [True, False],
+        "definition": ("Should spaces in identifiers be allowed?"),
+    },
+    "additional_allowed_characters": {
+        "definition": (
+            "Optional list of extra allowed characters, "
+            "in addition to alphanumerics (A-Z, a-z, 0-9) and underscores."
+        ),
+    },
+    "prefer_quoted_identifiers": {
+        "validation": [True, False],
+        "definition": (
+            "If ``True``, requires every identifier to be quoted. "
+            "Defaults to ``False``."
+        ),
+    },
+    "blocked_words": {
+        "definition": (
+            "Optional, comma-separated list of blocked words which should not be used "
+            "in statements."
+        ),
     },
 }
 
