@@ -8,6 +8,7 @@ https://cloud.google.com/bigquery/docs/reference/standard-sql/lexical#string_and
 
 import itertools
 
+from sqlfluff.core.dialects import load_raw_dialect
 from sqlfluff.core.parser import (
     Anything,
     BaseSegment,
@@ -30,9 +31,13 @@ from sqlfluff.core.parser import (
     OptionallyBracketed,
     Indent,
     Dedent,
+    Matchable,
+)
+from sqlfluff.core.parser.segments.base import BracketedSegment
+from sqlfluff.dialects.dialect_bigquery_keywords import (
+    bigquery_reserved_keywords,
+    bigquery_unreserved_keywords,
 )
-
-from sqlfluff.core.dialects import load_raw_dialect
 
 ansi_dialect = load_raw_dialect("ansi")
 bigquery_dialect = ansi_dialect.copy_as("bigquery")
@@ -41,6 +46,8 @@ bigquery_dialect.insert_lexer_matchers(
     # JSON Operators: https://www.postgresql.org/docs/9.5/functions-json.html
     [
         StringLexer("right_arrow", "=>", CodeSegment),
+        StringLexer("question_mark", "?", CodeSegment),
+        RegexLexer("atsign_literal", r"@[a-zA-Z_][\w]*", CodeSegment),
     ],
     before="equals",
 )
@@ -48,9 +55,9 @@ bigquery_dialect.insert_lexer_matchers(
 bigquery_dialect.patch_lexer_matchers(
     [
         # Quoted literals can have r or b (case insensitive) prefixes, in any order, to
-        # indicate a raw/regex string or byte sequence, respectively.  Allow escaped quote
-        # characters inside strings by allowing \" with an optional even multiple of
-        # backslashes in front of it.
+        # indicate a raw/regex string or byte sequence, respectively.  Allow escaped
+        # quote characters inside strings by allowing \" with an optional even multiple
+        # of backslashes in front of it.
         # https://cloud.google.com/bigquery/docs/reference/standard-sql/lexical#string_and_bytes_literals
         # Triple quoted variant first, then single quoted
         RegexLexer(
@@ -77,6 +84,13 @@ bigquery_dialect.add(
         type="literal",
         trim_chars=('"',),
     ),
+    SingleQuotedLiteralSegment=NamedParser(
+        "single_quote",
+        CodeSegment,
+        name="quoted_literal",
+        type="literal",
+        trim_chars=("'",),
+    ),
     DoubleQuotedUDFBody=NamedParser(
         "double_quote",
         CodeSegment,
@@ -106,12 +120,45 @@ bigquery_dialect.add(
         delimiter=Ref("CommaSegment"),
         allow_trailing=True,
     ),
+    QuestionMarkSegment=StringParser(
+        "?", SymbolSegment, name="question_mark", type="question_mark"
+    ),
+    AtSignLiteralSegment=NamedParser(
+        "atsign_literal",
+        CodeSegment,
+        name="atsign_literal",
+        type="literal",
+        trim_chars=("@",),
+    ),
+    # Add a Full equivalent which also allow keywords
+    NakedIdentifierSegmentFull=RegexParser(
+        r"[A-Z0-9_]*[A-Z][A-Z0-9_]*",
+        CodeSegment,
+        name="naked_identifier_all",
+        type="identifier",
+    ),
+    SingleIdentifierGrammarFull=OneOf(
+        Ref("NakedIdentifierSegment"),
+        Ref("QuotedIdentifierSegment"),
+        Ref("NakedIdentifierSegmentFull"),
+    ),
+    DefaultDeclareOptionsGrammar=Sequence(
+        "DEFAULT",
+        OneOf(
+            Ref("LiteralGrammar"),
+            Bracketed(Ref("SelectStatementSegment")),
+            Ref("BareFunctionSegment"),
+            Ref("FunctionSegment"),
+            Ref("ArrayLiteralSegment"),
+            Ref("TupleSegment"),
+            Ref("BaseExpressionElementGrammar"),
+        ),
+    ),
 )
 
 
 bigquery_dialect.replace(
     FunctionContentsExpressionGrammar=OneOf(
-        Ref("DatetimeUnitSegment"),
         Sequence(
             Ref("ExpressionSegment"),
             Sequence(OneOf("IGNORE", "RESPECT"), "NULLS", optional=True),
@@ -121,20 +168,13 @@ bigquery_dialect.replace(
     SimpleArrayTypeGrammar=Sequence(
         "ARRAY",
         Bracketed(
-            Ref("DatatypeIdentifierSegment"),
+            Ref("DatatypeSegment"),
             bracket_type="angle",
             bracket_pairs_set="angle_bracket_pairs",
         ),
     ),
-    # BigQuery also supports the special "Struct" construct.
-    BaseExpressionElementGrammar=ansi_dialect.get_grammar(
-        "BaseExpressionElementGrammar"
-    ).copy(insert=[Ref("TypelessStructSegment")]),
-    FunctionContentsGrammar=ansi_dialect.get_grammar("FunctionContentsGrammar").copy(
-        insert=[Ref("TypelessStructSegment")],
-        before=Ref("ExpressionSegment"),
-    ),
-    # BigQuery allows underscore in parameter names, and also anything if quoted in backticks
+    # BigQuery allows underscore in parameter names, and also anything if quoted in
+    # backticks
     ParameterNameSegment=OneOf(
         RegexParser(
             r"[A-Z_][A-Z0-9_]*", CodeSegment, name="parameter", type="parameter"
@@ -142,22 +182,51 @@ bigquery_dialect.replace(
         RegexParser(r"`[^`]*`", CodeSegment, name="parameter", type="parameter"),
     ),
     DateTimeLiteralGrammar=Nothing(),
+    JoinLikeClauseGrammar=Sequence(
+        AnyNumberOf(
+            Ref("FromPivotExpressionSegment"),
+            Ref("FromUnpivotExpressionSegment"),
+            min_times=1,
+        ),
+        Ref("TableAliasExpressionSegment", optional=True),
+    ),
+    NaturalJoinKeywords=Nothing(),
 )
 
 
+# Set Keywords
+bigquery_dialect.sets("unreserved_keywords").clear()
+bigquery_dialect.sets("unreserved_keywords").update(
+    [n.strip().upper() for n in bigquery_unreserved_keywords.split("\n")]
+)
+
+bigquery_dialect.sets("reserved_keywords").clear()
+bigquery_dialect.sets("reserved_keywords").update(
+    [n.strip().upper() for n in bigquery_reserved_keywords.split("\n")]
+)
+
 # Add additional datetime units
 # https://cloud.google.com/bigquery/docs/reference/standard-sql/date_functions#extract
 bigquery_dialect.sets("datetime_units").update(
-    ["MICROSECOND", "DAYOFWEEK", "ISOWEEK", "ISOYEAR", "DATE", "DATETIME", "TIME"]
+    ["MICROSECOND", "DAYOFWEEK", "ISOWEEK", "ISOYEAR"]
+)
+
+bigquery_dialect.sets("date_part_function_name").clear()
+bigquery_dialect.sets("date_part_function_name").update(
+    [
+        "DATE_DIFF",
+        "DATE_TRUNC",
+        "DATETIME_DIFF",
+        "DATETIME_TRUNC",
+        "EXTRACT",
+        "LAST_DAY",
+        "TIME_DIFF",
+        "TIME_TRUNC",
+        "TIMESTAMP_DIFF",
+        "TIMESTAMP_TRUNC",
+    ]
 )
 
-# Unreserved Keywords
-bigquery_dialect.sets("unreserved_keywords").add("SYSTEM_TIME")
-bigquery_dialect.sets("unreserved_keywords").remove("FOR")
-bigquery_dialect.sets("unreserved_keywords").add("STRUCT")
-bigquery_dialect.sets("unreserved_keywords").add("ORDINAL")
-# Reserved Keywords
-bigquery_dialect.sets("reserved_keywords").add("FOR")
 
 # In BigQuery, UNNEST() returns a "value table".
 # https://cloud.google.com/bigquery/docs/reference/standard-sql/query-syntax#value_tables
@@ -182,8 +251,8 @@ class QualifyClauseSegment(BaseSegment):
     type = "qualify_clause"
     match_grammar = StartsWith(
         "QUALIFY",
-        terminator=OneOf("WINDOW"),
-        enforce_whitespace_preceeding_terminator=True,
+        terminator=OneOf("WINDOW", "ORDER", "LIMIT"),
+        enforce_whitespace_preceding_terminator=True,
     )
 
     parse_grammar = Sequence(
@@ -196,7 +265,7 @@ class QualifyClauseSegment(BaseSegment):
 
 @bigquery_dialect.segment(replace=True)
 class SelectStatementSegment(BaseSegment):
-    """Enhance`SELECT` statement to include QUALIFY."""
+    """Enhance `SELECT` statement to include QUALIFY."""
 
     type = "select_statement"
     match_grammar = ansi_dialect.get_segment(
@@ -212,19 +281,19 @@ class SelectStatementSegment(BaseSegment):
 
 
 @bigquery_dialect.segment(replace=True)
-class ArrayLiteralSegment(BaseSegment):
-    """Override array literal segment to add Typeless Struct."""
+class UnorderedSelectStatementSegment(BaseSegment):
+    """Enhance unordered `SELECT` statement to include QUALIFY."""
 
-    type = "array_literal"
-    match_grammar = Bracketed(
-        Delimited(
-            OneOf(
-                Ref("ExpressionSegment"),
-                Ref("TypelessStructSegment"),
-            ),
-            optional=True,
-        ),
-        bracket_type="square",
+    type = "select_statement"
+    match_grammar = ansi_dialect.get_segment(
+        "UnorderedSelectStatementSegment"
+    ).match_grammar.copy()
+
+    parse_grammar = ansi_dialect.get_segment(
+        "UnorderedSelectStatementSegment"
+    ).parse_grammar.copy(
+        insert=[Ref("QualifyClauseSegment", optional=True)],
+        before=Ref("OverlapsClauseSegment", optional=True),
     )
 
 
@@ -270,15 +339,24 @@ bigquery_dialect.replace(
         type="identifier",
         trim_chars=("`",),
     ),
-    # Add two elements to the ansi LiteralGrammar
+    # Add three elements to the ansi LiteralGrammar
     LiteralGrammar=ansi_dialect.get_grammar("LiteralGrammar").copy(
-        insert=[Ref("DoubleQuotedLiteralSegment"), Ref("LiteralCoercionSegment")]
+        insert=[
+            Ref("DoubleQuotedLiteralSegment"),
+            Ref("LiteralCoercionSegment"),
+            Ref("ParameterizedSegment"),
+        ]
     ),
     PostTableExpressionGrammar=Sequence(
         Sequence(
             "FOR", "SYSTEM_TIME", "AS", "OF", Ref("ExpressionSegment"), optional=True
         ),
-        Sequence("WITH", "OFFSET", "AS", Ref("SingleIdentifierGrammar"), optional=True),
+        Sequence(
+            "WITH",
+            "OFFSET",
+            Sequence("AS", Ref("SingleIdentifierGrammar"), optional=True),
+            optional=True,
+        ),
     ),
     FunctionNameIdentifierSegment=OneOf(
         # In BigQuery struct() has a special syntax, so we don't treat it as a function
@@ -310,16 +388,43 @@ class FunctionSegment(BaseSegment):
     """
 
     type = "function"
-    match_grammar = Sequence(
+    match_grammar = OneOf(
         Sequence(
-            Ref("FunctionNameSegment"),
-            Bracketed(
-                Ref(
-                    "FunctionContentsGrammar",
-                    # The brackets might be empty for some functions...
-                    optional=True,
-                    ephemeral_name="FunctionContentsGrammar",
-                )
+            # Treat functions which take date parts separately
+            # So those functions parse date parts as DatetimeUnitSegment
+            # rather than identifiers.
+            Sequence(
+                Ref("DatePartFunctionNameSegment"),
+                Bracketed(
+                    Delimited(
+                        Ref("DatetimeUnitSegment"),
+                        Ref(
+                            "FunctionContentsGrammar",
+                            ephemeral_name="FunctionContentsGrammar",
+                        ),
+                    ),
+                ),
+            ),
+        ),
+        Sequence(
+            Sequence(
+                AnyNumberOf(
+                    Ref("FunctionNameSegment"),
+                    max_times=1,
+                    min_times=1,
+                    exclude=OneOf(
+                        Ref("DatePartFunctionNameSegment"),
+                        Ref("ValuesClauseSegment"),
+                    ),
+                ),
+                Bracketed(
+                    Ref(
+                        "FunctionContentsGrammar",
+                        # The brackets might be empty for some functions...
+                        optional=True,
+                        ephemeral_name="FunctionContentsGrammar",
+                    )
+                ),
             ),
             # Functions returning ARRYS in BigQuery can have optional
             # OFFSET or ORDINAL clauses
@@ -353,8 +458,8 @@ class FunctionSegment(BaseSegment):
                 ),
                 optional=True,
             ),
+            Ref("PostFunctionGrammar", optional=True),
         ),
-        Ref("PostFunctionGrammar", optional=True),
     )
 
 
@@ -362,6 +467,7 @@ class FunctionSegment(BaseSegment):
 class FunctionDefinitionGrammar(BaseSegment):
     """This is the body of a `CREATE FUNCTION AS` statement."""
 
+    type = "function_definition"
     match_grammar = Sequence(
         AnyNumberOf(
             Sequence(
@@ -439,18 +545,13 @@ class ReplaceClauseSegment(BaseSegment):
     type = "select_replace_clause"
     match_grammar = Sequence(
         "REPLACE",
-        OneOf(
-            # Multiple replace in brackets
-            Bracketed(
-                Delimited(
-                    # Not *really* a select target element. It behaves exactly
-                    # the same way however.
-                    Ref("SelectClauseElementSegment"),
-                    delimiter=Ref("CommaSegment"),
-                )
-            ),
-            # Single replace not in brackets.
-            Ref("SelectClauseElementSegment"),
+        Bracketed(
+            Delimited(
+                # Not *really* a select target element. It behaves exactly
+                # the same way however.
+                Ref("SelectClauseElementSegment"),
+                delimiter=Ref("CommaSegment"),
+            )
         ),
     )
 
@@ -499,6 +600,7 @@ class FunctionParameterListGrammar(BaseSegment):
 
     # Function parameter list. Note that the only difference from the ANSI
     # grammar is that BigQuery provides overrides bracket_pairs_set.
+    type = "function_parameter_list"
     match_grammar = Bracketed(
         Delimited(
             Ref("FunctionParameterGrammar"),
@@ -509,7 +611,7 @@ class FunctionParameterListGrammar(BaseSegment):
     )
 
 
-@bigquery_dialect.segment()
+@bigquery_dialect.segment(replace=True)
 class TypelessStructSegment(BaseSegment):
     """Expression to construct a STRUCT with implicit types.
 
@@ -521,19 +623,26 @@ class TypelessStructSegment(BaseSegment):
         "STRUCT",
         Bracketed(
             Delimited(
-                AnyNumberOf(
-                    Sequence(
-                        Ref("BaseExpressionElementGrammar"),
-                        Ref("AliasExpressionSegment", optional=True),
-                    ),
+                Sequence(
+                    Ref("BaseExpressionElementGrammar"),
+                    Ref("AliasExpressionSegment", optional=True),
                 ),
-                delimiter=Ref("CommaSegment"),
             ),
-            optional=True,
         ),
     )
 
 
+@bigquery_dialect.segment()
+class TupleSegment(BaseSegment):
+    """Expression to construct a TUPLE.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-types#tuple_syntax
+    """
+
+    type = "tuple"
+    match_grammar = Bracketed(Delimited(Ref("BaseExpressionElementGrammar")))
+
+
 @bigquery_dialect.segment()
 class NamedArgumentSegment(BaseSegment):
     """Named argument to a function.
@@ -579,25 +688,92 @@ ObjectReferenceSegment = ansi_dialect.get_segment("ObjectReferenceSegment")
 
 @bigquery_dialect.segment(replace=True)
 class ColumnReferenceSegment(ObjectReferenceSegment):  # type: ignore
-    """A reference to column, field or alias."""
+    """A reference to column, field or alias.
+
+    We override this for BigQuery to allow keywords in structures
+    (using Full segments) and to properly return references for objects.
+
+    Ref: https://cloud.google.com/bigquery/docs/reference/standard-sql/lexical
+    "A reserved keyword must be a quoted identifier if it is a standalone
+    keyword or the first component of a path expression. It may be unquoted
+    as the second or later component of a path expression."
+    """
 
     type = "column_reference"
+    match_grammar: Matchable = Sequence(
+        Ref("SingleIdentifierGrammar"),
+        Sequence(
+            OneOf(Ref("DotSegment"), Sequence(Ref("DotSegment"), Ref("DotSegment"))),
+            Delimited(
+                Ref("SingleIdentifierGrammarFull"),
+                delimiter=OneOf(
+                    Ref("DotSegment"), Sequence(Ref("DotSegment"), Ref("DotSegment"))
+                ),
+                terminator=OneOf(
+                    "ON",
+                    "AS",
+                    "USING",
+                    Ref("CommaSegment"),
+                    Ref("CastOperatorSegment"),
+                    Ref("StartSquareBracketSegment"),
+                    Ref("StartBracketSegment"),
+                    Ref("BinaryOperatorGrammar"),
+                    Ref("ColonSegment"),
+                    Ref("DelimiterSegment"),
+                    BracketedSegment,
+                ),
+                allow_gaps=False,
+            ),
+            allow_gaps=False,
+            optional=True,
+        ),
+        allow_gaps=False,
+    )
 
     def extract_possible_references(self, level):
-        """Extract possible references of a given level."""
+        """Extract possible references of a given level.
+
+        Overrides the parent-class function. BigQuery's support for things like
+        the following:
+        - Functions that take a table as a parameter (e.g. TO_JSON_STRING)
+          https://cloud.google.com/bigquery/docs/reference/standard-sql/
+          json_functions#to_json_string
+        - STRUCT
+
+        means that, without schema information (which SQLFluff does not have),
+        references to data are often ambiguous.
+        """
         level = self._level_to_int(level)
         refs = list(self.iter_raw_references())
         if level == self.ObjectReferenceLevel.SCHEMA.value and len(refs) >= 3:
-            return [refs[0]]
-        if level == self.ObjectReferenceLevel.TABLE.value and len(refs) >= 3:
-            # Ambiguous case: The table could be the first or second part, so
-            # return both.
-            return [refs[0], refs[1]]
-        if level == self.ObjectReferenceLevel.OBJECT.value and len(refs) >= 3:
+            return [refs[0]]  # pragma: no cover
+        if level == self.ObjectReferenceLevel.TABLE.value:
+            # One part: Could be a table, e.g. TO_JSON_STRING(t)
+            # Two parts: Could be dataset.table or table.column.
+            # Three parts: Could be table.column.struct or dataset.table.column.
+            # Four parts: dataset.table.column.struct
+            # Five parts: project.dataset.table.column.struct
+            # So... return the first 3 parts.
+            return refs[:3]
+        if (
+            level == self.ObjectReferenceLevel.OBJECT.value and len(refs) >= 3
+        ):  # pragma: no cover
             # Ambiguous case: The object (i.e. column) could be the first or
             # second part, so return both.
             return [refs[1], refs[2]]
-        return super().extract_possible_references(level)
+        return super().extract_possible_references(level)  # pragma: no cover
+
+    def extract_possible_multipart_references(self, levels):
+        """Extract possible multipart references, e.g. schema.table."""
+        levels_tmp = [self._level_to_int(level) for level in levels]
+        min_level = min(levels_tmp)
+        max_level = max(levels_tmp)
+        refs = list(self.iter_raw_references())
+        if max_level == self.ObjectReferenceLevel.SCHEMA.value and len(refs) >= 3:
+            return [tuple(refs[0 : max_level - min_level + 1])]
+        # Note we aren't handling other possible cases. We'll add these as
+        # needed.
+        return super().extract_possible_multipart_references(levels)
 
 
 @bigquery_dialect.segment()
@@ -660,16 +836,13 @@ class DeclareStatementSegment(BaseSegment):
     parse_grammar = Sequence(
         "DECLARE",
         Delimited(Ref("NakedIdentifierSegment")),
-        Ref("DatatypeIdentifierSegment"),
-        Sequence(
-            "DEFAULT",
-            OneOf(
-                Ref("LiteralGrammar"),
-                Bracketed(Ref("SelectStatementSegment")),
-                Ref("BareFunctionSegment"),
-                Ref("FunctionSegment"),
+        OneOf(
+            Ref("DatatypeSegment"),
+            Ref("DefaultDeclareOptionsGrammar"),
+            Sequence(
+                Ref("DatatypeSegment"),
+                Ref("DefaultDeclareOptionsGrammar"),
             ),
-            optional=True,
         ),
     )
 
@@ -690,25 +863,24 @@ class SetStatementSegment(BaseSegment):
             Bracketed(Delimited(Ref("NakedIdentifierSegment"))),
         ),
         Ref("EqualsSegment"),
-        OneOf(
-            Delimited(
-                OneOf(
-                    Ref("LiteralGrammar"),
-                    Bracketed(Ref("SelectStatementSegment")),
-                    Ref("BareFunctionSegment"),
-                    Ref("FunctionSegment"),
-                    Bracketed(
-                        Delimited(
-                            OneOf(
-                                Ref("LiteralGrammar"),
-                                Bracketed(Ref("SelectStatementSegment")),
-                                Ref("BareFunctionSegment"),
-                                Ref("FunctionSegment"),
-                            )
+        Delimited(
+            OneOf(
+                Ref("LiteralGrammar"),
+                Bracketed(Ref("SelectStatementSegment")),
+                Ref("BareFunctionSegment"),
+                Ref("FunctionSegment"),
+                Bracketed(
+                    Delimited(
+                        OneOf(
+                            Ref("LiteralGrammar"),
+                            Bracketed(Ref("SelectStatementSegment")),
+                            Ref("BareFunctionSegment"),
+                            Ref("FunctionSegment"),
                         )
-                    ),
-                )
-            )
+                    )
+                ),
+                Ref("ArrayLiteralSegment"),
+            ),
         ),
     )
 
@@ -721,7 +893,7 @@ class PartitionBySegment(BaseSegment):
     match_grammar = StartsWith(
         "PARTITION",
         terminator=OneOf("CLUSTER", "OPTIONS", "AS", Ref("DelimiterSegment")),
-        enforce_whitespace_preceeding_terminator=True,
+        enforce_whitespace_preceding_terminator=True,
     )
     parse_grammar = Sequence(
         "PARTITION",
@@ -738,7 +910,7 @@ class ClusterBySegment(BaseSegment):
     match_grammar = StartsWith(
         "CLUSTER",
         terminator=OneOf("OPTIONS", "AS", Ref("DelimiterSegment")),
-        enforce_whitespace_preceeding_terminator=True,
+        enforce_whitespace_preceding_terminator=True,
     )
     parse_grammar = Sequence(
         "CLUSTER",
@@ -803,3 +975,185 @@ class CreateTableStatementSegment(BaseSegment):
             optional=True,
         ),
     )
+
+
+@bigquery_dialect.segment(replace=True)
+class CreateViewStatementSegment(BaseSegment):
+    """A `CREATE VIEW` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#view_option_list
+    """
+
+    type = "create_view_statement"
+
+    match_grammar = Sequence(
+        "CREATE",
+        Ref("OrReplaceGrammar", optional=True),
+        "VIEW",
+        Ref("IfNotExistsGrammar", optional=True),
+        Ref("TableReferenceSegment"),
+        # Optional list of column names
+        Ref("BracketedColumnReferenceListGrammar", optional=True),
+        Ref("OptionsSegment", optional=True),
+        "AS",
+        OptionallyBracketed(Ref("SelectableGrammar")),
+    )
+
+
+@bigquery_dialect.segment()
+class ParameterizedSegment(BaseSegment):
+    """BigQuery allows named and argument based parameters to help preven SQL Injection.
+
+    https://cloud.google.com/bigquery/docs/parameterized-queries
+    """
+
+    type = "parameterized_expression"
+    match_grammar = OneOf(Ref("AtSignLiteralSegment"), Ref("QuestionMarkSegment"))
+
+
+@bigquery_dialect.segment()
+class FromPivotExpressionSegment(BaseSegment):
+    """A PIVOT expression.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/query-syntax#pivot_operator
+    """
+
+    type = "from_pivot_expression"
+    match_grammar = Sequence("PIVOT", Bracketed(Anything()))
+
+    parse_grammar = Sequence(
+        "PIVOT",
+        Bracketed(
+            Delimited(
+                Sequence(
+                    Ref("FunctionSegment"),
+                    Ref("AliasExpressionSegment", optional=True),
+                ),
+            ),
+            "FOR",
+            Ref("SingleIdentifierGrammar"),
+            "IN",
+            Bracketed(
+                Delimited(
+                    Sequence(
+                        Ref("LiteralGrammar"),
+                        Ref("AliasExpressionSegment", optional=True),
+                    ),
+                )
+            ),
+        ),
+    )
+
+
+@bigquery_dialect.segment()
+class UnpivotAliasExpressionSegment(BaseSegment):
+    """In BigQuery UNPIVOT alias's can be single or double quoted."""
+
+    type = "alias_expression"
+    match_grammar = Sequence(
+        Ref.keyword("AS", optional=True),
+        OneOf(
+            Ref("SingleQuotedLiteralSegment"),
+            Ref("DoubleQuotedLiteralSegment"),
+        ),
+    )
+
+
+@bigquery_dialect.segment()
+class FromUnpivotExpressionSegment(BaseSegment):
+    """An UNPIVOT expression.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/query-syntax#unpivot_operator
+    """
+
+    type = "from_unpivot_expression"
+    match_grammar = Sequence(
+        "UNPIVOT",
+        Sequence(
+            OneOf("INCLUDE", "EXCLUDE"),
+            "NULLS",
+            optional=True,
+        ),
+        OneOf(
+            Bracketed(
+                Ref("SingleIdentifierGrammar"),
+                "FOR",
+                Ref("SingleIdentifierGrammar"),
+                "IN",
+                Bracketed(
+                    Delimited(Ref("SingleIdentifierGrammar")),
+                    Ref("UnpivotAliasExpressionSegment", optional=True),
+                ),
+            ),
+            Bracketed(
+                Bracketed(
+                    Delimited(
+                        Ref("SingleIdentifierGrammar"),
+                        min_delimiters=1,
+                    ),
+                ),
+                "FOR",
+                Ref("SingleIdentifierGrammar"),
+                "IN",
+                Bracketed(
+                    Delimited(
+                        Sequence(
+                            Bracketed(
+                                Delimited(
+                                    Ref("SingleIdentifierGrammar"),
+                                    min_delimiters=1,
+                                ),
+                            ),
+                            Ref("UnpivotAliasExpressionSegment", optional=True),
+                        ),
+                    ),
+                ),
+            ),
+        ),
+    )
+
+
+@bigquery_dialect.segment()
+class TableAliasExpressionSegment(BaseSegment):
+    """A reference to an object with an `AS` clause, optionally with column aliasing."""
+
+    type = "table_alias_expression"
+    match_grammar = Sequence(
+        Ref("AliasExpressionSegment"),
+        # Optional column aliases too.
+        Bracketed(
+            Delimited(Ref("SingleIdentifierGrammar"), delimiter=Ref("CommaSegment")),
+            optional=True,
+        ),
+    )
+
+
+@bigquery_dialect.segment(replace=True)
+class InsertStatementSegment(BaseSegment):
+    """A `INSERT` statement.
+
+    N.B. not a complete implementation.
+    """
+
+    type = "insert_statement"
+    match_grammar = StartsWith("INSERT")
+    parse_grammar = Sequence(
+        "INSERT",
+        Ref.keyword("INTO", optional=True),
+        Ref("TableReferenceSegment"),
+        Ref("BracketedColumnReferenceListGrammar", optional=True),
+        Ref("SelectableGrammar"),
+    )
+
+
+@bigquery_dialect.segment(replace=True)
+class SamplingExpressionSegment(BaseSegment):
+    """A sampling expression.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/query-syntax#tablesample_operator
+    """
+
+    type = "sample_expression"
+    match_grammar = Sequence(
+        "TABLESAMPLE", "SYSTEM", Bracketed(Ref("NumericLiteralSegment"), "PERCENT")
+    )
