@@ -44,7 +44,6 @@ teradata_dialect.sets("unreserved_keywords").difference_update(
         # The following are moved to being reserved keywords
         "UNION",
         "TIMESTAMP",
-        "DATE",
     ]
 )
 
@@ -69,6 +68,7 @@ teradata_dialect.sets("unreserved_keywords").update(
         "MACRO",
         "MAXINTERVALS",
         "MAXVALUELENGTH",
+        "MEETS",
         "MERGEBLOCKRATIO",
         "PERCENT",
         "PROFILE",
@@ -76,6 +76,7 @@ teradata_dialect.sets("unreserved_keywords").update(
         "QUIT",
         "RUN",
         "SAMPLE",
+        "SEL",
         "STAT",
         "SUMMARY",
         "THRESHOLD",
@@ -84,7 +85,9 @@ teradata_dialect.sets("unreserved_keywords").update(
     ]
 )
 
-teradata_dialect.sets("reserved_keywords").update(["UNION", "TIMESTAMP", "DATE"])
+teradata_dialect.sets("reserved_keywords").update(["UNION", "TIMESTAMP"])
+
+teradata_dialect.sets("bare_functions").update(["DATE"])
 
 
 # BTEQ statement
@@ -200,7 +203,8 @@ class TdCollectStatisticsStatementSegment(BaseSegment):
     """A `COLLECT STATISTICS (Optimizer Form)` statement.
 
     # TODO: add expression
-    COLLECT [SUMMARY] (STATISTICS|STAT) [[COLUMN| [UNIQUE] INDEX] (expression (, expression ...)] ON TABLENAME
+    COLLECT [SUMMARY] (STATISTICS|STAT) [[COLUMN| [UNIQUE] INDEX]
+    (expression (, expression ...)] ON TABLENAME
     """
 
     type = "collect_statistics_statement"
@@ -218,7 +222,8 @@ class TdCollectStatisticsStatementSegment(BaseSegment):
         ),
         Delimited(
             OneOf(
-                # UNIQUE INDEX index_name ALL (column_name, ...) ORDER BY VALUES|HASH (column_name)
+                # UNIQUE INDEX index_name ALL (column_name, ...) ORDER BY VALUES|HASH
+                # (column_name)
                 Sequence(
                     Ref.keyword("UNIQUE", optional=True),
                     "INDEX",
@@ -271,7 +276,8 @@ class TdCommentStatementSegment(BaseSegment):
     """A `COMMENT` statement.
 
     COMMENT [ON] (object_kind_1|object_kind_2) name [[AS|IS] comment]
-    object_kind_1: (COLUMN|FUNCTION|GLOP SET|MACRO|MAP|METHOD|PROCEDURE|PROFILE|ROLE|TRIGGER|TYPE|VIEW)
+    object_kind_1: (COLUMN|FUNCTION|GLOP SET|MACRO|MAP|METHOD|PROCEDURE|PROFILE|ROLE|
+                    TRIGGER|TYPE|VIEW)
     object_kind_2: (DATABASE|FILE|TABLE|USER)
     """
 
@@ -407,15 +413,15 @@ class ColumnDefinitionSegment(BaseSegment):
         Ref("DatatypeSegment"),  # Column type
         Bracketed(Anything(), optional=True),  # For types like VARCHAR(100)
         AnyNumberOf(
-            Ref("ColumnOptionSegment", optional=True),
+            Ref("ColumnConstraintSegment", optional=True),
             # Adding Teradata specific column definitions
-            Ref("TdColumnOptionSegment", optional=True),
+            Ref("TdColumnConstraintSegment", optional=True),
         ),
     )
 
 
 @teradata_dialect.segment()
-class TdColumnOptionSegment(BaseSegment):
+class TdColumnConstraintSegment(BaseSegment):
     """Teradata specific column attributes.
 
     e.g. CHARACTER SET LATIN | [NOT] (CASESPECIFIC|CS) | (UPPERCASE|UC)
@@ -452,7 +458,8 @@ class TdColumnOptionSegment(BaseSegment):
 class TdCreateTableOptions(BaseSegment):
     """CreateTableOptions.
 
-    , NO FALLBACK, NO BEFORE JOURNAL, NO AFTER JOURNAL, CHECKSUM = DEFAULT, DEFAULT MERGEBLOCKRATIO
+    , NO FALLBACK, NO BEFORE JOURNAL, NO AFTER JOURNAL, CHECKSUM = DEFAULT
+    , DEFAULT MERGEBLOCKRATIO
     """
 
     type = "create_table_options_statement"
@@ -515,7 +522,8 @@ class TdTablePartitioningLevel(BaseSegment):
 
     column_partition := ([COLUMN|ROW] column_name (, column_name2, ...) NO AUTOCOMPRESS
 
-    partition_expression := CASE_N, RANGE_N, EXTRACT, expression and in case of multi-level in parenthesis
+    partition_expression := CASE_N, RANGE_N, EXTRACT, expression and in case of
+    multi-level in parenthesis
     """
 
     type = "td_partitioning_level"
@@ -706,7 +714,16 @@ teradata_dialect.replace(
         Ref("NakedIdentifierSegment"),
         Ref("QuotedIdentifierSegment"),
         Ref("TdCastIdentifierSegment"),
-    )
+    ),
+    SelectClauseSegmentGrammar=Sequence(
+        OneOf("SELECT", "SEL"),
+        Ref("SelectClauseModifierSegment", optional=True),
+        Indent,
+        Delimited(
+            Ref("SelectClauseElementSegment"),
+            allow_trailing=True,
+        ),
+    ),
 )
 
 
@@ -717,8 +734,8 @@ class QualifyClauseSegment(BaseSegment):
     type = "qualify_clause"
     match_grammar = StartsWith(
         "QUALIFY",
-        terminator=OneOf("ORDER", "LIMIT", "QUALIFY", "WINDOW"),
-        enforce_whitespace_preceeding_terminator=True,
+        terminator=OneOf(Sequence("ORDER", "BY"), "LIMIT", "QUALIFY", "WINDOW"),
+        enforce_whitespace_preceding_terminator=True,
     )
     parse_grammar = Sequence(
         "QUALIFY",
@@ -746,3 +763,85 @@ class SelectStatementSegment(BaseSegment):
         insert=[Ref("QualifyClauseSegment", optional=True)],
         before=Ref("OrderByClauseSegment", optional=True),
     )
+
+
+@teradata_dialect.segment(replace=True)
+class UnorderedSelectStatementSegment(BaseSegment):
+    """An unordered `SELECT` statement.
+
+    https://dev.mysql.com/doc/refman/5.7/en/select.html
+    """
+
+    type = "select_statement"
+    match_grammar = ansi_dialect.get_segment(
+        "UnorderedSelectStatementSegment"
+    ).match_grammar.copy()
+
+    parse_grammar = ansi_dialect.get_segment(
+        "UnorderedSelectStatementSegment"
+    ).parse_grammar.copy(
+        insert=[Ref("QualifyClauseSegment", optional=True)],
+        before=Ref("OverlapsClauseSegment", optional=True),
+    )
+
+
+@teradata_dialect.segment(replace=True)
+class SelectClauseModifierSegment(BaseSegment):
+    """Things that come after SELECT but before the columns.
+
+    Adds NORMALIZE clause:
+    https://docs.teradata.com/r/2_MC9vCtAJRlKle2Rpb0mA/UuxiA0mklFgv~33X5nyKMA
+    """
+
+    type = "select_clause_modifier"
+    match_grammar = OneOf(
+        "DISTINCT",
+        "ALL",
+        Sequence(
+            "NORMALIZE",
+            OneOf(
+                Sequence(
+                    "ON",
+                    "MEETS",
+                    "OR",
+                    "OVERLAPS",
+                ),
+                Sequence(
+                    "ON",
+                    "OVERLAPS",
+                ),
+                Sequence(
+                    "ON",
+                    "OVERLAPS",
+                    "OR",
+                    "MEETS",
+                ),
+                optional=True,
+            ),
+        ),
+    )
+
+
+@teradata_dialect.segment(replace=True)
+class SelectClauseSegment(BaseSegment):
+    """A group of elements in a select target statement.
+
+    Remove OVERLAPS as a terminator as this can be part of SelectClauseModifierSegment
+    """
+
+    type = "select_clause"
+    match_grammar = StartsWith(
+        Sequence(
+            OneOf("SELECT", "SEL"), Ref("WildcardExpressionSegment", optional=True)
+        ),
+        terminator=OneOf(
+            "FROM",
+            "WHERE",
+            Sequence("ORDER", "BY"),
+            "LIMIT",
+            Ref("SetOperatorSegment"),
+        ),
+        enforce_whitespace_preceding_terminator=True,
+    )
+
+    parse_grammar = ansi_dialect.get_segment("SelectClauseSegment").parse_grammar.copy()
