@@ -5,7 +5,13 @@ from typing import Tuple, List
 
 from sqlfluff.core.parser import WhitespaceSegment
 
-from sqlfluff.core.rules.base import BaseRule, LintResult, LintFix
+from sqlfluff.core.rules.base import (
+    BaseRule,
+    LintResult,
+    LintFix,
+    RuleContext,
+    EvalResultType,
+)
 from sqlfluff.core.rules.doc_decorators import document_fix_compatible
 
 
@@ -13,8 +19,9 @@ from sqlfluff.core.rules.doc_decorators import document_fix_compatible
 class Rule_L006(BaseRule):
     """Operators should be surrounded by a single whitespace.
 
-    | **Anti-pattern**
-    | In this example, there is a space missing space between the operator and 'b'.
+    **Anti-pattern**
+
+    In this example, there is a space missing between the operator and ``b``.
 
     .. code-block:: sql
 
@@ -23,8 +30,9 @@ class Rule_L006(BaseRule):
         FROM foo
 
 
-    | **Best practice**
-    | Keep a single space.
+    **Best practice**
+
+    Keep a single space.
 
     .. code-block:: sql
 
@@ -33,6 +41,11 @@ class Rule_L006(BaseRule):
         FROM foo
     """
 
+    # L006 works on operators so requires three operators.
+    # However some rules that inherit from here (e.g. L048) do not.
+    # So allow this to be configurable.
+    _require_three_children: bool = True
+
     _target_elems: List[Tuple[str, str]] = [
         ("type", "binary_operator"),
         ("type", "comparison_operator"),
@@ -51,7 +64,7 @@ class Rule_L006(BaseRule):
         if seg.name.endswith("_bracket"):
             if seg.name.startswith("start_" if before else "end_"):
                 return False
-        if seg.is_meta:
+        if seg.is_meta:  # pragma: no cover
             if before:
                 if seg.source_str.endswith(" ") or seg.source_str.endswith("\n"):
                     return False
@@ -73,7 +86,7 @@ class Rule_L006(BaseRule):
                 return segments[j]
         return None
 
-    def _eval(self, segment, **kwargs):
+    def _eval(self, context: RuleContext) -> EvalResultType:
         """Operators should be surrounded by a single whitespace.
 
         Rewritten to assess direct children of a segment to make
@@ -92,12 +105,12 @@ class Rule_L006(BaseRule):
         # be dealt with by the parent segment. That also means that we need
         # to have at least three children.
 
-        if len(segment.segments) <= 2:
+        if self._require_three_children and len(context.segment.segments) <= 2:
             return LintResult()
 
         violations = []
 
-        for idx, sub_seg in enumerate(segment.segments):
+        for idx, sub_seg in enumerate(context.segment.segments):
             check_before = False
             check_after = False
             before_anchor = sub_seg
@@ -119,7 +132,7 @@ class Rule_L006(BaseRule):
             # Is it a compound segment ending or starting with the target?
             elif sub_seg.segments:
                 # Get first and last raw segments.
-                raw_list = list(sub_seg.iter_raw_seg())
+                raw_list = list(sub_seg.get_raw_segments())
                 if len(raw_list) > 1:
                     leading = raw_list[0]
                     trailing = raw_list[-1]
@@ -141,7 +154,9 @@ class Rule_L006(BaseRule):
                         check_after = True
 
             if check_before:
-                prev_seg = self._find_segment(idx, segment.segments, before=True)
+                prev_seg = self._find_segment(
+                    idx, context.segment.segments, before=True
+                )
                 if self._missing_whitespace(prev_seg, before=True):
                     self.logger.debug(
                         "Missing Whitespace Before %r. Found %r instead.",
@@ -152,21 +167,23 @@ class Rule_L006(BaseRule):
                         LintResult(
                             anchor=before_anchor,
                             description="Missing whitespace before {}".format(
-                                before_anchor.raw[:10]
+                                before_anchor.raw
                             ),
                             fixes=[
-                                LintFix(
-                                    "create",
-                                    # NB the anchor here is always in the parent and not anchor
-                                    anchor=sub_seg,
-                                    edit=WhitespaceSegment(raw=" "),
+                                LintFix.create_before(
+                                    # NB the anchor here is always in the parent and not
+                                    # anchor
+                                    anchor_segment=sub_seg,
+                                    edit_segments=[WhitespaceSegment(raw=" ")],
                                 )
                             ],
                         )
                     )
 
             if check_after:
-                next_seg = self._find_segment(idx, segment.segments, before=False)
+                next_seg = self._find_segment(
+                    idx, context.segment.segments, before=False
+                )
                 if self._missing_whitespace(next_seg, before=False):
                     self.logger.debug(
                         "Missing Whitespace After %r. Found %r instead.",
@@ -177,14 +194,14 @@ class Rule_L006(BaseRule):
                         LintResult(
                             anchor=after_anchor,
                             description="Missing whitespace after {}".format(
-                                after_anchor.raw[-10:]
+                                after_anchor.raw
                             ),
                             fixes=[
-                                LintFix(
-                                    "create",
-                                    # NB the anchor here is always in the parent and not anchor
-                                    anchor=next_seg,
-                                    edit=WhitespaceSegment(raw=" "),
+                                LintFix.create_before(
+                                    # NB the anchor here is always in the parent and not
+                                    # anchor
+                                    anchor_segment=next_seg,
+                                    edit_segments=[WhitespaceSegment(raw=" ")],
                                 )
                             ],
                         )
@@ -192,3 +209,5 @@ class Rule_L006(BaseRule):
 
         if violations:
             return violations
+
+        return LintResult()
