@@ -1,14 +1,19 @@
 """Implementation of Rule L007."""
+from sqlfluff.core.rules.base import BaseRule, LintResult, RuleContext
+from sqlfluff.core.rules.doc_decorators import document_configuration
 
-from sqlfluff.core.rules.base import BaseRule, LintResult
+after_description = "Operators near newlines should be after, not before the newline"
+before_description = "Operators near newlines should be before, not after the newline"
 
 
+@document_configuration
 class Rule_L007(BaseRule):
-    """Operators near newlines should be after, not before the newline.
+    """Operators should follow a standard for being before/after newlines.
 
-    | **Anti-pattern**
-    | The • character represents a space.
-    | In this example, the operator '+' should not be at the end of the second line.
+    **Anti-pattern**
+
+    In this example, if ``operator_new_lines = after`` (or unspecified, as is the
+    default), then the operator ``+`` should not be at the end of the second line.
 
     .. code-block:: sql
 
@@ -18,8 +23,10 @@ class Rule_L007(BaseRule):
         FROM foo
 
 
-    | **Best practice**
-    | Place the operator after the newline. Unless specified by ``operator_new_lines = after``
+    **Best practice**
+
+    If ``operator_new_lines = after`` (or unspecified, as this is the default),
+    place the operator after the newline.
 
     .. code-block:: sql
 
@@ -27,12 +34,21 @@ class Rule_L007(BaseRule):
             a
             + b
         FROM foo
+
+    If ``operator_new_lines = before``, place the operator before the newline.
+
+    .. code-block:: sql
+
+        SELECT
+            a +
+            b
+        FROM foo
     """
 
     config_keywords = ["operator_new_lines"]
 
-    def _eval(self, segment, memory, parent_stack, **kwargs):
-        """Operators near newlines should be after, not before the newline.
+    def _eval(self, context: RuleContext) -> LintResult:
+        """Operators should follow a standard for being before/after newlines.
 
         We use the memory to keep track of whitespace up to now, and
         whether the last code segment was an operator or not.
@@ -43,25 +59,30 @@ class Rule_L007(BaseRule):
 
         """
         anchor = None
+        memory = context.memory
+        description = after_description
+        if self.operator_new_lines == "before":  # type: ignore
+            description = before_description
 
         # The parent stack tells us whether we're in an expression or not.
-        if parent_stack and parent_stack[-1].is_type("expression"):
-            if segment.is_code:
+        if context.parent_stack and context.parent_stack[-1].is_type("expression"):
+            if context.segment.is_code:
                 # This is code, what kind?
-                if segment.is_type("binary_operator", "comparison_operator"):
+                if context.segment.is_type("binary_operator", "comparison_operator"):
                     # If it's an operator, then check if in "before" mode
-                    if self.operator_new_lines == "before":
-                        # If we're in "before" mode, then check if newline since last code
+                    if self.operator_new_lines == "before":  # type: ignore
+                        # If we're in "before" mode, then check if newline since last
+                        # code
                         for s in memory["since_code"]:
                             if s.name == "newline":
                                 # Had a newline - so mark this operator as a fail
-                                anchor = segment
+                                anchor = context.segment
                                 # TODO: Work out a nice fix for this failure.
                 elif memory["last_code"] and memory["last_code"].is_type(
                     "binary_operator", "comparison_operator"
                 ):
                     # It's not an operator, but the last code was.
-                    if self.operator_new_lines != "before":
+                    if self.operator_new_lines != "before":  # type: ignore
                         # If in "after" mode, then check to see
                         # there is a newline between us and the last operator.
                         for s in memory["since_code"]:
@@ -70,18 +91,18 @@ class Rule_L007(BaseRule):
                                 anchor = memory["last_code"]
                                 # TODO: Work out a nice fix for this failure.
                 # Prepare memory for later
-                memory["last_code"] = segment
+                memory["last_code"] = context.segment
                 memory["since_code"] = []
             else:
                 # This isn't a code segment...
                 # Prepare memory for later
-                memory["since_code"].append(segment)
+                memory["since_code"].append(context.segment)
         else:
             # Reset the memory if we're not in an expression
             memory = {"last_code": None, "since_code": []}
 
         # Anchor is our signal as to whether there's a problem
         if anchor:
-            return LintResult(anchor=anchor, memory=memory)
+            return LintResult(anchor=anchor, memory=memory, description=description)
         else:
             return LintResult(memory=memory)
