@@ -1,10 +1,16 @@
 """Implementation of Rule L016."""
 
-from typing import Tuple
+from typing import cast, List, Optional, Sequence, Tuple
 
-from sqlfluff.core.parser import NewlineSegment, WhitespaceSegment
+from sqlfluff.core.parser import (
+    BaseSegment,
+    NewlineSegment,
+    RawSegment,
+    WhitespaceSegment,
+)
 
-from sqlfluff.core.rules.base import LintFix, LintResult
+from sqlfluff.core.rules.base import LintFix, LintResult, RuleContext
+from sqlfluff.core.rules.functional import sp
 from sqlfluff.core.rules.doc_decorators import (
     document_fix_compatible,
     document_configuration,
@@ -17,14 +23,17 @@ from sqlfluff.rules.L003 import Rule_L003
 class Rule_L016(Rule_L003):
     """Line is too long."""
 
+    _check_docstring = False
+
     config_keywords = [
         "max_line_length",
         "tab_space_size",
         "indent_unit",
         "ignore_comment_lines",
+        "ignore_comment_clauses",
     ]
 
-    def _eval_line_for_breaks(self, segments):
+    def _eval_line_for_breaks(self, segments: List[RawSegment]) -> List[LintFix]:
         """Evaluate the line for break points.
 
         We split the line into a few particular sections:
@@ -44,36 +53,50 @@ class Rule_L016(Rule_L003):
         indent_section = None
 
         class Section:
-            def __init__(self, segments, role, indent_balance, indent_impulse=0):
+            def __init__(
+                self,
+                segments: Sequence[RawSegment],
+                role: str,
+                indent_balance: int,
+                indent_impulse: Optional[int] = None,
+            ):
                 self.segments = segments
                 self.role = role
                 self.indent_balance = indent_balance
-                self.indent_impulse = indent_impulse
+                self.indent_impulse: int = indent_impulse or 0
 
             def __repr__(self):
-                return "<Section @ {pos}: {role} [{indent_balance}:{indent_impulse}]. {segments!r}>".format(
-                    role=self.role,
-                    indent_balance=self.indent_balance,
-                    indent_impulse=self.indent_impulse,
-                    segments="".join(elem.raw for elem in self.segments),
-                    pos=self.segments[0].get_start_point_marker()
-                    if self.segments
-                    else "",
+                return (
+                    "<Section @ {pos}: {role} [{indent_balance}:{indent_impulse}]. "
+                    "{segments!r}>".format(
+                        role=self.role,
+                        indent_balance=self.indent_balance,
+                        indent_impulse=self.indent_impulse,
+                        segments="".join(elem.raw for elem in self.segments),
+                        pos=self.segments[0].get_start_point_marker()
+                        if self.segments
+                        else "",
+                    )
                 )
 
             @property
-            def raw(self):
+            def raw(self) -> str:
                 return "".join(seg.raw for seg in self.segments)
 
             @staticmethod
-            def find_segment_at(segments, loc: Tuple[int, int]):
+            def find_segment_at(segments, loc: Tuple[int, int]) -> RawSegment:
                 for seg in segments:
                     if not seg.is_meta and seg.pos_marker.working_loc == loc:
                         return seg
+                raise ValueError("Segment not found")  # pragma: no cover
 
             def generate_fixes_to_coerce(
-                self, segments, indent_section, crawler, indent
-            ):
+                self,
+                segments: List[RawSegment],
+                indent_section: "Section",
+                crawler: Rule_L016,
+                indent: int,
+            ) -> List[LintFix]:
                 """Generate a list of fixes to create a break at this point.
 
                 The `segments` argument is necessary to extract anchors
@@ -110,7 +133,9 @@ class Rule_L016(Rule_L003):
 
                     # NOTE: Deal with commas and binary operators differently here.
                     # Maybe only deal with commas to start with?
-                    if any(seg.is_type("binary_operator") for seg in self.segments):
+                    if any(
+                        seg.is_type("binary_operator") for seg in self.segments
+                    ):  # pragma: no cover
                         raise NotImplementedError(
                             "Don't know how to deal with binary operators here yet!!"
                         )
@@ -118,12 +143,11 @@ class Rule_L016(Rule_L003):
                     # Remove any existing whitespace
                     for elem in self.segments:
                         if not elem.is_meta and elem.is_type("whitespace"):
-                            fixes.append(LintFix("delete", elem))
+                            fixes.append(LintFix.delete(elem))
 
                     # Create a newline and a similar indent
                     fixes.append(
-                        LintFix(
-                            "create",
+                        LintFix.create_before(
                             create_anchor,
                             [
                                 NewlineSegment(),
@@ -140,11 +164,10 @@ class Rule_L016(Rule_L003):
                     # Remove anything which is already here
                     for elem in self.segments:
                         if not elem.is_meta:
-                            fixes.append(LintFix("delete", elem))
+                            fixes.append(LintFix.delete(elem))
                     # Create a newline, create an indent of the relevant size
                     fixes.append(
-                        LintFix(
-                            "create",
+                        LintFix.create_before(
                             create_anchor,
                             [
                                 NewlineSegment(),
@@ -153,14 +176,17 @@ class Rule_L016(Rule_L003):
                         )
                     )
                     return fixes
-                raise ValueError(f"Unexpected break generated at {self}")
+                raise ValueError(
+                    f"Unexpected break generated at {self}"
+                )  # pragma: no cover
 
-        segment_buff = ()
-        whitespace_buff = ()
+        segment_buff: Tuple[RawSegment, ...] = ()
+        whitespace_buff: Tuple[RawSegment, ...] = ()
         indent_impulse = 0
         indent_balance = 0
         is_pause = False
 
+        seg: RawSegment
         for seg in segments:
             if indent_section is None:
                 if seg.is_type("whitespace") or seg.is_meta:
@@ -254,11 +280,12 @@ class Rule_L016(Rule_L003):
             role = "pausepoint"
         elif segment_buff:
             role = "content"
-        elif indent_impulse:
+        elif indent_impulse:  # pragma: no cover
             role = "breakpoint"
         else:
-            raise ValueError("Is this possible?")
-
+            # This can happen, e.g. with a long template line. Treat it as
+            # unfixable.
+            return []
         chunk_buff.append(
             Section(
                 segments=segment_buff + whitespace_buff,
@@ -275,7 +302,6 @@ class Rule_L016(Rule_L003):
         # First, do we ever go through a negative breakpoint?
         lowest_bal = min(sec.indent_balance for sec in chunk_buff)
         split_at = []  # split_at is probably going to be a list.
-        fixes = []
         if lowest_bal < 0:
             for sec in chunk_buff:
                 if sec.indent_balance == 0 and sec.indent_impulse < 0:
@@ -333,6 +359,7 @@ class Rule_L016(Rule_L003):
         fixes = []
         for split, indent in split_at:
             if split.segments:
+                assert indent_section
                 fixes += split.generate_fixes_to_coerce(
                     segments, indent_section, self, indent
                 )
@@ -342,14 +369,14 @@ class Rule_L016(Rule_L003):
         return fixes
 
     @staticmethod
-    def _gen_line_so_far(raw_stack, initial_buff=None):
+    def _gen_line_so_far(raw_stack: Tuple[RawSegment, ...]) -> List[RawSegment]:
         """Work out from the raw stack what the elements on this line are.
 
         Returns:
             :obj:`list` of segments
 
         """
-        working_buff = initial_buff or []
+        working_buff: List[RawSegment] = []
         idx = -1
         while True:
             if len(raw_stack) >= abs(idx):
@@ -360,38 +387,135 @@ class Rule_L016(Rule_L003):
                     working_buff.insert(0, s)
                     idx -= 1
             else:
-                break
+                break  # pragma: no cover
         return working_buff
 
-    def _eval(self, segment, raw_stack, **kwargs):
+    @classmethod
+    def _compute_segment_length(cls, segment: BaseSegment) -> int:
+        if segment.is_type("newline"):
+            # Generally, we won't see newlines, but if we do, simply ignore
+            # them. Rationale: The intent of this rule is to enforce maximum
+            # line length, and newlines don't make lines longer.
+            return 0
+
+        if "\n" in segment.pos_marker.source_str():
+            # Similarly we shouldn't see newlines in source segments
+            # However for templated loops it's often not possible to
+            # accurately calculate the segments. These will be caught by
+            # the first iteration of the loop (which is non-templated)
+            # so doesn't suffer from the same bug, so we can ignore these
+            return 0
+
+        # Compute the length of this segments in SOURCE space (before template
+        # expansion).
+        slice_length = (
+            segment.pos_marker.source_slice.stop - segment.pos_marker.source_slice.start
+        )
+        if slice_length:
+            return slice_length
+        else:
+            # If a segment did not originate from the original source, its slice
+            # length slice length will be zero. This occurs, for example, when
+            # other lint rules add indentation or other whitespace. In that
+            # case, compute the length of its contents.
+            return len(segment.raw)
+
+    def _compute_source_length(
+        self, segments: Sequence[BaseSegment], memory: dict
+    ) -> int:
+        line_len = 0
+        seen_slices = set()
+        for segment in segments:
+            if self.ignore_comment_clauses and segment in memory["comment_clauses"]:
+                continue
+
+            slice = (
+                segment.pos_marker.source_slice.start,
+                segment.pos_marker.source_slice.stop,
+            )
+            # Often, a single templated area of a source file will expand to
+            # multiple SQL tokens. Here, we use a set to avoid double counting
+            # the length of that text. For example, in BigQuery, we might
+            # see this source query:
+            #
+            # SELECT user_id
+            # FROM `{{bi_ecommerce_orders}}` {{table_at_job_start}}
+            #
+            # where 'table_at_job_start' is defined as:
+            # "FOR SYSTEM_TIME AS OF CAST('2021-03-02T01:22:59+00:00' AS TIMESTAMP)"
+            #
+            # So this one substitution results in roughly 10 segments (one per
+            # word or bit of punctuation). Each of these would have the same
+            # source slice, and if we didn't correct for this, we'd count the
+            # length of {{bi_ecommerce_orders}} roughly 10 times, resulting in
+            # vast overcount of the source length.
+            #
+            # :TRICKY: New segments (i.e. those introduced by earlier fixes)
+            # have empty source slices. We definitely want to count the length
+            # of these segments. We can be sure they aren't the tricky templated
+            # segment case described above because new segments are never templated
+            # (because "sqlfluff fix" produced them, not the templater!).
+            if (
+                slice[0] == slice[1] and not segment.is_meta
+            ) or slice not in seen_slices:
+                seen_slices.add(slice)
+                line_len += self._compute_segment_length(segment)
+        return line_len
+
+    def _eval(self, context: RuleContext) -> Optional[LintResult]:
         """Line is too long.
 
         This only triggers on newline segments, evaluating the whole line.
         The detection is simple, the fixing is much trickier.
 
         """
-        if segment.name == "newline":
-            # iterate to buffer the whole line up to this point
-            this_line = self._gen_line_so_far(raw_stack, [])
+        # Config type hints
+        self.max_line_length: int
+        self.ignore_comment_lines: bool
+        self.ignore_comment_clauses: bool
+
+        if not context.memory:
+            memory: dict = {"comment_clauses": set()}
         else:
+            memory = context.memory
+        if context.segment.name == "newline":
+            # iterate to buffer the whole line up to this point
+            this_line = self._gen_line_so_far(context.raw_stack)
+        else:
+            if self.ignore_comment_clauses and context.segment.is_type(
+                "comment_clause", "comment_equals_clause"
+            ):
+                comment_segment = context.functional.segment.children().first(
+                    sp.is_name("quoted_literal")
+                )
+                if comment_segment:
+                    memory["comment_clauses"].add(comment_segment.get())
+
             # Otherwise we're all good
-            return None
+            return LintResult(memory=memory)
 
         # Now we can work out the line length and deal with the content
-        line_len = sum(len(s.raw) for s in this_line)
+        line_len = self._compute_source_length(this_line, memory)
         if line_len > self.max_line_length:
             # Problem, we'll be reporting a violation. The
             # question is, can we fix it?
 
             # We'll need the indent, so let's get it for fixing.
             line_indent = []
-            idx = 0
             for s in this_line:
                 if s.name == "whitespace":
                     line_indent.append(s)
                 else:
                     break
 
+            # Don't even attempt to handle template placeholders as gets
+            # complicated if logic changes (e.g. moving for loops). Most of
+            # these long lines will likely be single line Jinja comments.
+            # They will remain as unfixable.
+            if this_line[-1].type == "placeholder":
+                self.logger.info("Unfixable template segment: %s", this_line[-1])
+                return LintResult(anchor=context.segment, memory=memory)
+
             # Does the line end in an inline comment that we can move back?
             if this_line[-1].name == "inline_comment":
                 # Is this line JUST COMMENT (with optional preceding whitespace) if
@@ -403,37 +527,60 @@ class Rule_L016(Rule_L003):
                         "Unfixable inline comment, alone on line: %s", this_line[-1]
                     )
                     if self.ignore_comment_lines:
-                        return LintResult()
+                        return LintResult(memory=memory)
                     else:
-                        return LintResult(anchor=segment)
+                        return LintResult(anchor=context.segment, memory=memory)
 
                 self.logger.info(
                     "Attempting move of inline comment at end of line: %s",
                     this_line[-1],
                 )
                 # Set up to delete the original comment and the preceding whitespace
-                delete_buffer = [LintFix("delete", this_line[-1])]
+                delete_buffer = [LintFix.delete(this_line[-1])]
                 idx = -2
                 while True:
                     if (
                         len(this_line) >= abs(idx)
                         and this_line[idx].name == "whitespace"
                     ):
-                        delete_buffer.append(LintFix("delete", this_line[idx]))
+                        delete_buffer.append(LintFix.delete(this_line[idx]))
                         idx -= 1
                     else:
-                        break
+                        break  # pragma: no cover
+                create_elements = line_indent + [
+                    this_line[-1],
+                    cast(RawSegment, context.segment),
+                ]
+                if (
+                    self._compute_source_length(create_elements, memory)
+                    > self.max_line_length
+                ):
+                    # The inline comment is NOT on a line by itself, but even if
+                    # we move it onto a line by itself, it's still too long. In
+                    # this case, the rule should do nothing, otherwise it
+                    # triggers an endless cycle of "fixes" that simply keeps
+                    # adding blank lines.
+                    self.logger.info(
+                        "Unfixable inline comment, too long even on a line by itself: "
+                        "%s",
+                        this_line[-1],
+                    )
+                    if self.ignore_comment_lines:
+                        return LintResult(memory=memory)
+                    else:
+                        return LintResult(anchor=context.segment, memory=memory)
                 # Create a newline before this one with the existing comment, an
                 # identical indent AND a terminating newline, copied from the current
                 # target segment.
-                create_buffer = [
-                    LintFix(
-                        "create", this_line[0], line_indent + [this_line[-1], segment]
-                    )
-                ]
-                return LintResult(anchor=segment, fixes=delete_buffer + create_buffer)
+                create_buffer = [LintFix.create_before(this_line[0], create_elements)]
+                return LintResult(
+                    anchor=context.segment,
+                    fixes=delete_buffer + create_buffer,
+                    memory=memory,
+                )
 
             fixes = self._eval_line_for_breaks(this_line)
             if fixes:
-                return LintResult(anchor=segment, fixes=fixes)
-            return LintResult(anchor=segment)
+                return LintResult(anchor=context.segment, fixes=fixes, memory=memory)
+            return LintResult(anchor=context.segment, memory=memory)
+        return LintResult(memory=memory)
