@@ -2,19 +2,25 @@
 
 from sqlfluff.core.parser import NewlineSegment, WhitespaceSegment
 
-from sqlfluff.core.rules.base import BaseRule, LintFix, LintResult
-from sqlfluff.core.rules.doc_decorators import document_fix_compatible
+from sqlfluff.core.rules.base import BaseRule, LintFix, LintResult, RuleContext
+from sqlfluff.core.rules.doc_decorators import (
+    document_fix_compatible,
+    document_configuration,
+)
 
 
 @document_fix_compatible
+@document_configuration
 class Rule_L018(BaseRule):
-    """WITH clause closing bracket should be aligned with WITH keyword.
+    """``WITH`` clause closing bracket should be aligned with ``WITH`` keyword.
 
-    | **Anti-pattern**
-    | The • character represents a space.
-    | In this example, the closing bracket is not aligned with WITH keyword.
+    **Anti-pattern**
 
-    .. code-block::
+    The ``•`` character represents a space.
+    In this example, the closing bracket is not aligned with ``WITH`` keyword.
+
+    .. code-block:: sql
+       :force:
 
         WITH zoo AS (
             SELECT a FROM foo
@@ -22,10 +28,11 @@ class Rule_L018(BaseRule):
 
         SELECT * FROM zoo
 
-    | **Best practice**
-    | Remove the spaces to align the WITH keyword with the closing bracket.
+    **Best practice**
 
-    .. code-block::
+    Remove the spaces to align the ``WITH`` keyword with the closing bracket.
+
+    .. code-block:: sql
 
         WITH zoo AS (
             SELECT a FROM foo
@@ -38,23 +45,23 @@ class Rule_L018(BaseRule):
     _works_on_unparsable = False
     config_keywords = ["tab_space_size"]
 
-    def _eval(self, segment, raw_stack, **kwargs):
+    def _eval(self, context: RuleContext) -> LintResult:
         """WITH clause closing bracket should be aligned with WITH keyword.
 
         Look for a with clause and evaluate the position of closing brackets.
         """
         # We only trigger on start_bracket (open parenthesis)
-        if segment.is_type("with_compound_statement"):
-            raw_stack_buff = list(raw_stack)
+        if context.segment.is_type("with_compound_statement"):
+            raw_stack_buff = list(context.raw_stack)
             # Look for the with keyword
-            for seg in segment.segments:
+            for seg in context.segment.segments:
                 if seg.name.lower() == "with":
                     seg_line_no = seg.pos_marker.line_no
                     break
-            else:
+            else:  # pragma: no cover
                 # This *could* happen if the with statement is unparsable,
                 # in which case then the user will have to fix that first.
-                if any(s.is_type("unparsable") for s in segment.segments):
+                if any(s.is_type("unparsable") for s in context.segment.segments):
                     return LintResult()
                 # If it's parsable but we still didn't find a with, then
                 # we should raise that.
@@ -81,7 +88,7 @@ class Rule_L018(BaseRule):
 
             balance = 0
             with_indent, with_indent_str = indent_size_up_to(raw_stack_buff)
-            for seg in segment.iter_segments(
+            for seg in context.segment.iter_segments(
                 expanding=["common_table_expression", "bracketed"], pass_through=True
             ):
                 if seg.name == "start_bracket":
@@ -100,10 +107,9 @@ class Rule_L018(BaseRule):
                             return LintResult(
                                 anchor=seg,
                                 fixes=[
-                                    LintFix(
-                                        "create",
+                                    LintFix.create_before(
                                         seg,
-                                        WhitespaceSegment(" " * (-indent_diff)),
+                                        [WhitespaceSegment(" " * (-indent_diff))],
                                     )
                                 ],
                             )
@@ -111,7 +117,7 @@ class Rule_L018(BaseRule):
                             # Is it all whitespace before the bracket on this line?
                             prev_segs_on_line = [
                                 elem
-                                for elem in segment.iter_segments(
+                                for elem in context.segment.iter_segments(
                                     expanding=["common_table_expression", "bracketed"],
                                     pass_through=True,
                                 )
@@ -123,20 +129,15 @@ class Rule_L018(BaseRule):
                             ):
                                 # We can move it back, it's all whitespace
                                 fixes = [
-                                    LintFix(
-                                        "create",
+                                    LintFix.create_before(
                                         seg,
                                         [WhitespaceSegment(with_indent_str)],
                                     )
-                                ] + [
-                                    LintFix("delete", elem)
-                                    for elem in prev_segs_on_line
-                                ]
+                                ] + [LintFix.delete(elem) for elem in prev_segs_on_line]
                             else:
                                 # We have to move it to a newline
                                 fixes = [
-                                    LintFix(
-                                        "create",
+                                    LintFix.create_before(
                                         seg,
                                         [
                                             NewlineSegment(),
