@@ -1,10 +1,9 @@
 """Implementation of Rule L019."""
 
-from typing import Dict, Any
+from typing import Any, Dict, Optional, Tuple
 
-from sqlfluff.core.parser import WhitespaceSegment
-
-from sqlfluff.core.rules.base import BaseRule, LintFix, LintResult
+from sqlfluff.core.parser import RawSegment, WhitespaceSegment
+from sqlfluff.core.rules.base import BaseRule, LintFix, LintResult, RuleContext
 from sqlfluff.core.rules.doc_decorators import (
     document_fix_compatible,
     document_configuration,
@@ -16,8 +15,9 @@ from sqlfluff.core.rules.doc_decorators import (
 class Rule_L019(BaseRule):
     """Leading/Trailing comma enforcement.
 
-    | **Anti-pattern**
-    | There is a mixture of leading and trailing commas.
+    **Anti-pattern**
+
+    There is a mixture of leading and trailing commas.
 
     .. code-block:: sql
 
@@ -27,10 +27,11 @@ class Rule_L019(BaseRule):
             c
         FROM foo
 
-    | **Best practice**
-    | By default sqlfluff prefers trailing commas, however it
-    | is configurable for leading commas. Whichever option you chose
-    | it does expect you to be consistent.
+    **Best practice**
+
+    By default, `SQLFluff` prefers trailing commas. However it
+    is configurable for leading commas. The chosen style must be used
+    consistently throughout your SQL.
 
     .. code-block:: sql
 
@@ -48,8 +49,6 @@ class Rule_L019(BaseRule):
             , b
             , c
         FROM foo
-
-
     """
 
     _works_on_unparsable = False
@@ -59,7 +58,8 @@ class Rule_L019(BaseRule):
     def _last_comment_seg(raw_stack):
         """Trace the raw stack back to the most recent comment segment.
 
-        A return value of `None` indicates no code segments preceding the current position.
+        A return value of `None` indicates no code segments preceding the current
+        position.
         """
         for segment in raw_stack[::-1]:
             if segment.is_comment:
@@ -67,17 +67,28 @@ class Rule_L019(BaseRule):
         return None
 
     @staticmethod
-    def _last_code_seg(raw_stack):
+    def _last_code_seg(raw_stack: Tuple[RawSegment, ...]) -> Optional[RawSegment]:
         """Trace the raw stack back to the most recent code segment.
 
-        A return value of `None` indicates no code segments preceding the current position.
+        A return value of `None` indicates no code segments preceding the current
+        position.
         """
         for segment in raw_stack[::-1]:
             if segment.is_code or segment.is_type("newline"):
                 return segment
         return None
 
-    def _eval(self, segment, raw_stack, memory, **kwargs):
+    @staticmethod
+    def _get_following_seg(
+        raw_stack: Tuple[RawSegment, ...], segment: RawSegment
+    ) -> RawSegment:
+        """Given a segment in raw_stack, return the segment following."""
+        idx = raw_stack.index(segment)
+        if idx < len(raw_stack):
+            return raw_stack[idx + 1]
+        raise ValueError("No following segment available")  # pragma: no cover
+
+    def _eval(self, context: RuleContext) -> Optional[LintResult]:
         """Enforce comma placement.
 
         For leading commas we're looking for trailing commas, so
@@ -89,7 +100,10 @@ class Rule_L019(BaseRule):
         leading comma to a trailing comma. We add whitespace after the leading
         comma when converting a trailing comma to a leading comma.
         """
-        if not memory:
+        # Config type hints
+        self.comma_style: str
+
+        if not context.memory:
             memory: Dict[str, Any] = {
                 # Trailing comma keys
                 #
@@ -113,15 +127,17 @@ class Rule_L019(BaseRule):
                 # The trailing comma violation segment to be removed during fixing
                 "last_trailing_comma_segment": None,
             }
+        else:
+            memory = context.memory
 
         if self.comma_style == "trailing":
             # A comma preceded by a new line == a leading comma
-            if segment.is_type("comma"):
-                last_seg = self._last_code_seg(raw_stack)
-                if last_seg.is_type("newline"):
+            if context.segment.is_type("comma"):
+                last_seg = self._last_code_seg(context.raw_stack)
+                if last_seg and last_seg.is_type("newline"):
                     # Recorded where the fix should be applied
-                    memory["last_leading_comma_seg"] = segment
-                    last_comment_seg = self._last_comment_seg(raw_stack)
+                    memory["last_leading_comma_seg"] = context.segment
+                    last_comment_seg = self._last_comment_seg(context.raw_stack)
                     inline_comment = (
                         last_comment_seg.pos_marker.line_no
                         == last_seg.pos_marker.line_no
@@ -141,35 +157,48 @@ class Rule_L019(BaseRule):
             if memory["insert_trailing_comma"]:
                 # Search for trailing whitespace to delete after the leading
                 # comma violation
-                if segment.is_type("whitespace"):
-                    memory["whitespace_deletions"] += [segment]
+                if context.segment.is_type("whitespace"):
+                    memory["whitespace_deletions"] += [context.segment]
                     return LintResult(memory=memory)
                 else:
                     # We've run out of whitespace to delete, time to fix
                     last_leading_comma_seg = memory["last_leading_comma_seg"]
+                    # Scan backwards to find the last code segment, skipping
+                    # over lines that are either entirely blank or just a
+                    # comment. We want to place the comma immediately after it.
+                    last_code_seg = None
+                    while last_code_seg is None or last_code_seg.is_type("newline"):
+                        last_code_seg = self._last_code_seg(
+                            context.raw_stack[
+                                : context.raw_stack.index(
+                                    last_code_seg
+                                    if last_code_seg
+                                    else memory["last_leading_comma_seg"]
+                                )
+                            ]
+                        )
                     return LintResult(
                         anchor=last_leading_comma_seg,
                         description="Found leading comma. Expected only trailing.",
                         fixes=[
-                            LintFix("delete", last_leading_comma_seg),
+                            LintFix.delete(last_leading_comma_seg),
                             *[
-                                LintFix("delete", d)
+                                LintFix.delete(d)
                                 for d in memory["whitespace_deletions"]
                             ],
-                            LintFix(
-                                "create",
-                                anchor=memory["anchor_for_new_trailing_comma_seg"],
-                                # Reuse the previous leading comma violation to
-                                # create a new trailing comma
-                                edit=last_leading_comma_seg,
+                            LintFix.create_before(
+                                anchor_segment=self._get_following_seg(
+                                    context.raw_stack, last_code_seg
+                                ),
+                                edit_segments=[last_leading_comma_seg],
                             ),
                         ],
                     )
 
         elif self.comma_style == "leading":
             # A new line preceded by a comma == a trailing comma
-            if segment.is_type("newline"):
-                last_seg = self._last_code_seg(raw_stack)
+            if context.segment.is_type("newline"):
+                last_seg = self._last_code_seg(context.raw_stack)
                 # no code precedes the current position: no issue
                 if last_seg is None:
                     return None
@@ -182,7 +211,7 @@ class Rule_L019(BaseRule):
             # Have we found a trailing comma violation?
             if memory["insert_leading_comma"]:
                 # Only insert the comma here if this isn't a comment/whitespace segment
-                if segment.is_code:
+                if context.segment.is_code:
                     last_comma_seg = memory["last_trailing_comma_segment"]
                     # Create whitespace to insert after the new leading comma
                     new_whitespace_seg = WhitespaceSegment()
@@ -190,11 +219,10 @@ class Rule_L019(BaseRule):
                         anchor=last_comma_seg,
                         description="Found trailing comma. Expected only leading.",
                         fixes=[
-                            LintFix("delete", anchor=last_comma_seg),
-                            LintFix(
-                                "create",
-                                anchor=segment,
-                                edit=[last_comma_seg, new_whitespace_seg],
+                            LintFix.delete(last_comma_seg),
+                            LintFix.create_before(
+                                anchor_segment=context.segment,
+                                edit_segments=[last_comma_seg, new_whitespace_seg],
                             ),
                         ],
                     )
