@@ -7,27 +7,27 @@ from sqlfluff.rules.L023 import Rule_L023
 
 @document_fix_compatible
 class Rule_L024(Rule_L023):
-    """Single whitespace expected after USING in JOIN clause.
+    """Single whitespace expected after ``USING`` in ``JOIN`` clause.
 
-    | **Anti-pattern**
+    **Anti-pattern**
 
-    .. code-block::
+    .. code-block:: sql
 
         SELECT b
         FROM foo
         LEFT JOIN zoo USING(a)
 
-    | **Best practice**
-    | The • character represents a space.
-    | Add a space after USING, to avoid confusing it
-    | for a function.
+    **Best practice**
 
-    .. code-block::
+    Add a space after ``USING``, to avoid confusing it
+    for a function.
+
+    .. code-block:: sql
+       :force:
 
         SELECT b
         FROM foo
-        LEFT JOIN zoo USING•(a)
-
+        LEFT JOIN zoo USING (a)
     """
 
     expected_mother_segment_type = "join_clause"
