@@ -1,17 +1,19 @@
 """Implementation of Rule L027."""
 
 from sqlfluff.core.rules.base import LintResult
-from sqlfluff.rules.L025 import Rule_L025
+from sqlfluff.rules.L020 import Rule_L020
 
 
-class Rule_L027(Rule_L025):
+class Rule_L027(Rule_L020):
     """References should be qualified if select has more than one referenced table/view.
 
-    NB: Except if they're present in a USING clause.
+    .. note::
+       Except if they're present in a ``USING`` clause.
 
-    | **Anti-pattern**
-    | In this example, the reference 'vee' has not been declared
-    | and the variables 'a' and 'b' are potentially ambiguous.
+    **Anti-pattern**
+
+    In this example, the reference ``vee`` has not been declared,
+    and the variables ``a`` and ``b`` are potentially ambiguous.
 
     .. code-block:: sql
 
@@ -19,8 +21,9 @@ class Rule_L027(Rule_L025):
         FROM foo
         LEFT JOIN vee ON vee.a = foo.a
 
-    | **Best practice**
-    |  Add the references.
+    **Best practice**
+
+    Add the references.
 
     .. code-block:: sql
 
@@ -32,7 +35,7 @@ class Rule_L027(Rule_L025):
     def _lint_references_and_aliases(
         self,
         table_aliases,
-        value_table_function_aliases,
+        standalone_aliases,
         references,
         col_aliases,
         using_cols,
@@ -46,9 +49,20 @@ class Rule_L027(Rule_L025):
         # Check all the references that we have.
         for r in references:
             this_ref_type = r.qualification()
+            # Discard column aliases that
+            # refer to the current column reference.
+            col_alias_names = [
+                c.alias_identifier_name
+                for c in col_aliases
+                if r not in c.column_reference_segments
+            ]
             if (
                 this_ref_type == "unqualified"
-                and r.raw not in col_aliases
+                # Allow unqualified columns that
+                # are actually aliases defined
+                # in a different select clause element.
+                and r.raw not in col_alias_names
+                # Allow columns defined in a USING expression.
                 and r.raw not in using_cols
             ):
                 violation_buff.append(
