@@ -1,16 +1,22 @@
 """Implementation of Rule L028."""
 
-from sqlfluff.core.rules.base import LintResult
+from sqlfluff.core.rules.base import LintResult, EvalResultType, RuleContext
 from sqlfluff.core.rules.doc_decorators import document_configuration
-from sqlfluff.rules.L025 import Rule_L025
+from sqlfluff.rules.L020 import Rule_L020
 
 
 @document_configuration
-class Rule_L028(Rule_L025):
+class Rule_L028(Rule_L020):
     """References should be consistent in statements with a single table.
 
-    | **Anti-pattern**
-    | In this example, only the field `b` is referenced.
+    .. note::
+       This rule is disabled by default for BigQuery due to its use of
+       structs which trigger false positives. It can be enabled with the
+       ``force_enable = True`` flag.
+
+    **Anti-pattern**
+
+    In this example, only the field ``b`` is referenced.
 
     .. code-block:: sql
 
@@ -19,8 +25,9 @@ class Rule_L028(Rule_L025):
             foo.b
         FROM foo
 
-    | **Best practice**
-    |  Remove all the reference or reference all the fields.
+    **Best practice**
+
+    Add or remove references to all fields.
 
     .. code-block:: sql
 
@@ -38,12 +45,13 @@ class Rule_L028(Rule_L025):
 
     """
 
-    config_keywords = ["single_table_references"]
+    config_keywords = ["single_table_references", "force_enable"]
+    _allow_select_alias = False
 
     def _lint_references_and_aliases(
         self,
         table_aliases,
-        value_table_function_aliases,
+        standalone_aliases,
         references,
         col_aliases,
         using_cols,
@@ -53,13 +61,13 @@ class Rule_L028(Rule_L025):
         # How many aliases are there? If more than one then abort.
         if len(table_aliases) > 1:
             return None
-        standalone_aliases = [t[0] for t in value_table_function_aliases]
         # A buffer to keep any violations.
         violation_buff = []
         # Check all the references that we have.
         seen_ref_types = set()
         for ref in references:
-            # We skip any unqualified wildcard references (i.e. *). They shouldn't count.
+            # We skip any unqualified wildcard references (i.e. *). They shouldn't
+            # count.
             if not ref.is_qualified() and ref.is_type("wildcard_identifier"):
                 continue
             # Oddball case: Column aliases provided via function calls in by
@@ -69,6 +77,11 @@ class Rule_L028(Rule_L025):
             # entirely rather than trying to enforce anything.
             if ref.raw in standalone_aliases:
                 continue
+
+            # Certain dialects allow use of SELECT alias in WHERE clauses
+            col_alias_names = [c.alias_identifier_name for c in col_aliases]
+            if self._allow_select_alias and ref.raw in col_alias_names:
+                continue
             this_ref_type = ref.qualification()
             if self.single_table_references == "consistent":
                 if seen_ref_types and this_ref_type not in seen_ref_types:
@@ -84,11 +97,29 @@ class Rule_L028(Rule_L025):
                 violation_buff.append(
                     LintResult(
                         anchor=ref,
-                        description="{} reference {!r} found in single table select.".format(
-                            this_ref_type.capitalize(), ref.raw
-                        ),
+                        description="{} reference {!r} found in single table "
+                        "select.".format(this_ref_type.capitalize(), ref.raw),
                     )
                 )
             seen_ref_types.add(this_ref_type)
 
         return violation_buff or None
+
+    def _eval(self, context: RuleContext) -> EvalResultType:
+        """Override base class for dialects that use structs, or SELECT aliases."""
+        # Config type hints
+        self.force_enable: bool
+
+        # Some dialects use structs (e.g. column.field) which look like
+        # table references and so incorrectly trigger this rule.
+        if (
+            context.dialect.name in ["bigquery", "hive", "redshift"]
+            and not self.force_enable
+        ):
+            return LintResult()
+
+        # Certain dialects allow use of SELECT alias in WHERE clauses
+        if context.dialect.name in ["snowflake", "redshift"]:
+            self._allow_select_alias = True
+
+        return super()._eval(context=context)
