@@ -1,13 +1,13 @@
 """Testing utils for rule plugins."""
 from sqlfluff.core import Linter
-from sqlfluff.core.errors import SQLParseError
+from sqlfluff.core.errors import SQLParseError, SQLTemplaterError
 from sqlfluff.core.rules import get_ruleset
 from sqlfluff.core.config import FluffConfig
 from typing import Tuple, List, NamedTuple, Optional
 from glob import glob
 
 import pytest
-import oyaml as yaml
+import yaml
 
 
 class RuleTestCase(NamedTuple):
@@ -37,6 +37,11 @@ def load_test_cases(
         y = yaml.safe_load(raw)
 
         rule = y.pop("rule")
+        global_config = y.pop("configs", None)
+        if global_config:
+            for i in y:
+                if not ("configs" in y[i].keys()):
+                    y[i].update({"configs": global_config})
         ids.extend([rule + "_" + t for t in y])
         test_cases.extend([RuleTestCase(rule=rule, **v) for k, v in y.items()])
 
@@ -61,8 +66,10 @@ def assert_rule_fail_in_sql(code, sql, configs=None, line_numbers=None):
     print(f"Errors Found: {lerrs}")
     for e in lerrs:
         if e.desc().startswith("Unexpected exception"):
-            pytest.fail(f"Linter failed with {e.desc()}")
-    parse_errors = list(filter(lambda v: type(v) == SQLParseError, lerrs))
+            pytest.fail(f"Linter failed with {e.desc()}")  # pragma: no cover
+    parse_errors = list(
+        filter(lambda v: isinstance(v, (SQLParseError, SQLTemplaterError)), lerrs)
+    )
     if parse_errors:
         pytest.fail(f"Found the following parse errors in test case: {parse_errors}")
     if not any(v.rule.code == code for v in lerrs):
@@ -72,14 +79,14 @@ def assert_rule_fail_in_sql(code, sql, configs=None, line_numbers=None):
         )
     if line_numbers:
         actual_line_numbers = [e.line_no for e in lerrs]
-        if line_numbers != actual_line_numbers:
+        if line_numbers != actual_line_numbers:  # pragma: no cover
             pytest.fail(
                 "Expected errors on lines {}, but got errors on lines {}".format(
                     line_numbers, actual_line_numbers
                 )
             )
-    # The query should already have been fixed if possible so just return the raw.
-    return linted.tree.raw
+    fixed, _ = linted.fix_string()
+    return fixed
 
 
 def assert_rule_pass_in_sql(code, sql, configs=None):
@@ -87,11 +94,15 @@ def assert_rule_pass_in_sql(code, sql, configs=None):
     # Configs allows overrides if we want to use them.
     cfg = FluffConfig(configs=configs)
     r = get_rule_from_set(code, config=cfg)
-    parsed = Linter(config=cfg).parse_string(sql)
+    linter = Linter(config=cfg)
+    rendered = linter.render_string(sql, fname="<STR>", config=cfg, encoding="utf-8")
+    parsed = linter.parse_rendered(rendered, recurse=True)
     if parsed.violations:
         pytest.fail(parsed.violations[0].desc() + "\n" + parsed.tree.stringify())
     print(f"Parsed:\n {parsed.tree.stringify()}")
-    lerrs, _, _, _ = r.crawl(parsed.tree, dialect=cfg.get("dialect_obj"))
+    lerrs, _, _, _ = r.crawl(
+        parsed.tree, [], dialect=cfg.get("dialect_obj"), templated_file=rendered[0]
+    )
     print(f"Errors Found: {lerrs}")
     if any(v.rule.code == code for v in lerrs):
         pytest.fail(f"Found {code} failures in query which should pass.", pytrace=False)
